// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

if (!Cc) const Cc = Components.classes;
if (!Ci) const Ci = Components.interfaces;

const CREDIT_REDIR = 'http://www.flock.com/blogged-with-flock';

const RECENT_POSTS_NUMBER = 25;
var gParams;
var gB;
var gPostListUpdated = false;
var gPostPermalink = "";
var gWebService;
var gPublicationCancelled = false;

var gErrorCount = 0;
var gTimeout;

var gBlogService = Components.classes['@flock.com/flock-blog;1']
      .getService(Components.interfaces['flockIBlogService']);

var faves_coop = Components.classes["@flock.com/singleton;1"]
                           .getService(Components.interfaces.flockISingleton)
                           .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                           .wrappedJSObject;

// nsISimpleEnumerator implementation
function simpleEnumerator (aArray) {
    aArray.hasMoreElements = function () { 
        return this.length != 0; 
    }
    aArray.getNext = function () { 
        return this.shift (); 
    }
    return aArray;
}

// nsIStringEnumerator implementation
function stringEnumerator (aArray) {
  aArray.hasMore = function () { 
    return this.length != 0; 
  }
  aArray.getNext = function () { 
    var rval = this.shift (); 
    return rval;
  }
  return aArray;
}

function createTags(aString) {
  // ''.split(',') returns [''], but we want []
  var tags = aString.replace(' ', '');
  if (tags == '') {
    return stringEnumerator([]);
  } else {
    return stringEnumerator(tags.split(','));
  }
}

function categoryFactory(aId, aLabel) {
  this.id = aId;
  this.label = aLabel;
}

categoryFactory.prototype = {
  getInterfaces: function (count) {
    var interfaceList = [Components.interfaces.flockIBlogCategory, Components.interfaces.nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
  },
  QueryInterface: function (iid) {
    if (!iid.equals(Components.interfaces.flockIBlogCategory))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  },
  getHelperForLanguage: function (count) {return null;}
}


function setupBlog(aId) {
  gB = faves_coop.get(aId);
  try {
    gWebService = Cc[gB.getParent().serviceId].getService(Ci.flockIBlogWebService);
  } catch(e) {
    dump("*** ERROR: Couldn't create the component: "+gB.getParent().serviceId+"\n");
  }
  
  // Enable categories if supported
  switch (gWebService.supportsCategories) {
    case 0:
      $('flock-blog-categories-container').setAttribute('collapsed', true);
      break;
    case 1:
      $('flock-blog-categories-container').setAttribute('collapsed', false);
      $('flock-blog-categories').setAttribute('seltype', 'single');
      break;
    case 2:
      $('flock-blog-categories-container').setAttribute('collapsed', false);
      $('flock-blog-categories').setAttribute('seltype', 'multiple');
      break;
  }
  if (gWebService.metadataOverlay && gWebService.metadataOverlay != "") {
    $('extra-attributes').setAttribute('src', gWebService.metadataOverlay);
    $('extra-attributes').setAttribute('collapsed', 'false');
    // set the right size
  } else {
    $('extra-attributes').setAttribute('collapsed', 'true');
  }

  // Enable post replace if supported
  $('flock-blog-newreplace').setAttribute('collapsed', !gWebService.supportsPostReplace);


  flock_blogEditorSetupCategories();
  if ((!refreshCategories.inProgress) && (gWebService.supportsCategories > 0))
    refreshCategories.exec();
  // Update the post list if needed
  if ($('radio-update').selected) 
    flock_blogUpdatePostList();

  window.sizeToContent();   
}


function flock_blogMetadataLoad() {
  gParams  = window.arguments[0];
  
  document.documentElement.getButton('cancel').parentNode.setAttribute('hidden', 'true');
  // TODO: switch to the default blog (not the first one)
  populatesBlogList();
  var accts = $("blog-account-selector");
  if(!accts.selectedItem)
    accts.selectedIndex = 0;

  setupBlog(accts.selectedItem.id);
  $("accept-button").focus();
}


function populatesBlogList() {
  var blogList = $("flock-blog-accounts-popup");
  var accounts = faves_coop.get('http://flock.com/rdf#AccountsRoot');

  while (blogList.firstChild)
    blogList.removeChild(blogList.firstChild);  
  var enum_ = accounts.children.enumerate();
  while (enum_.hasMoreElements()) {
    var account = enum_.getNext();
    if (!account) continue; // getNext() can return NULL when hasMoreElements() is TRUE.
    var enum2 = account.children.enumerate();
    while (enum2.hasMoreElements()) {
      var child = enum2.getNext();
      if (child.isInstanceOf(faves_coop.Blog)) {
        var item = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
        item.setAttribute("class", "menuitem-iconic");
        item.setAttribute("id", child.id());
        item.setAttribute("label", child.title);
        item.setAttribute("src", account.favicon);
        item.setAttribute("oncommand", "setupBlog('"+child.id().replace(/'/g, "\\'")+"');");
        blogList.appendChild(item);
      }
    }
  }

  var accts = $("blog-account-selector");
  var defaultBlogId = gBlogService.getDefaultBlog();
  if (defaultBlogId == "") {
    accts.selectedIndex = 0;
    defaultBlogId = accts.selectedItem.id;
    gBlogService.setDefaultBlog(defaultBlogId);
  }
  accts.selectedItem = $(defaultBlogId);
}


function flock_blogEditorSetupCategories() {
  var catList = $('flock-blog-categories');

  var enum_ = gB.categories.enumerate();
  while (catList.getRowCount() > 0)
    catList.removeItemAt(0);
  while (enum_.hasMoreElements()) {
    var category = enum_.getNext();
    catList.appendItem(category.name, category.categoryId);
  }
}

function promptUsernamePassword(aBlogId) {
  var strbundle = document.getElementById("blogStringBundle");
  var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"].
          getService(Ci.nsIPromptService);

  var account = gBlogService.getAccount(aBlogId);
  var coopAccount = faves_coop.get(aBlogId).getParent();

  var username = { value: account.username };
  var password = { value: account.password };

  var title = strbundle.getString("flock.blog.metadata.error.auth.title");
  var msg = strbundle.getString("flock.blog.metadata.error.auth");
  var result = prompts.promptUsernameAndPassword(window,
                                                 title,
                                                 msg,
                                                 username,
                                                 password,
                                                 null,
                                                 {
                                                   value: false
                                                 });

  if (username.value != account.username) {
    coopAccount.accountId = username.value;
  }
  if (password.value != account.password) {
    var acUtils = Cc["@flock.com/account-utils;1"].
      getService(Ci.flockIAccountUtils);
    acUtils.setPassword(coopAccount.id(), username.value, password.value);
    this.refreshCategories.exec();
  }

  return result;
}

var refreshCategories = {
  exec: function() {
    this.inProgress = true;
    $("categories-throbber").setAttribute("hidden", "false");
    gWebService.getCategoryList(this, gB.id());
  },
  // Implement refresh listener
  onFault: function RC_onFault(aFault) {
    this.inProgress = false;
    $("categories-throbber").setAttribute("hidden", "true");
  },
  onResult: function RC_onResult(aResult) {
    var svc = Components.classes['@flock.com/flock-blog;1'].getService(Components.interfaces.flockIBlogService);
    while (aResult.hasMoreElements()) {
      category = aResult.getNext().QueryInterface(Components.interfaces.flockIBlogCategory);
      svc.addCategory(gB.id(), category.id, category.label);
    }
    flock_blogEditorSetupCategories();
    this.inProgress = false;
    $("categories-throbber").setAttribute("hidden", "true");
  },
  onError: function RC_onError(aError) {
    this.inProgress = false;
    $("categories-throbber").setAttribute("hidden", "true");

    var strbundle = document.getElementById("blogStringBundle");
    if (aError.errorCode == Ci.flockIError.BLOG_INVALID_AUTH) {
      if (promptUsernamePassword(gB.id())) {
        this.exec();
      }
    } else {
      var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"]
        .getService(Ci.nsIPromptService);
      var title = strbundle.getString("flock.blog.metadata.error");
      var msg = strbundle.getString("flock.blog.metadata.error.refreshingCategories")
                    + "\n" + aError.errorString;
      prompts.alert(window, title, msg);
    }
  },

  inProgress: false
}
 
function onRadioCheck(value){
  if (value == 1) {
    if (!gPostListUpdated)
      flock_blogUpdatePostList();
    $('blog-post-selector').setAttribute('disabled', 'false');
  } else {
    $('blog-post-selector').setAttribute('disabled', 'true');
  }
}

function flock_blogUpdatePostList(){
  gPostListUpdated = true;

  var strbundle = document.getElementById("blogStringBundle");
  // var selectedBlog = "urn:flock:blog:settings:" + $("blog-account-selector").selectedItem.value;
  var listener = new populatePostList(gB.id());
  $('blog-post-selector').removeAllItems();
  $('blog-post-selector').setAttribute('label', strbundle.getString("flock.blog.metadata.loadingPostList"));
  $('blog-post-selector').setAttribute('src', 'chrome://global/skin/throbber/Throbber-small.gif');

  gWebService.getRecentPosts(listener, gB.id(), RECENT_POSTS_NUMBER);
}

function populatePostList(selectedBlog, force_update) {
  this.selectedBlog = selectedBlog;
}

populatePostList.prototype = {
  onResult: function PPL_onResult(result) {
    var strbundle = document.getElementById("blogStringBundle");
    $('blog-post-selector').setAttribute('label', strbundle.getString("flock.blog.metadata.selectPost"));
    $('blog-post-selector').setAttribute('src', '');
    var posts = $('blog-post-selector');
    while (result.hasMoreElements()) {
      var post = result.getNext().QueryInterface(Ci.flockIBlogPost);
      var item = posts.appendItem(post.title, post.postid);
      item.post = post;
    }
  },

  onFault: function PPL_onFault(aError) {
    gPostListUpdated = false;
    tryAgain = false;
    var strbundle = document.getElementById("blogStringBundle");
    if (aError.errorCode == Ci.flockIError.BLOG_INVALID_AUTH) {
      if (promptUsernamePassword(gB.id())) {
        flock_blogUpdatePostList();
        tryAgain = true;
      }
    } else {
      var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"]
        .getService(Ci.nsIPromptService);

      var title = strbundle.getString("flock.blog.metadata.error");
      var msg = strbundle.getString("flock.blog.metadata.error.retrievingPost");
      msg += "\n\n";
      msg += "[" + aError.serviceErrorCode + "] " + aError.serviceErrorString;

      prompts.alert(window, title, msg);
    }
    if (!tryAgain) {
      $("blog-post-selector").setAttribute("src", "");
      $("blog-post-selector").setAttribute("disabled", "true");
      $("blog-post-selector").setAttribute("label",
                                           strbundle.getString("flock.blog.metadata.unavailable"));
      $("newreplace-radiogroup").selectedIndex = 0;
    }
  },
  onError: function PPL_onError(aError) {
    gPostListUpdated = false;
    var strbundle = document.getElementById("blogStringBundle");
    var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"]
      .getService(Ci.nsIPromptService);

    var title = strbundle.getString("flock.blog.metadata.error");
    var msg = strbundle.getString("flock.blog.metadata.error.retrievingPost");
    prompts.alert(window, title, msg);
  }
}

function blogCloseMetadata() {
  window.opener.published = true;
  window.opener.blogName = gB.title;
  if ($('view-on-your-blog').getAttribute('checked')){
    window.opener.permalink = gB.URL;
  }
  $('cancel-button').click();
}

function categorySupport() {
  if(!gSelectedBlog) return false;
  if(!gSelectedBlogAccountAPI) return false;
  if(gSelectedBlog.match(FLOCK_BLOG_DRAFTS_FOLDER)) return false;
  return !this.inProgress;
}


function getExtraParameters() {
  var result = new Array();

  var extra = $("extra-attributes").contentDocument.getElementById('extraData');
  if (!extra)
    return result;
  var rowsTag = extra.getElementsByTagName('rows')[0];
  var rows = rowsTag.getElementsByTagName('row');
  for (i=0; i<rows.length; i++) {
    result.push(rows[i].getAttribute('value'));
  }
  return stringEnumerator(result);
}


function metadataPublishPost() {
  var strbundle = document.getElementById("blogStringBundle");
  var creditEntity = strbundle.getString("flock.blog.metadata.credit");
  var creditProduct = strbundle.getString("flock.blog.metadata.credit.product");
  var creditLink = <a href={CREDIT_REDIR}
                      style="color: #999; font-weight: bold;"
                      target="_new"
                      title="Flock Browser">{creditProduct}</a>;
  var creditBlurb = creditEntity.replace("%1", creditLink.toXMLString());
  var flockCredit = '<div class="flockcredit" '
                  + 'style="text-align: right; color: #CCC; font-size: x-small;">'
                  + creditBlurb
                  + '</div>';
  var newPost = !($('radio-update').selected);
  var html = gParams.html;
  if ($('creditFlock').checked)
    html += flockCredit;

  var strbundle = document.getElementById("blogStringBundle");

  if (!newPost && !($('blog-post-selector').selectedItem)) {
    var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);
    var msg = strbundle.getString("flock.blog.metadata.selectToReplace");
    prompts.alert(window, strbundle.getString("flock.blog.metadata.missingInfo"), msg);
    return;
  }

  // Make sure the blog still exists
  if (!faves_coop.Blog.exists(gB.id())) {
    populatesBlogList();
    var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);
    var msg = strbundle.getString("flock.blog.metadata.selectOther");
    prompts.alert(window, strbundle.getString("flock.blog.metadata.missingAccount"), msg);
    return;
  }

  var post = {};
  post.getInterfaces = function (count) {
    var interfaceList = [Components.interfaces['flockIBlogPost'], nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
  };
  post.QueryInterface = function (iid) {
    if (!iid.equals(Components.interfaces.flockIBlogPost))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  };

  if (!newPost){
    post  = $('blog-post-selector').selectedItem.post;
  }

  // window.success = false;

  post.title = gParams.title;
  post.description = html;
  post.tags = createTags(gParams.tags);
  post.extra = getExtraParameters();

  var params = {
    service: gWebService,
    blog: gB,
    post: post,
    newPost: newPost
  };

  $("metadata-deck").selectedIndex = 1;

  Cc["@flock.com/metrics-service;1"]
    .getService(Ci.flockIMetricsService)
    .report("BlogDialog-Publish", params.service.shortName);
  
  blogTriggerPublish(params.service, params.blog, params.post, params.newPost);
}

function blogTriggerPublish(service, blog, post, newPost) {
  window.gB = blog;
  // Clean the HTML
  // Livejournal doesn't like <br />
  if (post.html)
    post.html = post.html.replace(/<br \/>/g, '<br/>');
  var notifications = gBlogService.getNotifications();

  post.categories = flock_blogEditorGetCategoryArray();

  if(newPost) {
    var listener = BlogPublishListener;
    service.newPost(listener, blog.id(), post, true, notifications);
  } else {
    var listener = BlogEditListener;
    service.editPost(listener, blog.id(), post, true, notifications);
  }
}

function flock_blogEditorGetCategoryArray() {
  var rval = new Array();
  var categories = $("flock-blog-categories").selectedItems;
  for(var i=0; i<categories.length; ++i) {
    var categoryItem = categories[i];
    var cat = new categoryFactory(categoryItem.getAttribute("value"), categoryItem.getAttribute("label"));
    rval.push(cat);
  }
  return simpleEnumerator(rval);
}

var BlogPublishListener = {
  failCount: 0,
  selectedPost: null,
  onResult: function(aResult) {
    if (gPublicationCancelled) {
      // TODO: delete the published post
    } else {
      gBlogService.moveTo(gParams.draftId, gB.id());
      blogCloseMetadata();
    }
  },

  onFault: function (fault) {
    if (!gErrorCount || (gErrorCount < 2)) {
      gErrorCount = gErrorCount + 1;
      publish_onload(); // Try again
    } else {
      var msg;
      window.clearTimeout(gTimeout);
      this.showErrorUI(fault.errorString);
    }
  },

  onError: function (error) {
    if (!gErrorCount || (gErrorCount < 2)) {
      gErrorCount = gErrorCount + 1;
      // Try again
      metadataPublishPost();
    } else {
      window.clearTimeout(gTimeout);
      this.showErrorUI(error.errorString);
    }
  },
    
  showErrorUI: function aMessage(aMsg) {
    try {
      $('metadata-deck').selectedIndex = 2;      
      $('publishing-error').firstChild.nodeValue = aMsg;
    } catch (ex) {
      dump (ex + '\n');
    }
  }
};

var BlogEditListener = {
  failCount: 0,
  selectedPost: null,
  onResult: function(result) {
    window.clearTimeout(gTimeout);
    gBlogService.moveTo(gParams.draftId, gB.id());
    if(result) {
      blogCloseMetadata();
    } else {
      $("metadata-deck").selectedIndex=0;
      blogCloseMetadata();
    }
  },

  onFault: function (fault) {
    if(this.failCount++<2) {
      publish_onload(); // Try again
    } else {
      var msg;
      window.clearTimeout(gTimeout);
      alert(fault.errorString);
    }
  },

  onError: function (errorMsg) {
    var strbundle = document.getElementById("blogStringBundle");
    var msg = strbundle.getString("flock.blog.publishing.error");
    window.clearTimeout(gTimeout);
    alert(msg);
  }
};

function timeOutError() {
  var strbundle = document.getElementById("blogStringBundle");
  msg = strbundle.getString("flock.blog.publishing.timeError")+"\n";
  alert(msg);
  $('cancel-button').click();
};

function publish_onload() {
  var params  = window.arguments[0];
  gTimeout = window.setTimeout("timeOutError()", 60000);
  blogTriggerPublish(params.service, params.blog, params.post, params.newPost);
}
