// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

Cc = Components.classes;
Ci = Components.interfaces;

var gService = null;

/*
 * Coop stuff
 */
var faves_coop = Cc["@flock.com/singleton;1"]
                 .getService(Ci.flockISingleton)
                 .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                 .wrappedJSObject;

function onLoad() {
  // Populates the service list
  serviceList = $("serviceList");

  // Do we need to select a given service first?
  var params = window.arguments[0];
  
  // Load the accounts for the first service
  var services = getFlockWebServices()
  for (var i = 0; i < services.length; i++) {
    var contractID = services[i][0];
    var svc = services[i][1];
    var accountitem = document.createElement("listitem");
    accountitem.setAttribute('label', svc.title);
    accountitem.setAttribute('value', contractID);
    accountitem.setAttribute('image', svc.icon);
    accountitem.setAttribute('class', 'listitem-iconic');
//    accountitem.setAttribute('onclick', 'loadService("'+contractID+'")');
    accountitem.svc = svc;
    serviceList.appendChild(accountitem);
  }

  if (params && params.serviceid)
    loadService(params.serviceid);

  updateButtons();
}


function getServiceConfirmation(aUrn) {
  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefService)
                .getBranch("flock.service.confirmation.");
  var value = null;
  try{
    value = prefs.getBoolPref (aUrn);
  } catch(e) {
    return true; // default value is to ask the user
  }
  return value;
}

function setServiceConfirmation(aUrn, aValue) {
  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefService)
                .getBranch("flock.service.confirmation.");
  prefs.setBoolPref (aUrn, aValue);
}


function loadService(aContractID) {
  gService = Cc[aContractID].getService(Ci.flockIWebService);
  var accountList = $('accountList');
  $("accounts-label").value = "Accounts on "+gService.title+":";

  // Clean the account list
  while (accountList.getRowCount() > 0) {
    accountList.removeItemAt(0);
  }
  
  // Populate the account list
  var enum = faves_coop.accounts_root.children.enumerate();
  while (enum.hasMoreElements()) {
    var account = enum.getNext();
    if (!account) continue; // getNext() can return NULL when hasMoreElements() is TRUE.
    if (account.serviceId == aContractID) {
      var accountitem = document.createElement("listitem");
      var label = account.name;
      if (account.service.logoutOption) {
        if (account.isAuthenticated) {
          label += " - logged in";
        } else {
          label += " - logged out";
        }
      } else {
        label += " - configured";
      }
      accountitem.setAttribute('label', label);
      accountitem.setAttribute('value', account.id());
      accountitem.setAttribute('style', 'font-style: italic;');
      accountList.appendChild(accountitem);
    }
  }

  // Select the first account or show the login instructions
  if (accountList.getRowCount() > 0) {
    $('accountDeck').selectedIndex = 0;
    accountList.selectedIndex = 0;
  }
  else {
    $('accountDeck').selectedIndex = 1;
    $('login-link').setAttribute('onclick', 'openUILinkIn("'+gService.url+'", "window");window.close();');
    $('login-link').textContent = gService.url;
  }

  // update the loginDetect button
  var loginDetect = getServiceConfirmation(gService.urn);
  $('dont-logindetect').setAttribute('checked', !loginDetect);
}


function updateService() {
  var cid = $('serviceList').selectedItem.value;

  loadService(cid);
  updateButtons();
}


function updateButtons() {
  var noselection = ($('accountList').selectedItem == null);
  $('logout-button').setAttribute('disabled', noselection);
  $('forget-button').setAttribute('disabled', noselection);

  if (!noselection) {
    var id = $('accountList').selectedItem.value;
    var account = faves_coop.get(id);
    $('logout-button').setAttribute('disabled', !account.service.logoutOption);
    try {
      // Account selected AND service implements flockISocialWebService
      gService.QueryInterface(Ci.flockISocialWebService);
      $('showInSidebar-check').setAttribute('disabled', 'false');
      $('showInSidebar-check').setAttribute('checked', account.showInSidebar);
    }
    catch(e) {
      // Account selected, but service doesn't implement flockISocialWebService
      $('showInSidebar-check').setAttribute('disabled', 'true');
      $('showInSidebar-check').setAttribute('checked', 'false');
    }
  }
  else {
    // No account selected
    $('showInSidebar-check').setAttribute('disabled', true);
    $('showInSidebar-check').setAttribute('checked', false);
  }
}


function forgetAccount() {
  var svc = $('serviceList').selectedItem.svc;
  var cid = $('serviceList').selectedItem.value;
  var id = $('accountList').selectedItem.value;

  var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
  if (prompts.confirm(window, "Forget Account", "This action will remove this account completely! Are you sure this is what you want to do?")) {
    svc.getAccount(id).logout(null);
    svc.removeAccount(id);
    loadService(cid);
  }
}


function logoutAccount() {
  var svc = $('serviceList').selectedItem.svc;
  var cid = $('serviceList').selectedItem.value;
  var id = $('accountList').selectedItem.value;

  var acct = svc.getAccount(id);
  acct.logout(null);
  loadService(cid);
}


function toggleShowInSidebar(aChecked) {
  var id = $('accountList').selectedItem.value;
  var account = faves_coop.get(id);
  account.showInSidebar = aChecked;
}


function toggleLoginDetection(aChecked) {
  var svc = $('serviceList').selectedItem.svc;
  setServiceConfirmation(svc.urn, !aChecked);
}


var getFlockWebServices = function() {
  var CATMAN = Cc['@mozilla.org/categorymanager;1'].getService(Ci.nsICategoryManager);
  var webServices = new Array();
  var e = CATMAN.enumerateCategory('flockWebService');
  while (e.hasMoreElements()) {
    try {
      var entry = e.getNext().QueryInterface(Ci.nsISupportsCString);
      if (!entry) { continue; }
      var contractID = CATMAN.getCategoryEntry('flockWebService', entry.data);
      var svc = Cc[contractID].getService(Ci.flockIWebService);
      webServices.push([contractID, svc]);
    } catch (e) { 
      dump(e);
    }
  }
  dump('Found '+webServices.length+' flock web services');
  return webServices;
}
