//
// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL
//

function doMigrations() {
  var args = [];
  if ("arguments" in window)
    args = window.arguments;

  var oldVersion = args.shift();
  var services = args;

  var serviceLabel  = document.getElementById("migration-info");
  var msgLabel      = document.getElementById("migration-message");
  var progressMeter = document.getElementById("migration-progress");

  for (var i = 0; i < services.length; i++) {
    var service = services[i];

    var basePercent = Math.round((i / services.length) * 100);

    serviceLabel.value = service.migrationName;
    msgLabel.value = "";
    progressMeter.value = basePercent;
    yield false;

    var progressWatcher = {
      divisor: services.length,
      msgLabel: msgLabel,
      progressMeter: progressMeter,
      basePercent: basePercent,
      onUpdate: function(percent, message) {
        dump("PROGRESS: " + percent + " " + message + "\n");
        this.msgLabel.value = message;
        if (percent >= 0) {
          var value = Math.round(this.basePercent + percent / this.divisor);
          this.progressMeter.value = value;
        }
      }
    };

    try {
      var classDesc = service.classDescription;
      dump("Migrate " + classDesc + "\n"); 
      var ctxt = service.startMigration(oldVersion, progressWatcher); 
      while (service.doMigrationWork(ctxt))
        yield false;
      dump("done with " + classDesc + "\n"); 
    }
    catch (e) {
      dump("Exception in migrating " + classDesc + " " + e + "\n");
    }
    finally {
      try {
        service.finishMigration(ctxt);
      }
      catch (e) { }
    }
  }
  
  // only do the following when migrating from 0.7.x (Cardinal)
  if (oldVersion.substr(0, 3) == "0.7") {
    // reset toolbars during migration as a lot has changed.
    resetToolbars();
  } else if ((oldVersion.substr(0, 3) == "0.9") ||
             (oldVersion.substr(0, 3) == "1.0"))
  {
    mungeToolbars(oldVersion.substr(0, 3));
  }

  // Append the myworld to the users home page if it is not already there
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefBranch);
  if (prefService) {
    if (prefService.prefHasUserValue("browser.startup.homepage") &&
        prefService.getPrefType("browser.startup.homepage") == prefService.PREF_STRING) 
    {
      var currHomePage = prefService.getCharPref("browser.startup.homepage");
      if (!currHomePage.match(/.*about:myworld.*/)) {
        currHomePage = "about:myworld|" + currHomePage;
        prefService.setCharPref("browser.startup.homepage", currHomePage);
      }
    }
  }

  serviceLabel.value = "Done";
  msgLabel.value = "";
  progressMeter.value = 100;
  yield true;
}

// delete localstore.rdf to reset toolbars.
function resetToolbars() {
  const nsIDirectoryServiceContractID = "@mozilla.org/file/directory_service;1";
  const nsIProperties = Components.interfaces.nsIProperties;
  var directoryService =  Components.classes[nsIDirectoryServiceContractID]
                                    .getService(nsIProperties);
  var localstoreFile = directoryService.get("LStoreS", Components.interfaces.nsIFile);
  if (localstoreFile.exists())
    localstoreFile.remove(false);
}

function mungeToolbars(aOldVer) {
  const RDFS = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                       .getService(Components.interfaces.nsIRDFService);
  var ds = RDFS.GetDataSource("rdf:local-store");
  var currentset = RDFS.GetResource("currentset");

  // Remove the feedback and help buttons
  var node = RDFS.GetResource("chrome://browser/content/browser.xul#nav-bar");
  var navbar = ds.GetTarget(node, currentset, true);
  if (navbar && navbar instanceof Components.interfaces.nsIRDFLiteral) {
    var currentNavBar = navbar.Value;
    var newNavBar = currentNavBar.replace("feedback-button", "")
                                 .replace("help-button", "")
                                 .replace(/,,+/g, ",")
                                 .replace(/,$/, "");
    if (currentNavBar != newNavBar) {
      ds.Change(node, currentset, navbar, RDFS.GetLiteral(newNavBar));
    }
  }

  // Add the People and Webmail buttons to the Flock Bar
  node = RDFS.GetResource("chrome://browser/content/browser.xul#PersonalToolbar");
  var flockBar = ds.GetTarget(node, currentset, true);
  if (flockBar && flockBar instanceof Components.interfaces.nsIRDFLiteral) {
    var currentFlockBar = flockBar.Value;
    var newFlockBar = currentFlockBar;
    if (currentFlockBar.length == 0) {
      if (aOldVer == "0.9") {
        newFlockBar = "people-button,webmail-button";
      } else {
        newFlockBar = "webmail-button";
      }
    } else {
      if (newFlockBar.indexOf("webmail-button") == -1) {
        if (newFlockBar.indexOf("news-button,separator") != -1) {
          newFlockBar = newFlockBar.replace("news-button,separator",
                                            "news-button,webmail-button,separator");
        } else if (newFlockBar.indexOf("news-button") != -1) {
          newFlockBar = newFlockBar.replace("news-button",
                                            "news-button,webmail-button,separator");
        } else {
          newFlockBar = "webmail-button," + newFlockBar;
        }
      }
      if (aOldVer == "0.9") {
        if (currentFlockBar.indexOf("people-button") == -1) {
          if (currentFlockBar.indexOf("my-world-button,separator") != -1) {
            newFlockBar = newFlockBar.replace("my-world-button,separator",
                                              "$&,people-button");
          } else if (newFlockBar.indexOf("my-world-button") != -1) {
            newFlockBar = newFlockBar.replace("my-world-button",
                                              "$&,separator,people-button");
          } else {
            newFlockBar = "people-button," + newFlockBar;
          }
        }
      }
    }

    if (currentFlockBar != newFlockBar) {
      ds.Change(node, currentset, flockBar, RDFS.GetLiteral(newFlockBar));
    }
  }

  // Make sure the Flock Bar is visible
  var collapsed = RDFS.GetResource("collapsed");
  var target = ds.GetTarget(node, collapsed, true);
  if (target) {
    ds.Unassert(node, collapsed, target);
  }
}

function migrate() {
  var migrationRunner = doMigrations();

  var run = {
    notify: function(aTimer) {
      if (migrationRunner.next()) {
        aTimer.cancel();
        window.close();
      }
    }
  };

  const nsITimer = Components.interfaces.nsITimer;
  var timer = Components.classes['@mozilla.org/timer;1']
    .createInstance(nsITimer);
  timer.initWithCallback(run, 0, nsITimer.TYPE_REPEATING_SLACK);
}
