// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const GET_ACCOUNT_URL = "http://flock.com/favorites-services";
const favoritesService = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-favorites']
                                   .getService(Components.interfaces.flockIFavoritesService);

var bmsvc = favoritesService.QueryInterface(Components.interfaces.nsIBookmarksService);
var flockServiceObserver = Components.classes["@mozilla.org/observer-service;1"]
                                     .getService(Components.interfaces.nsIObserverService);

var faves_coop = Components.classes['@flock.com/singleton;1']
                           .getService(Components.interfaces.flockISingleton)
                           .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                           .wrappedJSObject;
var bookmarksHelper = Components.classes['@flock.com/singleton;1']
                .getService(Components.interfaces.flockISingleton)
                .getSingleton('chrome://browser/content/bookmarks/flockBookmarksHelper.js')
                .wrappedJSObject;
var doEdited = false;
var onlineAccounts = null;

var Logger = Components.Constructor("@flock.com/logger;1", Components.interfaces.flockILogger, "init");

var gMetrics = Components.classes["@flock.com/metrics-service;1"]
               .getService(Components.interfaces.flockIMetricsService);

var progressListener = {
  _logger: new Logger("webprogress"),
  lastUrl: null,
  STATE_STOP: Components.interfaces.nsIWebProgressListener.STATE_STOP,
  STATE_IS_NETWORK: Components.interfaces.nsIWebProgressListener.STATE_IS_NETWORK,
  STATE_START: Components.interfaces.nsIWebProgressListener.STATE_START,
  mButton: null,
  currentUrl: null,
  
  onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) { },
  
  onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) { },

  onLocationChange: function(aWebProgress, aRequest, aLocation) {
    var url = aLocation.spec;
    // strip anchors
    url = url.replace(/#.*/, '');
    if (this.lastUrl != url) {
      this._newLocation(url);
      this.lastUrl = url;
    }
  },
  
  onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) { },

  onSecurityChange: function(aWebProgress, aRequest, aStatus) { },

  _updateButton: function() {
      starButton.disabled=false;
      updateState();
  },

  _newLocation: function(aUrl) {
    var nBox = gBrowser.getNotificationBox();
    nBox.clearDiscoveryMessages();
    this.currentUrl = aUrl;
    updateLastVisitedDate(aUrl); 
    this._updateButton();
  }
};


/*
 * Start update Favorite and Explorer button
 *
 */

var publishListener = {
  onError: function() {
    var message = flockGetString("favorites/favorites",
                                 "flock.favs.overlay.favorite-publication-error",
                                 [bookmark.URL, service[0]]);
    var nBox = gBrowser.getNotificationBox();
    nBox.appendUniqueNotification(message,
                                  "favorite-publication-error",
                                  "chrome://browser/skin/Info.png",
                                  nBox.FLOCK_PRIORITY_HIGH,
                                  null);
  },
  onSuccess: function() {
    updateState();
  }
}

function updateState() {
  var docshell= $$("content").webNavigation;
  updateStarButton(docshell);
  //updateExplorerButton(docshell);
  updateLastVisitedDate(docshell.currentURI.spec);
}

function updateStarStateForBookmark() {
  var currentUrl = $("content").mCurrentBrowser.contentWindow.location + "";
  var onlineAccounts = [];
  // Is it a local bookmark?
  var isFav = favoritesService.isBookmarked (currentUrl);
  // Is it an online bookmark?
  var isOnline = false;
  if (!isFav) {
    var accounts = faves_coop.get('http://flock.com/rdf#AccountsRoot');
    var _enum = accounts.children.enumerate();
    while (_enum.hasMoreElements()) {
      var account = _enum.getNext();
      if (!account) continue; // getNext() can return NULL when hasMoreElements() is TRUE.
      try {
        var svc = Cc[account.serviceId].getService(Ci.flockIWebService);
        svc.QueryInterface(Ci.flockIBookmarkWebService);
        // The account IS on a flockIBookmarkWebService
        if (svc.exists(account.accountId, currentUrl))
        {
          onlineAccounts.push(account.accountId);
          isOnline = true;
        }
      }
      catch(e) {
        // The account is NOT on a flockIBookmarkWebService
        // dump(e);
      }
    } 
  }

  var prefsService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  var star2colors = false;
  try {
    star2colors = prefsService.getBoolPref("flock.favorites.star2colors");
  } catch(e) {
    // the pref is not set
  }
  if (star2colors)
    isFav = isFav ? 'on' : (isOnline ? 'on2' : 'off');
  else
    isFav = (isFav || isOnline) ? 'on' : 'off';

  starButton.lightButton(isFav);
}

function updateFaveState(button, docshell) {
  var decorations = docshell.document._flock_decorations;
  var url = docshell.currentURI.spec;
  button.clear();
  var haveItems = false;
  if (progressListener.verifiedFeeds[url] && progressListener.verifiedFeeds[url].length >0) {
    var feedArray = new Array();
    for(var i = 0; i<progressListener.verifiedFeeds[url].length ; i++) {
      var feed = progressListener.verifiedFeeds[url][i]; 
      button.addNewsFeed(feed);   
    }
    haveItems = true;
  }
  if (decorations) {
    if (decorations.person) {
      var person = decorations.person;
      button.addPerson(person);
      haveItems = true;
      if (addPersonDetectedNotification) addPersonDetectedNotification();
  }
      
    if (decorations.mediaArr) {
      var mediaArr = docshell.document._flock_decorations.mediaArr;
      for (var i = 0; i < mediaArr.length; i++) { 
        var media = mediaArr[i];
        button.addMedia(media);
      }
      haveItems = true;
    }
  }
   
  button.updateState();
  return haveItems;
}

function openMediabarViaItems(service, query, label)
{
  if (mediabar.lock()) {
    gMetrics.report("MediaBar-Open", "Discovery");
    mediabar.loadQuery(service, query, label);
    mediabar.unlock();
    
    // Pop message bar for media detection discovery
    var favItem = faves_coop.MediaQuery.find({query: query, isPollable: true});
    if (favItem.length == 0) {
      var msg = flockGetString("favorites/favorites",
                               "flock.favs.media.discovery");
      var nBox = _getNotificationBox();
      nBox.appendUniqueNotification(msg,
                                    "media-discovery-info",
                                    "chrome://browser/skin/flock/favorites/media-icon.png",
                                    nBox.FLOCK_PRIORITY_LOW,
                                    null,
                                    null);
    }
  }
}

function _getNotificationBox() {
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
  var win = wm.getMostRecentWindow("navigator:browser");
  if (win) {
    var gBrowser = win.getBrowser();
    var notificationBox = gBrowser.getNotificationBox();
 
    return notificationBox;
  }

  return null;
}

function openMediabarViaDiscovery(aMenuitem)
{
  var service = aMenuitem.getAttribute("service");
  var query = aMenuitem.getAttribute("query");
  var label = aMenuitem.getAttribute("label");

  aMenuitem.parentNode.hidePopup();
  openMediabarViaItems(service, query, label);
}


function updateDiscoveredMediaPopup(aDocshell)
{
  var mediaBtn = $("favoritesPhotoButton");
  var mediaPopup = mediaBtn.firstChild;
  var rtval_firstquery = null;
  // Clear out the popup menu
  while (mediaPopup.firstChild) {
    mediaPopup.removeChild(mediaPopup.firstChild);
  }
  // Repopulate the menu based on document decorations
  var docshell = (aDocshell) ? aDocshell : $$("content").webNavigation;
  var decorations = docshell.document._flock_decorations;
  if (decorations && decorations.mediaArr) {
    for (var i = 0; i < decorations.mediaArr.length; i++) {
      var mStream = decorations.mediaArr[i];
      if (mStream) {
        var menuitem = document.createElement("menuitem");
        menuitem.setAttribute("service", mStream.service);
        menuitem.setAttribute("discovered", "false");
        menuitem.setAttribute("query", mStream.query);
        menuitem.setAttribute("label", mStream.label);
        menuitem.setAttribute("oncommand", "openMediabarViaDiscovery(this);");
        var urn = "urn:media:favorites:"+mStream.service+":"+mStream.query;
        if (faves_coop.get(urn)) {
          menuitem.setAttribute("class", "menuitem-iconic");
          menuitem.setAttribute("image",
                                "chrome://flock/skin/common/star16.png");
        }
        mediaPopup.appendChild(menuitem);
        if (!rtval_firstquery) { rtval_firstquery = mStream.query; }
      }
    }
  }
  // Return value will be NULL if no media was found, else it will be the
  // query string for the first discovered media stream
  return rtval_firstquery;
}

function flashForDiscover(count, name, element)
{
  if(count==10) return;
  
  var menuitems = element.getElementsByAttribute('discovered', 'false');
  var val = 'true';
  if(menuitems.length == 0)
  {
    menuitems = element.getElementsByAttribute('discovered', 'true');
    val = 'false';
  }
  for(var i=0; i<menuitems.length; i++)
  {
    menuitems[i].setAttribute('discovered', val);
    
  }
  count = count+ 1;
  var method =  "flashForDiscover( "+ count +", '" +name+"' ,$('"+name+"'));";
  setTimeout(method, 500);
}
function updateStarButton(docshell)
{
  // Check for discovered Media
  var mediaBtn = $("favoritesPhotoButton");
  var firstQuery = updateDiscoveredMediaPopup(docshell);
  var decorations = docshell.document._flock_decorations;
  if (firstQuery ) {
    // Media was discovered
    mediaBtn.removeAttribute("disabled");
    mediaBtn.setAttribute("tooltiptext", flockGetString( "favorites/favorites",
                                  "flock.favs.overlay.mediaBtn.new" ));
    showMediaMessageBar(decorations);
  }else {
    // No media was discovered
    mediaBtn.setAttribute("disabled", "true");
    mediaBtn.setAttribute("tooltiptext", flockGetString( "favorites/favorites",
                                  "flock.favs.overlay.mediaBtn.desc" ));
  }
  updateStarStateForBookmark(docshell);
}


function showMediaMessageBar(decorations)
{
  if (decorations && decorations.mediaArr) {
    for (var i = 0; i < decorations.mediaArr.length; i++) {
        var mStream = decorations.mediaArr[i];
        var url = mStream.service+"+"+mStream.query;
        if (mStream && mediabar.needDiscovery(gBrowser, mStream.service, mStream.query, url))
        {
          // Show message bar
          var message = flockGetString( "favorites/favorites",
                                    "flock.favs.overlay.discovery-media" );
          var nBox = gBrowser.getNotificationBox();
          var links = [{
            label: flockGetString("favorites/favorites",
                              "flock.favs.overlay.show-me-how"),
            callback: function() {
              var method = "flashForDiscover("+ 1 +",'favoritesPhotoButton',$('favoritesPhotoButton'));";  
              setTimeout(method, 500);
              $('favoritesPhotoButton').click();
           }
      }];
      nBox.appendUniqueNotification( message,
                                     "discovery-media",
                                     null,
                                     nBox.FLOCK_PRIORITY_LOW,
                                     null, links, url );
      break;  
    }
  }
}
}
function updateExplorerButton(docshell) {
  var explorerButton = $('exploreButton');
  var haveItems = updateFaveState(explorerButton, docshell);
  if (!haveItems) {
    $('exploreButton').disabled = true;
  }
  else {
    $('exploreButton').disabled = false;
  }   
}

function editCurrentBookmark(URL) {
  // if star is waiting we do not want to open dialog
  if (starButton.isStarWaiting()) {
    return;
  }

  if (!URL || typeof(URL) != "string") {
    URL = gBrowser.currentURI.spec;
  }

  var coopObj = faves_coop.Bookmark.get_from_url(URL);

  if (doEdited) {
    starButton.waitCursor('on');
    setTimeout(
      function() {
        openFavoritesEditDialog(coopObj);
        starButton.waitCursor('off');
      },
      3000);
  } else {
    openFavoritesEditDialog(coopObj);
  }
}

function openFavoritesEditDialog(aCoopObj) {
  var selection = { item: [ { Value: aCoopObj.id() } ] };
  var retval = {};

  var windowMediator = Components
                       .classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);

  // See if the dialog is not already open
  var oldWin = null;
  var allWindows = windowMediator.getEnumerator(null);
  while (allWindows.hasMoreElements()) {
    var win = allWindows.getNext();
    if (win.arguments && win.arguments.length > 0 &&
        win.arguments[0] &&
        win.arguments[0].item == selection.item)
    {
      oldWin = win;
      break;
    }
  }

  if (oldWin) {
    oldWin.focus();
  } else {
    // bookmarksProperties.xul takes a menuitem/treeitem instead of a URL
    // (yes, that sucks)

    openDialog("chrome://browser/content/bookmarks/bookmarksProperties.xul",
               "", "centerscreen,modal,resizable=no",
               selection, retval);
  }

  doEdited = retval.publishedOnline;
}

function starClicked(aTriggeringEvent)
{

#ifdef XP_MACOSX
 var accel = aTriggeringEvent.metaKey;
#else
 var accel = aTriggeringEvent.ctrlKey;
#endif
  if(accel)
  {
    var URL = gBrowser.currentURI.spec;
    if (favoritesService.isBookmarked (URL)) {
      editCurrentBookmark();
    }else
    {
      var name = gBrowser.contentTitle;
      var description = BookmarksUtils.getDescriptionFromDocument(gBrowser.contentDocument);
      var charset = gBrowser.contentDocument.characterSet;
      showAddBookmarksDialog(name, URL, description, null, charset);
    }
  }else
  {
    starCurrentPage();
  }
}

function starCurrentPage(dArgs) {
  var starStared = false;
  if (starButton.getStarState() == "off") {
    starStared = true;
  }
  gMetrics.report("Navigation-StarClick", starStared);

  var URL = null;
  if (dArgs && dArgs.url) {
    URL = dArgs.url;
  }else
  {
    URL = gBrowser.currentURI.spec;
  }
  if (gBrowser.currentURI.spec == URL &&
      starButton.getStarState() == "on2")
  {
    showOnlineBookmarkProperties(URL);
  }
  else if (favoritesService.isBookmarked (URL)) {
    if (dArgs) {
      editCurrentBookmark(URL);
    } else {
      editCurrentBookmark();
    }
  }
  else {
    var name = gBrowser.contentTitle;
    var description = BookmarksUtils
                      .getDescriptionFromDocument(gBrowser.contentDocument);
    var charset = gBrowser.contentDocument.characterSet;
    if (dArgs) {
      name = dArgs.name;
      description = dArgs.description;
    }
    // Does the user want to get the dialog?
    var prefsService = Components.classes["@mozilla.org/preferences-service;1"]
                                 .getService(Components.interfaces.nsIPrefBranch);
    var starShowProperties = prefsService
                             .getBoolPref("flock.favorites.starShowProperties");
    if (starShowProperties) {
      // Popup the new BM dialog
      showAddBookmarksDialog(name, URL, description, null, charset);
    }
    else {
      // Create the bookmark and inform the user
      if (prefsService.getBoolPref("flock.favorites.do.local"))
      {
        var folder = prefsService.getCharPref("flock.favorites.do.folder");
        if(folder != '')
        {
          BMDS.addBookmarkToFolder(folder, name, URL, "", description, charset, "", "");
        }else
        {
          BMDS.createBookmarkWithTags (name, URL, "", description, charset, "", "");
        }
      }
      var prefBranch = prefsService.getBranch("flock.favorites.do.");
      var list = prefBranch.getChildList("", {});
      var bookmark = new bookmarksHelper.Bookmark;
      bookmark.URL = URL;
      bookmark.name = name;
      bookmark.tags = "";
      bookmark.description = "";
      
      var publishList = [];
      for (var i =0; i<list.length; i++) {
        try {
          if (prefsService.getBoolPref("flock.favorites.do."+list[i])) {
            var isPrivate = false;
            try {
                isPrivate = prefsService.getBoolPref("flock.favorites.private." + list[i]);
            } catch (ex) {
            }
            var service = list[i].split("--");
            if(service.length == 2) {
              var svc = Cc[service[0]].getService(Ci.flockIBookmarkWebService);
              svc.publish(publishListener, service[1], bookmark, isPrivate);
              var found = false;
              for (var k = 0; k < publishList.length; k++) {
                if (publishList[k] == list[i]) {
                  found = true;
                  break;
                }
              }
              if (!found) {
                publishList.push(list[i]);
              }
              doEdited = true;
            }
          }
        }catch(e) {}
      }
      
      showStarMessage(prefsService.getBoolPref("flock.favorites.do.local"), publishList);
    }
  }
  updateStarStateForBookmark();
}

function showOnlineBookmarkProperties(url)
{
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Ci.flockIAccountUtils);
  var accounts = acUtils.getAccountsByInterface("flockIBookmarkWebService");
  var bookmark = null;
  while (accounts.hasMoreElements()) {
    var next_acc = accounts.getNext();
    var account = next_acc.QueryInterface(Ci.flockIWebServiceAccount);
    var c_acct = faves_coop.get(account.urn);
    var svc = Cc[c_acct.serviceId].getService(Ci.flockIWebService);
    try {
      svc.QueryInterface(Ci.flockIBookmarkWebService);
      bookmark = faves_coop.get(svc.urn+":"+c_acct.accountId+':'+url);
      if (bookmark) {
        // found a online bookmark for this URL
        break;
      }
    } catch(ex) {
    }
  }
  if (bookmark) {
    showAddBookmarksDialog(bookmark.name, url, bookmark.description,
                           bookmark.tags);
  }
}

function showAddBookmarksDialog(name, url, description, tags, charset, dArgs) {
  dump("****** showAddBookmarksDialog: " + url + "\n");

    var prefsSvc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  var defaultFolder = prefsSvc.getCharPref('flock.favorites.do.folder');
  var dArgs = { 
    name: name,
    url: url,
    bWebPanel: false,
    folderURI: defaultFolder,
    description: description,
    tags: tags,
    charset: charset,
    listener: publishListener
  };
    
  openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable=no,modal", dArgs)
  if ((favoritesService.isBookmarked (url)) || (dArgs.publishList.length > 0)) {
    showStarMessage(favoritesService.isBookmarked (url), dArgs.publishList);
  }
}

function showStarMessage(aLocal, aPublishList) {
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                           .getService(Components.interfaces.flockIAccountUtils);
   var prefsService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

   // Does the user really wants to see it?
   var starMessageBar = prefsService.getBoolPref("flock.favorites.starMessageBar");
   if (!starMessageBar)
     return;

   // are any online bookmark service accounts configured?
   var accountsEnum = acUtils.getAccountsByInterface("flockIBookmarkWebService");

   var message;
   var links = [{
     label: flockGetString("favorites/favorites",
                           "flock.favs.overlay.favorite-added.openSidebar"),
     callback: function() { toggleSidebarWithMetric("viewBookmarksSidebar",
                                                    "FavesSidebar-Open",
                                                    null,
                                                    "StarFavesAddDiscovery",
                                                    true); }
   }];


   if (accountsEnum.hasMoreElements()) {
     var shortnameStr = "";
     for (var k = 0; k < aPublishList.length; k++ ) {
       var temp = aPublishList[k].split("--");
       var bmSvc = Cc[temp[0]].getService(Ci.flockIWebService);
       shortnameStr = shortnameStr + "the " + temp[1] + " account on " 
                    + bmSvc.shortName + ", ";
     }

     if (shortnameStr) {
       if (aLocal) {
         // Local favorite and online
         shortnameStr = shortnameStr.substring(0, shortnameStr.length - 2);
         message = flockGetString("favorites/favorites",
                                  "flock.favs.overlay.favorite-added.local");
         message += "  ";
         message += flockGetString("favorites/favorites",
                                   "flock.favs.overlay.favorite-added.alsoOnline",
                                   [shortnameStr]);
       }
       else {
         // Online only: "This page has been added to " + shortnameStr + "."
         message = flockGetString("favorites/favorites",
                                  "flock.favs.overlay.favorite-added.online",
                                  [shortnameStr]);
       }
     }
     else {
       // Local favorite, online account is configured but we didn't publish online
       message = flockGetString("favorites/favorites",
                                "flock.favs.overlay.favorite-added.local");
       message += "  ";
       message += flockGetString("favorites/favorites",
                                 "flock.favs.overlay.favorite-added.notOnline");
     }
   }
   else {
     // Local favorite, no account is configured
     message = flockGetString("favorites/favorites",
                              "flock.favs.overlay.favorite-added.local");
     links.push({
       label: flockGetString("favorites/favorites",
                             "flock.favs.overlay.favorite-added.getAccount"),
       callback: function() { toggleSidebarWithMetric("flock_AccountsSidebarBroadcaster",
                                                      "AccountSidebar-Open",
                                                      null,
                                                      "StarFavesAddDiscovery",
                                                      true);
       var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
       setTimeout(function() { obs.notifyObservers(null, 'toggleAccountsArea', "bookmarks");},
             300); }
     });

   }
   message += " " + flockGetString("favorites/favorites",
                "flock.favs.overlay.favorite-added.clickToEdit");
  var nBox = gBrowser.getNotificationBox();
  var buttons = [{
    label: flockGetString("favorites/favorites",
                          "flock.favs.overlay.favorite-added.options"),
    accessKey: flockGetString("favorites/favorites",
                              "flock.favs.overlay.favorite-added.options.accesskey"),
    callback: starCurrentPage,
    forceopen: true
  }];
  nBox.appendUniqueNotification(message,
                                "favorite-added",
                                null,
                                nBox.FLOCK_PRIORITY_MEDIUM,
                                buttons,
                                links);
}

function turnOffDiscoveryMessage(aValue)
{
  var nBox = gBrowser.getNotificationBox();
  nBox.dontShowMessage(aValue);
}

function dontShowStarMessage()
{
  var prefsService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  prefsService.setBoolPref("flock.favorites.starMessageBar", false);
}

/*
 * End update Favorite and Explorer button
 *
 */

var favoritesObserverService = Components.classes["@mozilla.org/observer-service;1"]
                                         .getService( Components.interfaces.nsIObserverService );

function updateLastVisitedDate(aUrl)
{
  favoritesService.updateLastVisitedDate(aUrl, null)
}

var mediaObserver;
var flockDocumentReadyObserver;
var favoritesObserver;
var starButton;
var gTimer = null;

function showMessages(targetDoc)
{
   var browser = gBrowser.getBrowserForDocument(targetDoc);
   var nBox = gBrowser.getNotificationBox(browser);
   nBox.showDiscoveryMessages();
} 
function onLoad_favoritesOverlay() {

  starButton = $('favoritesButton');
    
  mediaObserver = { observe: function (aSubject, aTopic, aData) { updateState(); } };
  favoritesObserverService.addObserver(mediaObserver, 'media', false);

  flockDocumentReadyObserver = { observe: function (aSubject, aTopic, aData) { updateState(); setTimeout(showMessages, 500, aSubject)} };
  favoritesObserverService.addObserver(flockDocumentReadyObserver, 'FlockDocumentReady', false);

  favoritesObserver = {
    onAssert: function (dataSource, source, property, target) {
      if (property.Value == "http://home.netscape.com/NC-rdf#URL") {
        target.QueryInterface(Components.interfaces.nsIRDFLiteral);
        if (target.Value == gBrowser.currentURI.spec) {
          updateState();
        }
      }
    },
    onUnassert: function (dataSource, source, property, target) {
      if (property.Value == "http://home.netscape.com/NC-rdf#URL" &&  
          source.Value.match(gBrowser.currentURI.spec)) 
       {
        if (gTimer)
          gTimer.cancel();
        else
          gTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        gTimer.initWithCallback({
          notify: function() { updateState(); }
        }, 0, Ci.nsITimer.TYPE_ONE_SHOT);
      }
    },
    onBeginUpdateBatch: function (dataSource) {
    },
    onChange: function (dataSource, source, property, oldTarget, newTarget) {
    },
    onEndUpdateBatch: function (dataSource) {
    },
    onMove: function (dataSource, oldSource, newSource, property, target) {
    }
  };
  favoritesService.AddObserver(favoritesObserver);

  starCurrentObserver = { observe: function (aSubject, aTopic, aData) {
	// STEVO:
	// When the user presses ctrl+d this observer gets fired, for each window open.
	
	// Here we need to check if we are the currently focused window since this
	// observe will be fired for each open window.
	// If we are not the currently focused(or front) window we need to abort.
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
	                   .getService(Components.interfaces.nsIWindowMediator);
	var enumerator = wm.getZOrderDOMWindowEnumerator("navigator:browser", true);
	if (enumerator.hasMoreElements()) {
	  var win = enumerator.getNext();
	  if (win != window) {
		  return;
	  }
	}
	
    var dArgs = null;
    if(aData)
    {
      aData = aData.split(';')
      if(aData.length == 5)
      {
        dArgs = {
          name: (aData[0] == 'undefined')?'':aData[0],
          url: (aData[1] == 'undefined')?'':aData[1],
          charset: (aData[2] == 'undefined')?'':aData[2],
          bWebPanel: (aData[3] == 'undefined')?'':aData[3],
          description: (aData[4] == 'undefined')?'':aData[4]
        }
      }
     
    }
   
    starCurrentPage(dArgs);
    
   } 
   };
  favoritesObserverService.addObserver(starCurrentObserver, 'starCurrent', false );
  // our general purpose web progress listener, apparently
  progressListener.mButton = starButton;

  window.getBrowser().addProgressListener(progressListener,
                       Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
}

function onUnload_favoritesOverlay() {
  if (favoritesObserver) favoritesService.RemoveObserver(favoritesObserver);
  if (mediaObserver) favoritesObserverService.removeObserver( mediaObserver, 'media' );
  if (starCurrentObserver) favoritesObserverService.removeObserver( starCurrentObserver, 'starCurrent' );
  if (flockDocumentReadyObserver) favoritesObserverService.removeObserver( flockDocumentReadyObserver, 'FlockDocumentReady');
  window.getBrowser().removeProgressListener(progressListener);
}

window.addEventListener("load", onLoad_favoritesOverlay, false);
window.addEventListener("unload", onUnload_favoritesOverlay, false);

// XXX erwan: temporary UI before we decide what the UE will be
function Bookmark() {
  this.URL = "";
  this.name = "";
  this.description = "";
  this.tags = "";
  this.time = "";
}

Bookmark.prototype = {
  getInterfaces: function (count) {
    var interfaceList = [Components.interfaces.flockIBookmark, Components.interfaces.nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
  },
  QueryInterface: function (iid) {
    if (!iid.equals(Components.interfaces.flockIBookmark))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

function showEmptyInOnlineFaves(aTarget) {
  var accounts = faves_coop.get('http://flock.com/rdf#AccountsRoot');
  var _enum = accounts.children.enumerate();
  var noaccount = true;
  while (_enum.hasMoreElements() && noaccount) {
    var account = _enum.getNext();
    if (!account) continue; // getNext() can return NULL when hasMoreElements() is TRUE.
    var svc = Cc[account.serviceId].getService(Ci.flockIWebService);
    try {
      svc.QueryInterface(Ci.flockIBookmarkWebService);
      noaccount = false;
    }
    catch (e) {}
  }
  if (noaccount) {
    var EmptyMsg = BookmarksUtils.getLocaleString("emptyFolder");
    var emptyElement = document.createElementNS(gXUL_NS, "menuitem");
    emptyElement.setAttribute("id", "empty-menuitem");
    emptyElement.setAttribute("label", EmptyMsg);
    emptyElement.setAttribute("disabled", "true");

    aTarget.appendChild(emptyElement);   
  }
}

function openTagInTabs(aId) {
  var theEnum = faves_coop.get(aId).children.enumerate();
  var urlList = [];
  while (theEnum.hasMoreElements())
    urlList.push(theEnum.getNext().URL);

  if (urlList.length > 0)
    gBrowser.loadTabs(urlList, true, true);
}


function populatesTag(aMenuPopup) {
  // Clean the list
  while(aMenuPopup.childNodes.length > 0)
    aMenuPopup.removeChild(aMenuPopup.lastChild);
  // Bookmarks
  var tag = faves_coop.get(aMenuPopup.id);
  var _enum = tag.children.enumerate();
  var count = 0;
  while (_enum.hasMoreElements()) {
    var bm = _enum.getNext();
    var item = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
    item.setAttribute("id", bm.id());
    item.setAttribute("label", bm.name);
    aMenuPopup.appendChild(item);
    count++;
  }
  // Open all in tab (only if number of bm > 1)
  if (count > 1) {
    item = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuseparator");
    aMenuPopup.appendChild(item); 
    item = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
    item.setAttribute("class", "openintabs-menuitem");
    item.setAttribute("label", BookmarksUtils.getLocaleString("cmd_bm_openfolder"));
    item.setAttribute("accesskey", BookmarksUtils.getLocaleString("cmd_bm_openfolder_accesskey"));
    aMenuPopup.appendChild(item);
  }
}

function populatesOnlineBookmarks(aMenuPopup) {
  var accountId = aMenuPopup.getAttribute("accountId");
  var serviceId = aMenuPopup.getAttribute("serviceId");

  // Clean the list
  while(aMenuPopup.childNodes.length > 0)
    aMenuPopup.removeChild(aMenuPopup.lastChild);

  var stream = faves_coop.get(aMenuPopup.id);
  if (stream.isRefreshing) {
    var item = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
    item.setAttribute("id", aMenuPopup.id);
    item.setAttribute("label", flockGetString("favorites/favorites", "flock.favs.overlay.refreshing"));
    item.setAttribute("disabled", "true");
    aMenuPopup.appendChild(item);
  }

  // Fill it with the tags
  var _enum = stream.children.enumerate();
  while (_enum.hasMoreElements()) {
    var tag = _enum.getNext();
    var menu = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menu");
    menu.setAttribute("label", tag.name);
    menu.setAttribute("id", tag.id());
    var menupopup = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menupopup");
    menupopup.setAttribute("id", tag.id());
    menupopup.setAttribute("onpopupshowing", "populatesTag(this);event.stopPropagation()");
    menu.appendChild(menupopup);
    aMenuPopup.appendChild(menu);
  }
}

function populatesFeedFolder (aMenuPopup, aFeedFolder) {
  var feedManager = Components.classes["@flock.com/feed-manager;1"].getService(Components.interfaces.flockIFeedManager);

  // Clean the list
  while(aMenuPopup.childNodes.length > 0)
    aMenuPopup.removeChild(aMenuPopup.lastChild);

  var children = aFeedFolder.getChildren();
  while (children.hasMoreElements()) {
    var folderItem = children.getNext();
    if (folderItem instanceof Components.interfaces.flockIFeedFolder) {
      folderItem.QueryInterface(Components.interfaces.flockIFeedFolder);
      var menu = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menu");
      menu.setAttribute("label", folderItem.getTitle());
      var menupopup = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menupopup");
      menupopup.feedFolder = folderItem;
      menupopup.setAttribute("onpopupshowing", "populatesFeedFolder(this, this.feedFolder);event.stopPropagation()");
      menu.appendChild(menupopup);
      aMenuPopup.appendChild(menu);     
    }
    else if (folderItem instanceof Components.interfaces.flockIFeed) {
      folderItem.QueryInterface(Components.interfaces.flockIFeed);
      var menuitem = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
      menuitem.setAttribute("label", folderItem.getTitle());
      if (folderItem.getFavicon())
        menuitem.setAttribute("image", folderItem.getFavicon().spec);
      menuitem.setAttribute("url", folderItem.getURL().spec);
      menuitem.setAttribute("oncommand", "newsSidebarOpen('FeedsMenuItemClick'); "
                            + "openUILink('flock://favorites/urn:flock:feed:' +"
                            + " this.getAttribute('url')); "
                            + "event.stopPropagation();");
      aMenuPopup.appendChild(menuitem);     
    }
  }
  
  BookmarksMenu.showEmptyItem(aMenuPopup);
}

function populatesFeedContext (aMenuPopup) {
  var feedManager = Components.classes["@flock.com/feed-manager;1"].getService(Components.interfaces.flockIFeedManager);

  var news = feedManager.getFeedContext("news");
  populatesFeedFolder(aMenuPopup, news.getRoot());
}

function publishCurrent(accountId, serviceId) {
  var svc = Cc[serviceId].getService(Ci.flockIBookmarkWebService);

  var bookmark = new Bookmark;
  bookmark.URL = gBrowser.currentURI.spec;
  bookmark.name = gBrowser.contentTitle;

  var listener = {
    onSuccess: function (aSubject, aTopic) {
      updateState();
    },
    onError: function (aSubject, aTopic, aError) {
      var msg = flockGetString("favorites/favorites", "flock.favs.overlay.publish.error");
      alert(msg);
    }
  }

  svc.publish(listener, accountId, bookmark, false);
}

function publishCurrentPage() {
  var bm = new Bookmark;
  bm.URL = gBrowser.currentURI.spec;
  bm.name = gBrowser.contentTitle;
  var params = {
    bookmark: bm
  };
  window.openDialog("chrome://browser/content/flock/favorites/publishOnline.xul", "_blank", "modal,centerscreen", params);
}

function debug(aStr) {
  dump('favoritesOverlay.js ' + aStr + '\n');
}
