// vim: ts=4 sw=4 expandtab cindent foldmethod=marker
//
// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL
//

// CONSTANTS
//Interfaces
const nsIProperties             = Components.interfaces.nsIProperties;
const nsILocalFile              = Components.interfaces.nsILocalFile;
const nsIPrefBranch             = Components.interfaces.nsIPrefBranch;
const flockIFavoritesService    = Components.interfaces.flockIFavoritesService;
const flockIFavoriteProperties  = Components.interfaces.flockIFavoriteProperties;
const nsIClassInfo              = Components.interfaces.nsIClassInfo;
const nsIRDFCompositeDataSource = Components.interfaces.nsIRDFCompositeDataSource;
const nsIRDFRemoteDataSource    = Components.interfaces.nsIRDFRemoteDataSource;
const nsIRDFDataSource          = Components.interfaces.nsIRDFDataSource;
const nsIRDFObserver            = Components.interfaces.nsIRDFObserver;
const nsIRDFService             = Components.interfaces.nsIRDFService;
const nsIRDFContainerUtils      = Components.interfaces.nsIRDFContainerUtils;
const nsIRDFContainer           = Components.interfaces.nsIRDFContainer;
const nsIRDFNode                = Components.interfaces.nsIRDFNode;
const nsIRDFResource            = Components.interfaces.nsIRDFResource;
const nsIRDFLiteral             = Components.interfaces.nsIRDFLiteral;
const nsIRDFInt                 = Components.interfaces.nsIRDFInt;
const nsIRDFDate                = Components.interfaces.nsIRDFDate;
const nsIRDFBlob                = Components.interfaces.nsIRDFBlob;
const nsIWindowWatcherService   = Components.interfaces.nsIWindowWatcherService;
const nsITimer                  = Components.interfaces.nsITimer;


// contract-ids
const FAVORITES_CONTRACTID      = '@mozilla.org/rdf/datasource;1?name=flock-favorites';
const COMPOSITE_CONTRACTID      = '@mozilla.org/rdf/datasource;1?name=composite-datasource';
const MEMORY_DS_CONTRACTID      = '@mozilla.org/rdf/datasource;1?name=in-memory-datasource';
const RDFSERVICE_CONTRACTID     = '@mozilla.org/rdf/rdf-service;1';
const RDFCONTUTILS_CONTRACTID   = '@mozilla.org/rdf/container-utils;1';
const CONTAINER_CONTRACTID      = '@mozilla.org/rdf/container;1';
const DIRECTORY_SVC_CONTRACTID  = '@mozilla.org/file/directory_service;1';
const LOCAL_FILE_CONTRACTID     = '@mozilla.org/file/local;1';
const PREFERENCES_CONTRACTID    = '@mozilla.org/preferences-service;1';
const WINDOWWATCHER_CONTRACTID  = '@mozilla.org/embedcomp/window-watcher;1';
const TIMER_CONTRACTID          = '@mozilla.org/timer;1';


// services
const RDFS = Components.classes[RDFSERVICE_CONTRACTID].getService (nsIRDFService);


// rdf namespaces
function Namespace (ns) { return function (arg) { return RDFS.GetResource (ns+arg); } }
const W3RDF                     = Namespace ('http://www.w3.org/1999/02/22-rdf-syntax-ns#');
const NSRDF                     = Namespace ('http://home.netscape.com/NC-rdf#');
const WBRDF                     = Namespace ('http://home.netscape.com/WEB-rdf#');
const FLRDF                     = Namespace ('http://flock.com/rdf#');

// rdf properties
const RDF_URL                   = NSRDF ('URL');
const RDF_TAGS                  = FLRDF ('tags');
const RDF_TAG                   = FLRDF ('tag');
const RDF_WSID                  = FLRDF ('wsid');
const RDF_NAME                  = NSRDF ('Name');
const RDF_TYPE                  = W3RDF ('type');
const RDF_FAVORITE              = FLRDF ('Favorite');
const RDF_COLLECTION            = FLRDF ('Collection');
const RDF_LIVEMARK              = NSRDF ('Livemark');
const RDF_FEEDURL               = NSRDF ('FeedURL');
const RDF_FEEDENABLED           = FLRDF ('feedEnabled');
const RDF_SHORTCUTURL           = NSRDF ('ShortcutURL');
const RDF_DESCRIPTION           = NSRDF ('Description');
const RDF_LOCAL                 = FLRDF ('local');
const RDF_PRIVATE               = FLRDF ('private');
const RDF_LASTMODIFIEDDATE      = WBRDF ('LastModifiedDate');
const RDF_ADDEDDATE             = FLRDF ('FavoriteAddDate');
const RDF_FAVORITESROOT         = FLRDF ('FavoritesRoot');
const RDF_COLLECTIONSROOT       = FLRDF ('CollectionsRoot');
const RDF_TAGSROOT              = FLRDF ('TagsRoot');
const RDF_INSTANCEOF            = W3RDF ('instanceOf');
const RDF_SEQ                   = W3RDF ('Seq');
const RDF_ISFEED                = FLRDF ('isFeed');
const RDF_FAVICON               = FLRDF ('favicon');
const RDF_TOOLBAR               = FLRDF ('toolbar');
const RDF_STORE                 = FLRDF ('store');
const RDF_VERSION               = FLRDF ('version');


// constants from the idl
const FIELD_WSID                = flockIFavoriteProperties.FIELD_WSID;
const FIELD_TAGS                = flockIFavoriteProperties.FIELD_TAGS;
const FIELD_NAME                = flockIFavoriteProperties.FIELD_NAME;
const FIELD_SHORTCUTURL         = flockIFavoriteProperties.FIELD_SHORTCUTURL;
const FIELD_DESCRIPTION         = flockIFavoriteProperties.FIELD_DESCRIPTION;
const FIELD_NSYNC               = flockIFavoriteProperties.FIELD_NSYNC;
const FIELD_LASTMODIFIEDDATE    = flockIFavoriteProperties.FIELD_LASTMODIFIEDDATE;
const FIELD_ADDEDDATE           = flockIFavoriteProperties.FIELD_ADDEDDATE;
const FIELD_LOCAL               = flockIFavoriteProperties.FIELD_LOCAL;
const FIELD_PRIVATE             = flockIFavoriteProperties.FIELD_PRIVATE;
const FIELD_FEED                = flockIFavoriteProperties.FIELD_FEED;
const FIELD_FAVICON             = flockIFavoriteProperties.FIELD_FAVICON;

// prefs
const PREF_NAME                 = 'flock.favorites.webservice.name';
const PREF_USERNAME             = 'flock.favorites.webservice.username';
const PREF_PASSWORD             = 'flock.favorites.webservice.password';
const PREF_WEB_SERVICE_ENABLED  = 'flock.favorites.webservice.enabled';
const PREF_FAVORITES_INITIAL    = 'flock.favorites.initialSetup';


// regular expressions
INVALID_XML_CHARACTERS = /[^\u0009\u000A\u000D\u0020-\uD7FF\uE000-\uFFFD]/g;

// a types system for favorites properties 
var FavoritesPropertyTypes = {}
FavoritesPropertyTypes.string = {
    toIdl: function (node) { return node.QueryInterface (nsIRDFLiteral).Value; },
    toRdf: function (value) { 
        return RDFS.GetLiteral (value.replace (INVALID_XML_CHARACTERS, '')); },
};
FavoritesPropertyTypes.date = {
    toIdl: function (node) { return node.QueryInterface (nsIRDFDate).Value/1000; },
    toRdf: function (value) { return RDFS.GetDateLiteral (value*1000); },
};
FavoritesPropertyTypes.boolean = {
    toIdl: function (node) { 
               return node.QueryInterface (nsIRDFLiteral).Value=="true"; },
    toRdf: function (value) { return RDFS.GetLiteral (value?"true":"false"); },
};

// a magic object for managing favorite properties
var FavoritePropertyMetadata = new Object ();
FavoritePropertyMetadata.list = new Array ();
FavoritePropertyMetadata.list.push ({
    name:        'wsid', 
    rdfProperty: RDF_WSID,
    idlProperty: 'wsid',
    idlMask:     FIELD_WSID,
    type:        FavoritesPropertyTypes.string,
});
FavoritePropertyMetadata.list.push ({
    name:        'tags', 
    rdfProperty: RDF_TAGS,
    idlProperty: 'tags',
    idlMask:     FIELD_TAGS,
    type:        FavoritesPropertyTypes.string,
    compare:     function (a, b) {
        // handle nullness
        if (a == null && b == null) {
            return true;
        } else if (a == null && b != null) {
            return false;
        } else if (a != null && b == null) {
            return false;
        }
        var list_a = a.split(/ *, */);
        var list_b = b.split(/ *, */);
        for (var i in list_b) {
            if (list_a.indexOf (list_b[i]) == -1) {
                return false;
            }
        }
        for (var i in list_a) {
            if (list_b.indexOf (list_a[i]) == -1) {
                return false;
            }
        }
        return true;
    },
});
FavoritePropertyMetadata.list.push ({
    name:        'name', 
    rdfProperty: RDF_NAME,
    idlProperty: 'name',
    idlMask:     FIELD_NAME,
    type:        FavoritesPropertyTypes.string,
});
FavoritePropertyMetadata.list.push ({
    name:        'shortcutURL', 
    rdfProperty: RDF_SHORTCUTURL,
    idlProperty: 'shortcutURL',
    idlMask:     FIELD_SHORTCUTURL,
    type:        FavoritesPropertyTypes.string,
});
FavoritePropertyMetadata.list.push ({
    name:        'description', 
    rdfProperty: RDF_DESCRIPTION,
    idlProperty: 'description',
    idlMask:     FIELD_DESCRIPTION,
    type:        FavoritesPropertyTypes.string,
});
FavoritePropertyMetadata.list.push ({
    name:        'lastModifiedDate', 
    rdfProperty: RDF_LASTMODIFIEDDATE,
    idlProperty: 'lastModifiedDate',
    idlMask:     FIELD_LASTMODIFIEDDATE,
    type:        FavoritesPropertyTypes.date,
});
FavoritePropertyMetadata.list.push ({
    name:        'addedDate', 
    rdfProperty: RDF_ADDEDDATE,
    idlProperty: 'addedDate',
    idlMask:     FIELD_ADDEDDATE,
    type:        FavoritesPropertyTypes.date,
});
FavoritePropertyMetadata.list.push ({
    name:        'local', 
    rdfProperty: RDF_LOCAL,
    idlProperty: 'local',
    idlMask:     FIELD_LOCAL,
    type:        FavoritesPropertyTypes.boolean,
});
FavoritePropertyMetadata.list.push ({
    name:        'private', 
    rdfProperty: RDF_PRIVATE,
    idlProperty: 'private',
    idlMask:     FIELD_PRIVATE,
    type:        FavoritesPropertyTypes.boolean,
});
FavoritePropertyMetadata.list.push ({
    name:        'feed', 
    rdfProperty: RDF_ISFEED,
    idlProperty: 'feed',
    idlMask:     FIELD_FEED,
    type:        FavoritesPropertyTypes.boolean,
});
FavoritePropertyMetadata.list.push ({
    name:        'favicon', 
    rdfProperty: RDF_FAVICON,
    idlProperty: 'favicon',
    idlMask:     FIELD_FAVICON,
    type:        FavoritesPropertyTypes.string,
});
FavoritePropertyMetadata.byRdfProperty = function (prop) {
    var value = prop.ValueUTF8;
    for (var i=0; i<this.list.length; i++) {
        if (this.list[i].rdfProperty.ValueUTF8 == value) {
            return this.list[i];
        }
    }
    return null;
}
FavoritePropertyMetadata.repr = function (props) {
    var value = '{ ';
    for (var i=0; i<this.list.length; i++) {
        if (props.fields & this.list[i].idlMask) {
            value = value + this.list[i].idlProperty + ': "' +
                props[this.list[i].idlProperty] + '", ';
        }
    }
    return value + '}';
}

// utility functions

// a toString method for RDF nodes
function rdf_repr (node) {
        try { 
            var value = node.QueryInterface (nsIRDFResource).ValueUTF8;
            value = value.replace ('http://www.w3.org/1999/02/22-rdf-syntax-ns#', 'W3:');
            value = value.replace ('http://home.netscape.com/NC-rdf#', 'NS:');
            value = value.replace ('http://developer.mozilla.org/rdf/vocabulary/forward-proxy#', 'FP:');
            value = value.replace ('http://flock.com/rdf#', 'FL:');
            return value;
        }
        catch (e) { }

        try { 
            var date = node.QueryInterface (nsIRDFDate);
            return 'date:'+date.Value;
        }
        catch (e) { }

        try { 
            var blob = node.QueryInterface (nsIRDFBlob);
            return 'blob: length'+blob.Length;
        }
        catch (e) { }

        try { 
            var int = node.QueryInterface (nsIRDFInt);
            return 'int:'+int.Value;
        }
        catch (e) { }

        try { 
            var literal = node.QueryInterface (nsIRDFLiteral);
            return '"'+literal.Value+'"';
        }
        catch (e) { }
        
        return 'unknown:'+node;
}

// debug rdf
function debug_rdf (data_source) {
    var ser=Components.classes["@mozilla.org/rdf/xml-serializer;1"].createInstance(Components.interfaces.nsIRDFXMLSerializer);
    ser.init(data_source);
    ser.QueryInterface(Components.interfaces.nsIRDFXMLSource).Serialize(
    {
        close : function(){debug("\n");},
        flush : function(){},
        write : function (buffer,count){ debug (buffer); return count; },
        writeFrom : function (stream,count){},
        isNonBlocking: false
    });
}

function flockFavoritesUtils () { } 
flockFavoritesUtils.prototype.setProperties = function (aDataSource, aResource, aProperties, aMask) {
    var mask = aProperties.fields & (aMask?aMask:-1);
    var change_mask = 0;
    for (var i = 0; i < FavoritePropertyMetadata.list.length; i++) {
        var prop = FavoritePropertyMetadata.list[i];
        if (mask & prop.idlMask) {
            if (aProperties[prop.idlProperty] == null) {
                // can't set to null
                continue;
            }
            // does that property already exist?
            var old_target = aDataSource.GetTarget (aResource, prop.rdfProperty, true);
            if (old_target) {
                if (prop.type.toIdl (old_target) != 
                        aProperties[prop.idlProperty]) {
                    aDataSource.Change (aResource, prop.rdfProperty, old_target,
                            prop.type.toRdf (aProperties[prop.idlProperty]));
                    change_mask | prop.idlMask;
                }
            } else {
                aDataSource.Assert (aResource, prop.rdfProperty, 
                        prop.type.toRdf (aProperties[prop.idlProperty]), true);
                change_mask | prop.idlMask;
            }
        }
    }
    // we want an url property
    var l = RDFS.GetLiteral (aResource.ValueUTF8);
    if (!aDataSource.HasAssertion (aResource, RDF_URL, l, true)) {
        aDataSource.Assert (aResource, RDF_URL, l, true);
    }
    return change_mask;
}
flockFavoritesUtils.prototype.getProperties = function (aDataSource, aResource, aMask) {
    var mask = aMask?aMask:-1;
    var properties = new Object ();
    properties.fields = 0;
    for (var i=0; i<FavoritePropertyMetadata.list.length; i++) {
        if (FavoritePropertyMetadata.list[i].idlMask & mask == 0) {
            // we don't care
            continue;
        }
        var property = FavoritePropertyMetadata.list[i].rdfProperty;
        var target = aDataSource.GetTarget (aResource, property, true);
        if (target == null) { continue; }
        var idlprop = FavoritePropertyMetadata.list[i].idlProperty
        properties[idlprop] = FavoritePropertyMetadata.list[i].type.toIdl (target);
        properties.fields |= FavoritePropertyMetadata.list[i].idlMask;
    }
    return properties;
}
flockFavoritesUtils.prototype.removeProperties = function (aDataSource, aResource) {
    // remove all the properties
    for (var i=0; i<FavoritePropertyMetadata.list.length; i++) {
        var property = FavoritePropertyMetadata.list[i].rdfProperty;
        var targets = aDataSource.GetTargets (aResource, property, true);
        while (targets.hasMoreElements ()) {
            aDataSource.Unassert (aResource, property, targets.getNext ());
        }
    }

    // remove the favorite type
    if (aDataSource.HasAssertion (aResource, RDF_TYPE, RDF_FAVORITE, true)) {
        aDataSource.Unassert (aResource, RDF_TYPE, RDF_FAVORITE);
    }

    // remove the sequence type
    if (aDataSource.HasAssertion (aResource, RDF_TYPE, RDF_SEQ, true)) {
        aDataSource.Unassert (aResource, RDF_TYPE, RDF_SEQ);
    }

    // remove the url property
    var l = RDFS.GetLiteral (aResource.ValueUTF8);
    if (aDataSource.HasAssertion (aResource, RDF_URL, l, true)) {
        aDataSource.Unassert (aResource, RDF_URL, l);
    }

    // FIXME: make it no longer a sequence if it is
}
flockFavoritesUtils.prototype.propertiesCompare = function (aProperties1, aProperties2, aMask) {  
    for (var i=0; i<FavoritePropertyMetadata.list.length; i++) {
        var fpm = FavoritePropertyMetadata.list[i];
        if ( !(fpm.idlMask & aMask) ) {
            // we're not looking at this field
            continue;
        }
        if (fpm.idlMask & aProperties1.fields != 
                fpm.idlMask & aProperties2.fields) {
            // the field is set on one but not the other
            return false;
        }

        if ( !(fpm.idlMask & aProperties1.fields) ) {
            // neither is set - not much we can tell from this
            continue;
        }

        var prop1 = aProperties1[fpm.idlProperty];
        var prop2 = aProperties2[fpm.idlProperty];
        if (fpm.compare) {
            if (!fpm.compare (prop1, prop2)) {
                return false;
            }
        } else {
            if (prop1 != prop2) {
                return false;
            }
        }
    }
    return true;
}
flockFavoritesUtils.prototype.propertiesToString = function (aProperties) { 
    return FavoritePropertyMetadata.repr (aProperties);
}
