// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

var discoveredFeeds = {};
var verifiedFeeds = {};

var newsFeedContextObserver = {
  onSubscribe: function(context, feed) {},

  onUnsubscribe: function(context, feed) {},

  onUnreadCountChange: function(context, unreadCount) {
    var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
    var strBundle = stringBundleService.createBundle("chrome://flock/locale/feeds/feeds.properties");

    var tooltip;
    if (unreadCount > 0) {
      $("flock_NewsSidebarBroadcaster").setAttribute("hasNew", true);
      tooltip = strBundle.formatStringFromName("flock.feed.tooltip1", [unreadCount], 1);
    } else {
      $("flock_NewsSidebarBroadcaster").removeAttribute("hasNew");
      tooltip = strBundle.GetStringFromName("flock.feed.tooltip2");
    }

    var closeLabel = strBundle.GetStringFromName("flock.feed.tooltip.close");
    var openLabel = strBundle.GetStringFromName("flock.feed.tooltip.open");

    try {
      var preText = $("news-button").checked ? closeLabel + " " : openLabel + " ";
      $("flock_NewsSidebarBroadcaster").setAttribute("tooltiptext", preText + tooltip);
    } catch (e) {}
  }
};

var newsDNDObserver = {
  canDrop: function(aEvent, aDragSession) {
    // Don't allow drops from outside browser or from media bar
    var node = aDragSession.sourceNode;
    if (!node || node.nodeName.toLowerCase() == "photo") {
      return false;
    }
    return true;
  },
    
  onDragOver: function(aEvent, aFlavour, aDragSession) {},
    
  onDragEnter: function(aEvent, aDragSession) {},
    
  onDragExit: function(aEvent, aDragSession) {},

  onDragStart: function(aEvent, aXferData, aDragAction) {},
  
  onDrop: function(aEvent, aXferData, aDragSession) {
    var url;
    switch(aXferData.flavour.contentType) {
      case "application/x-moz-file":
        var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
        var fileHandler = ios.getProtocolHandler("file").QueryInterface(Ci.nsIFileProtocolHandler);
        url = fileHandler.getURLSpecFromFile(aXferData.data);
        break;

      case "text/x-moz-url":
        url = aXferData.data.split("\n")[0];
        break;
    }

    var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
    var newsContext = feedManager.getFeedContext("news");
    var newsRoot = newsContext.getRoot();
    newsRoot.subscribeURL(url, null);
  },
  
  getSupportedFlavours: function() {
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
    flavourSet.appendFlavour("text/x-moz-url");
    return flavourSet;
  }
};

var feedsWebProgressListener = {
  lastUrl: null,
  currentUrl: null,
  
  onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {},
  
  onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {},
  
  onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {},

  onSecurityChange: function(aWebProgress, aRequest, aStatus) {},
  
  onLocationChange: function(aWebProgress, aRequest, aLocation) {
    if (aWebProgress.DOMWindow == gBrowser.selectedBrowser.contentWindow) {
      var url = aLocation.spec;
      // strip anchors
      url = url.replace(/#.*/, "");
      if (this.lastUrl != url) {
        this._newLocation(url);
        this.lastUrl = url;
      }
    }
  },
  
  _newLocation: function(aUrl) {
    this.currentUrl = aUrl;
    updateFeedButton();
  }
};

const FEED_TYPE_PRIORITY = { 'atom': 5, 'atom03': 4, 'rss2': 3, 'rss1': 2 };

// handler for LINK elements used in feed discovery
var feedsLinkHandler = {
  logger: Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger),
  feedManager: Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager),
  ios: Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService),
  
  handleEvent: function (aEvt) {
    var logger = this.logger;
    logger.init("feeddiscovery");

    var element = aEvt.target;
    var documentUrl = element.ownerDocument.URL;

    if (element.localName == "LINK" && element.rel == "alternate") {
      if (element.type == "application/rss+xml" ||
          element.type == "application/atom+xml" ||
          element.type == "text/xml") {
        logger.info("discovered feed: url: " + element.href);
        
        var feedManagerListener = {
          url: documentUrl,
          feedManager: this.feedManager,

          onGetFeedComplete: function(feed)
          {
            var url = feed.getURL();
            var title = feed.getTitle();
            var type = feed.getType();
            var finalURL = feed.getFinalURL();

            logger.info("verified feed: url: " + url.spec + " title: " +
                        title + " type: " + type);

            var documentVerifiedFeeds = verifiedFeeds[this.url];

            for (var c = 0; c < documentVerifiedFeeds.length; c++) {
              var testFeed = documentVerifiedFeeds[c];
              var testTitle = testFeed.getTitle();
              var testType = testFeed.getType();
              var testFinalURL = testFeed.getFinalURL();

              if (testFinalURL && finalURL && testFinalURL.equals(finalURL))
                return;

              if (testTitle == title && testType != type) {
                if (FEED_TYPE_PRIORITY[type] > FEED_TYPE_PRIORITY[testType]) {
                  documentVerifiedFeeds[c] = feed;
                }
                return;
              }
            }

            documentVerifiedFeeds.push(feed);
            updateFeedButton();
          },
          
          onError: function(error)
          {
            /* just ignore, bad feed */
          }
        };

        if (!(documentUrl in discoveredFeeds)) {
          discoveredFeeds[documentUrl] = [];
          verifiedFeeds[documentUrl] = [];
        }

        if (!(element.href in discoveredFeeds[documentUrl])) {
          discoveredFeeds[documentUrl][element.href] = {title: element.title, type: element.type};
          try {
            var url = this.ios.newURI(element.href, null, null);
            this.feedManager.getFeedMetadata(url, feedManagerListener);
          } catch (e) {}
        }
      }
    }
  }
};

var feedButtonStateChangeHandler = {
  handleEvent: function (aEvt) {
    var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
    var newsFeedContext = feedManager.getFeedContext("news");
    newsFeedContextObserver.onUnreadCountChange(newsFeedContext, newsFeedContext.getRoot().getUnreadCount());
  }
};

function updateFeedButton() {
  var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
  var stringBundle = stringBundleService.createBundle("chrome://flock/locale/favorites/favorites.properties");
  var url = feedsWebProgressListener.currentUrl;
  var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
  var newsContext = feedManager.getFeedContext("news");
  if (url in verifiedFeeds) {
    $("favoritesFeedButton").removeAttribute("disabled");
    if (verifiedFeeds[url].length == 1) {
      var tooltip = stringBundle.GetStringFromName("flock.favs.feeds.desc.1");
    } else {
      var tooltip = stringBundle.formatStringFromName("flock.favs.feeds.desc.2", [verifiedFeeds[url].length], 1);
    }
    $("favoritesFeedButton").setAttribute("tooltiptext", tooltip);
    for(var i= 0; i< verifiedFeeds[url].length; i++)
    {
      var feedURL = verifiedFeeds[url][i].getURL();
      if (feedURL && !newsContext.existsSubscription(feedURL)) {
        updateMessageBar();
        break;
      }
    }
  } else {
    $("favoritesFeedButton").setAttribute("disabled", true);
    var tooltip = stringBundle.GetStringFromName("flock.favs.feeds.desc.3");
    $("favoritesFeedButton").setAttribute("tooltiptext", tooltip);
  }
}

function updateMessageBar() {
  var message = flockGetString("favorites/favorites",
                               "flock.favs.overlay.discovery-feeds");

  var nBox = gBrowser.getNotificationBox();
  const priority = nBox.PRIORITY_WARNING_MEDIUM;

  var host = gBrowser.currentURI.host;
  // ignore subdomains 
  var domain = host.substring(host.indexOf(".") + 1, host.length);

  if (domain.indexOf(".") == -1) { // host does not have a sub-domain
    domain = host;
  }

  var links = [{
    label: flockGetString("favorites/favorites",
                          "flock.favs.overlay.show-me-how"),
    callback: function() {  
      $('favoritesFeedButton').click();
      var method = "flashForDiscover("+ 1 +",'favoritesFeedButton',$('favoritesFeedButton'));";  
      setTimeout(method, 500);
    }
  }];

  nBox.appendUniqueNotification(message,
                                "feeds-discovered",
                                "chrome://flock/skin/feeds/feed.png",
                                nBox.FLOCK_PRIORITY_LOW,
                                priority,
                                links,
                                domain);
}

function updateDiscoveredFeedsPopup() {
  var documentVerifiedFeeds = verifiedFeeds[feedsWebProgressListener.currentUrl]; 

  var feedButton = $("favoritesFeedButton");
  var container = feedButton.firstChild;

  while (container.lastChild)
    container.removeChild(container.lastChild);
  
  var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
  var newsContext = feedManager.getFeedContext("news");
  
  var feed, newMenuItem, url, title, attrs;
  for (var c = 0; c < documentVerifiedFeeds.length; c++) {
    feed = documentVerifiedFeeds[c];
    newMenuItem = document.createElement("menuitem");

    url = feed.getURL();
    title = feed.getTitle();

    attrs = { class: "flock-discovered-feeds-item menuitem-iconic",
                  label: title,
                  oncommand: "Components.classes['@flock.com/metrics-service;1']" 
                             + ".getService(Components.interfaces.flockIMetricsService)"
                             + ".report('FeedsSidebar-Open', 'Discovery'); "
                             + "openUILinkIn('" + url.spec + "', 'current'); "  
                             + "event.stopPropagation();"
                };

    if (url && newsContext.existsSubscription(url)) {
      attrs.image = "chrome://flock/skin/common/star16.png";
    }

    for (p in attrs) {
      newMenuItem.setAttribute(p, attrs[p]);
    }

    newMenuItem.setAttribute('discovered', 'false');
    container.appendChild(newMenuItem);
  }
}

function newsSidebarClose() {
  if ($("flock_NewsSidebarBroadcaster").getAttribute("checked")) {
    toggleSidebar('flock_NewsSidebarBroadcaster');
  }
}

function newsSidebarOpen(aMetricsValue) {
  if (!$("flock_NewsSidebarBroadcaster").getAttribute("checked")) {
    toggleSidebarWithMetric("flock_NewsSidebarBroadcaster",
                            "FeedsSidebar-Open",
                            null,
                            aMetricsValue);
  }
}

function feedsInit() {
  var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
  var newsFeedContext = feedManager.getFeedContext("news");
  newsFeedContext.addObserver(newsFeedContextObserver);
  // init the icon state
  newsFeedContextObserver.onUnreadCountChange(newsFeedContext, newsFeedContext.getRoot().getUnreadCount());
  // add feed discovery listener
  gBrowser.addEventListener("DOMLinkAdded", feedsLinkHandler, false);
  gBrowser.addProgressListener(feedsWebProgressListener, Ci.nsIWebProgress.NOTIFY_LOCATION);
  // add listener to change tooltip text depending on open/close state
  $("news-button").addEventListener("DOMAttrModified", feedButtonStateChangeHandler, false);
}

function feedsClose() {
  var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
  var newsFeedContext = feedManager.getFeedContext("news");
  newsFeedContext.removeObserver(newsFeedContextObserver);
  gBrowser.removeProgressListener(feedsWebProgressListener);
  $("news-button").removeEventListener("DOMAttrModified", feedButtonStateChangeHandler, false);
}


window.addEventListener("load", feedsInit, false);
window.addEventListener("unload", feedsClose, false);
