//
// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL
//

var feedUtils = {};

feedUtils.loadFeedsFromLocalOPML =
function feedUtils_loadFeedsFromLocalOPML(aURL, aListener, aCtxt) {
  const CC = Components.classes;
  const CI = Components.interfaces;

  var ios = CC["@mozilla.org/network/io-service;1"]
            .getService(CI.nsIIOService);
  var url = ios.newURI(aURL, null, null);

  var fm = CC["@flock.com/feed-manager;1"].getService(CI.flockIFeedManager);
  var newsContext = fm.getFeedContext("news");

  var opmlService = CC["@flock.com/opml-service;1"]
                    .getService(CI.flockIOpmlService);
  opmlService.subscribe(url, newsContext.getRoot(), true, aListener, aCtxt);
}

feedUtils.getURLFromFile =
function feedUtils_getURLFromFile(aFile) {
  const CC = Components.classes;
  const CI = Components.interfaces;

  var ios = CC["@mozilla.org/network/io-service;1"]
            .getService(CI.nsIIOService);
  var fileHandler = ios.getProtocolHandler("file")
                    .QueryInterface(CI.nsIFileProtocolHandler);
  return fileHandler.getURLSpecFromFile(aFile);
}


feedUtils.isValidURL =
function feedUtils_isValidURL(aURL) {
  const CC = Components.classes;
  const CI = Components.interfaces;

  const VALID_SCHEMES = ["http", "https", "file"];

  var ios = CC["@mozilla.org/network/io-service;1"]
            .getService(CI.nsIIOService);
  try {
    var uri = ios.newURI(aURL, null, null);
    return (VALID_SCHEMES.indexOf(uri.scheme) != -1);
  } catch (ex) {
    return false;
  }
}

feedUtils.addFeedURL =
function feedUtils_addFeedURL() {
  window.openDialog("chrome://flock/content/feeds/addFeedUrl.xul",
                    "AddFeedUrl",
                    "modal,centerscreen");
}

feedUtils.importOPML =
function feedUtils_importOPML() {
  window.openDialog("chrome://flock/content/feeds/importOPML.xul",
                    "ImportOPML",
                    "modal,centerscreen");
}

feedUtils.exportOPML =
function feedUtils_exportOPML() {
  const CC = Components.classes;
  const CI = Components.interfaces;

  var sbs = CC["@mozilla.org/intl/stringbundle;1"]
            .getService(CI.nsIStringBundleService);
  var bundle =
    sbs.createBundle("chrome://flock/locale/feeds/feeds.properties");

  var title = bundle.GetStringFromName("flock.feed.export.title");
  var filterText = bundle.GetStringFromName("flock.feed.export.opmlFiles");
  var defaultFileName =
    bundle.GetStringFromName("flock.feed.export.defaultFileName");
  var opmlTitle = bundle.GetStringFromName("flock.feed.export.opmlTitle");

  var filePicker = CC["@mozilla.org/filepicker;1"]
                   .createInstance(CI.nsIFilePicker);
  filePicker.init(window, title, CI.nsIFilePicker.modeSave);
  filePicker.appendFilter(filterText, "*.opml");
  filePicker.appendFilters(CI.nsIFilePicker.filterAll);
  filePicker.defaultExtension = "opml";
  filePicker.defaultString = defaultFileName + ".opml";

  if (filePicker.show() != CI.nsIFilePicker.returnCancel) {
    var fm = CC["@flock.com/feed-manager;1"].getService(CI.flockIFeedManager);
    var newsContext = fm.getFeedContext("news");

    var opmlService = CC["@flock.com/opml-service;1"]
                      .getService(CI.flockIOpmlService);
    opmlService.export(newsContext.getRoot(), opmlTitle, filePicker.file);
  }
}
