/*
 * BEGIN FLOCK GPL
 *
 * Copyright Flock Inc. 2005-2008
 * http://flock.com
 *
 * This file may be used under the terms of the
 * GNU General Public License Version 2 or later (the "GPL"),
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * END FLOCK GPL
 */

#include "flockLucene.h"
#include "flockLuceneThread.h"

#include "nsCOMPtr.h"

#include "nsIProxyObjectManager.h"


NS_IMPL_ISUPPORTS1(flockLucene, flockILucene)

flockLucene::flockLucene()
{
  /* member initializers and constructor code */
}

flockLucene::~flockLucene()
{
  /* destructor code */
}

NS_IMETHODIMP
flockLucene::AddDocument(const char *aURI, 
    const char *aType,
    const nsAString & aURL, 
    const nsAString & aTitle,
    const nsAString & aTags, 
    const nsAString & aDescription, 
    const nsAString & aText,
    flockILuceneListener *aListener)
{
  nsresult rv = NS_OK;

  // get the flockILuceneListener proxy
  nsCOMPtr<flockILuceneListener> listenerProxy;
  rv = NS_GetProxyForObject(NS_UI_THREAD_EVENTQ,
                            NS_GET_IID(flockILuceneListener),
                            aListener,
                            PROXY_ASYNC | PROXY_ALWAYS,
                            getter_AddRefs(listenerProxy));
  NS_ENSURE_SUCCESS(rv, rv);

  return mProxyAsync->AddDocument(aURI, aType, aURL, aTitle,
                                  aTags, aDescription, aText,
                                  listenerProxy);
}

NS_IMETHODIMP
flockLucene::AddDocumentSync(const char *aURI, 
    const char *aType,
    const nsAString & aURL, 
    const nsAString & aTitle,
    const nsAString & aTags, 
    const nsAString & aDescription, 
    const nsAString & aText)
{
  return mProxySync->AddDocumentSync(aURI, aType, aURL, aTitle, aTags, aDescription, aText);
}

NS_IMETHODIMP
flockLucene::DeleteDocument(const char *aURI, flockILuceneListener *aListener)
{
  nsresult rv = NS_OK;

  // get the flockILuceneListener proxy
  nsCOMPtr<flockILuceneListener> listenerProxy;
  rv = NS_GetProxyForObject(NS_UI_THREAD_EVENTQ,
                            NS_GET_IID(flockILuceneListener),
                            aListener,
                            PROXY_ASYNC | PROXY_ALWAYS,
                            getter_AddRefs(listenerProxy));
  NS_ENSURE_SUCCESS(rv, rv);
  
  return mProxyAsync->DeleteDocument(aURI, listenerProxy);
}

NS_IMETHODIMP
flockLucene::DeleteDocumentSync(const char *aURI)
{
  return mProxySync->DeleteDocumentSync(aURI);
}

NS_IMETHODIMP
flockLucene::Search(const nsAString &aQuery, const char *aType, PRInt32 aMaxResults, flockILuceneListener *aListener)
{
  nsresult rv = NS_OK;

  // get the flockILuceneListener proxy
  nsCOMPtr<flockILuceneListener> listenerProxy;
  rv = NS_GetProxyForObject(NS_UI_THREAD_EVENTQ,
                            NS_GET_IID(flockILuceneListener),
                            aListener,
                            PROXY_ASYNC | PROXY_ALWAYS,
                            getter_AddRefs(listenerProxy));
  NS_ENSURE_SUCCESS(rv, rv);

  return mProxyAsync->Search(aQuery, aType, aMaxResults, listenerProxy);
}

NS_IMETHODIMP
flockLucene::Init(nsIFile *aIndexDir)
{
  nsresult rv = NS_OK;

  mThread = new flockLuceneThread();
  mThread->Init(aIndexDir);
  mThread->GetProxyAsync(getter_AddRefs(mProxyAsync));
  mThread->GetProxySync(getter_AddRefs(mProxySync));
  
  return rv;
}
