// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

Components.utils.import("resource:///modules/FlockCryptoHash.jsm");

function PHOTO_DEBUG(x) { dump("photo.js: "+x+"\n"); }

var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                       .getService(Components.interfaces.mozIJSSubScriptLoader);

flock_photo = {};
flock_photouploader = {};

flock_photouploader.__defineGetter__("service", function(){
  if (flock_photouploader._service) return flock_photouploader._service;
  flock_photouploader._service = Components.classes["@mozilla.org/rdf/datasource;1?name=flock-photo-uploads"]
                                           .getService(Components.interfaces.flockIPhotoUploadService);
  return flock_photouploader._service; 
});

flock_photo.util = {};

flock_photo.util.helper = {};

flock_photo.util.__defineGetter__("gm", function () {
  if (flock_photo._gm) return flock_photo._gm;
  flock_photo.util._gm = Components.classes['@flock.com/imagescaler;1']
                                   .getService(Components.interfaces.flockIImageScaler);
  return flock_photo.util._gm;
});

flock_photo.util.__defineGetter__("gmAsync", function () {
  if (flock_photo._gmAsync) return flock_photo._gmAsync;
  flock_photo.util._gmAsync = Components.classes['@flock.com/asyncImageScaler;1']
                                        .getService(Components.interfaces.flockIAsyncImageScaler);
  return flock_photo.util._gmAsync;
});


flock_photo.util.createArgs = function() {
  var rval = new Array();
  for (var i = 0; i < arguments.length; ++i) {
    if (arguments[i]==null) continue;
    rval.push(arguments[i] + "");
  }
  var argString = rval.join(" ");
  PHOTO_DEBUG("ARGS: "+argString);
  return rval;
}

flock_photo.util.createArgsAsync = function() {
  var rval = Components.classes["@mozilla.org/array;1"]
                       .createInstance(Components.interfaces.nsIMutableArray);
  for (var i = 0; i < arguments.length; ++i) {
    var string = Components.classes["@mozilla.org/supports-string;1"]
                           .createInstance(Components.interfaces.nsISupportsString);
    if (arguments[i] == null) continue;
    string.data = arguments[i] + "";
    rval.appendElement(string, false);
  }
  return rval;
}

flock_photo.util.resize = function(aSize, aSrc, aDest) {
  var dims = aSize + "x" + aSize + ">";
  // var args = flock_photo.util.createArgs("convert", "-size", dims, aSrc, "-resize", dims, aDest);
  var args = flock_photo.util.createArgs("convert", "-size", dims, aSrc, "-coalesce", "-geometry", dims, aDest);
  flock_photo.util.gm.doMagick(args, args.length);
}

flock_photo.util.thumb = function(aSize, aSrc, aDest) {
  var dims = aSize + "x" + aSize;
  var args = flock_photo.util.createArgs("convert", aSrc,
    "-resize", "x" + aSize*2, "-resize",  aSize*2 + "x<",
    "-resize", "50%", "-gravity", "center", "-crop", dims + "+0+0",
    "-page",
    "0x0+0+0",
    aDest);
  flock_photo.util.gm.doMagick(args, args.length);
}

flock_photo.util.resizeAsync = function(aSize, aSrc, aDest, aListener) {
  var dims = aSize + "x" + aSize + ">";
  var args = flock_photo.util.createArgsAsync("convert", "-size", dims, aSrc, "-quality", "90", "-coalesce", "-geometry", dims, aDest);
  flock_photo.util.gmAsync.doAsyncMagick(args, aListener);
}

flock_photo.util.thumbAsync = function(aSize, aSrc, aDest, aListener) {
  var dims = aSize + "x" + aSize;
  var args = flock_photo.util.createArgsAsync("convert", aSrc,
    "-resize", "x" + aSize*2, "-resize",  aSize*2 + "x<",
    "-resize", "50%", "-quality", "90", "-gravity", "center", "-crop", dims + "+0+0",
    "-page",
    "0x0+0+0",
    aDest);
  flock_photo.util.gmAsync.doAsyncMagick(args, aListener);
}

flock_photo.util.rotate = function(aDegrees, aSrc, aDest) {
  var args = flock_photo.util.createArgs("convert", aSrc, "-rotate", aDegrees, aDest);
  flock_photo.util.gm.doMagick(args, args.length);
}

flock_photo.util.cropExt = function(aSrc, aDest, aWidth, aHeight, aHorzOffset, aVertOffset, aRot, aSize) {
  var dims = aWidth + "x" + aHeight;
  var rot = aRot;
  var args = flock_photo.util.createArgs("convert", aSrc,
  "-coalesce", "-crop",  dims + "+" + aHorzOffset + "+" + aVertOffset,
  "-page",
  "0x0+0+0",
  (aRot?"-rotate":null),
  (aRot?aRot:null),
  (aSize?"-resize":null),
  (aSize?aSize + "x" + aSize + ">":null),
  aDest);
  flock_photo.util.gm.doMagick(args, args.length);
}

flock_photo.util.getDimensions = function(aSrc) {
  var aDims = {};
  aDims.x = 0;
  aDims.y = 0;

  var args = flock_photo.util.createArgs("identify", aSrc);
  flock_photo.util.gm.doMagick(args, args.length);
  var profileDir = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components.interfaces.nsIProperties)
                             .get("ProfD", Components.interfaces.nsIFile);
  if (profileDir) {
    var inFile = Components.classes["@mozilla.org/file/local;1"]
                           .createInstance(Components.interfaces.nsILocalFile);
    inFile.initWithPath(profileDir.path);
    inFile.append("flockimageinfo.txt");
    var rawText = flock_photo.util.readFile(inFile);

    // could this be better done wtih a REGEXP???
    rawText.match(/.+?(\d+)x(\d+)\+(\d+)\+(\d+).+/);

    aDims.x = RegExp.$1;
    aDims.y = RegExp.$2;
    return aDims;
  }
  return null;
}

flock_photo.util.readFile = function(aFile) {
  try {
     var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                              .createInstance(Components.interfaces.nsIFileInputStream);
     inStream.init(aFile, 0x01, 0444, null);

     var sInStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
                               .createInstance(Components.interfaces.nsIScriptableInputStream);
     sInStream.init(inStream);
  } catch (e) { alert(e); }

  try {
    var str = sInStream.read(aFile.fileSize - 1);
  } catch (e) { PHOTO_DEBUG(e); }
  sInStream.close();
  inStream.close();
  return str;
}

flock_photo.util.getTmpDir = function() {
  var sinkFile = Components.classes["@mozilla.org/file/directory_service;1"]
                            .getService(Components.interfaces.nsIProperties)
                            .get("ProfD", Components.interfaces.nsIFile);
  sinkFile.append("flock_imagecache");
  if (!sinkFile.exists()) {
    sinkFile.createUnique(1, 0700);
  }
  return sinkFile;
}

flock_photo.util.getFileFromPath = function(aPath) {
  var file = Components.classes["@mozilla.org/file/local;1"]
                       .createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath(aPath);
  return file;
}

flock_photo.util.removeFileBySpec = function(aSpec, aRecursive) {
  PHOTO_DEBUG(aSpec);
  try {
    var file = flock_photo.util.getFileFromURL(aSpec);
    file.remove(aRecursive);
  }
  catch(e) {
  }
}

flock_photo.util.removeFileByPath = function(aPath, aRecursive) {
  PHOTO_DEBUG("removeFileByPath('"+aPath+"')");
  try {
    var file = flock_photo.util.getFileFromPath(aPath);
    file.remove(aRecursive);
  }
  catch(e) {
  }
}

flock_photo.util.getFileFromPath = function(aPath) {
  var file = Components.classes["@mozilla.org/file/local;1"]
                       .createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath(aPath);
  return file;
}

flock_photo.util.getFileFromURL = function(aURL) {
  var ios = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Components.interfaces.nsIIOService);
  var fileHandler = ios.getProtocolHandler("file")
                       .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
  var sourceSpec = fileHandler.getFileFromURLSpec(aURL);
  var sourceFile = Components.classes["@mozilla.org/file/local;1"]
                             .createInstance(Components.interfaces.nsILocalFile);
  sourceFile.initWithFile(sourceSpec);
  return sourceFile;
}

flock_photo.util.getURLFromFile = function(aFile) {
  var ios = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Components.interfaces.nsIIOService);
  var fileHandler = ios.getProtocolHandler("file")
                       .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
  return fileHandler.getURLSpecFromFile(aFile);
}

flock_photo.util.transferExif = function(sourceUrl, targetUrl) {
  var sourceFile = flock_photo.util.getFileFromURL(sourceUrl);
  var targetFile = flock_photo.util.getFileFromURL(targetUrl);
  var sourcePath = sourceFile.path;
  var targetPath = targetFile.path;
  
  var args = flock_photo.util.createArgs("convert", sourcePath, sourcePath + ".exif");
  flock_photo.util.gm.doMagick(args, args.length);
  
  try {
    var exifFile = Components.classes["@mozilla.org/file/local;1"]
                             .createInstance(Components.interfaces.nsILocalFile);
    exifFile.initWithPath(sourcePath + ".exif");
    exifFile.moveTo(null, exifFile.leafName + ".app1");
    
    args = flock_photo.util.createArgs("convert", "-profile", sourcePath + ".exif.app1", targetPath, targetPath);
    flock_photo.util.gm.doMagick(args, args.length);
    flock_photo.util.removeFileByPath(sourcePath + ".exif.app1");
  } catch(e) {}
}

flock_photo.util.callCount = 0;
flock_photo.util.getUniqueFname = function(aFName) {
  aFName.match(/.+\.(.+)/);
  var ext = RegExp.$1;
  if (!ext) ext = "";
  var now = new Date();
  var name = aFName + now + flock_photo.util.callCount++;
  var sinkFileName = FlockCryptoHash.md5(name);
  sinkFileName = sinkFileName + "." + ext;
  return sinkFileName;
}

flock_photo.util.contextUploadImage = function() {
  // If there is no account, offer to configure one now
  if (!flock_photouploader.service.hasOnePhotoAccount()) {
    flock_photouploader.service.promptForAccount();
    return;
  }


  var node = window.document.popupNode;
  var contentDetector = new nsContentDetector();
  contentDetector.setTarget(node);

  var photoUrls = new Array(); 
  photoUrls[0] = contentDetector.imageURL;
  dump("====== upload "+contentDetector.imageURL+"\n");
  var dropProcessor = new PhotoDropProcessor(photoUrls, null);
  dropProcessor.start();
  flock_photouploader.service.launchUploader();
}

function PhotoDropProcessor(aList, aHoverChild)  {
  this.mList = aList;
  this.mHoverChild = aHoverChild;
  this.mLauncherFunc = flock_photo.util.launchUploader;
  this.mUploaderSvc = flock_photouploader.service;
}

PhotoDropProcessor.prototype = {
  mList: null,
  start: function() {
    var str = "";
    if (this.mHoverChild) {
      var newThing = Components.classes['@flock.com/photo-upload;1']
                               .createInstance(Components.interfaces.flockIPhotoUpload);
      newThing.id = this.mHoverChild.getAttribute("id");
      this.mHoverChild = newThing;
    }
    this.mUploaderSvc.addListener(this);
    this.next();
  },
  next: function() {
    if (this.mList.length == 0) {
      this.mUploaderSvc.startImportQueue(this);
      return;
    }
    var url = this.mList.shift();
    if (url.split(":")[0] == "file") {
      this.mUploaderSvc.addMediaToImportQueue(url, this);
      this.next();
    } else {
      // dnding from a remote source
      var URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
                               .getService(Components.interfaces.nsIURIFixup);
      var uri = URIFixup.createFixupURI(url, URIFixup.FIXUP_FLAG_NONE);
      
      var IOService = Components.classes["@mozilla.org/network/io-service;1"]
                                .getService(Components.interfaces.nsIIOService);
      var channel = IOService.newChannelFromURI(uri);
      
      var self = this;
      
      var streamListener = {
        processor: self,
        sinkFile: null,
        sinkBufferStream: null,
        
        onStartRequest: function(request, context) {
          var extension;
          switch (channel.contentType) {
            case "image/jpeg": extension = "jpg"; break;
            case "image/png": extension = "png"; break;
            case "image/gif": extension = "gif"; break;
            default: extension = "jpg"; break;
          }
          //var sinkFileName = "remoteImage." + extension;
          var sinkFileName = url.split("/").pop().replace(/[\?=&]/g, '');
          this.sinkFile = Components.classes["@mozilla.org/file/directory_service;1"]
                                    .getService(Components.interfaces.nsIProperties)
                                    .get("ProfD", Components.interfaces.nsIFile);
          this.sinkFile.append("flock_imagecache");
          if (!this.sinkFile.exists()) {
            this.sinkFile.createUnique(this.sinkFile.DIRECTORY_TYPE, 0700);
          }
          if (sinkFileName) this.sinkFile.append(sinkFileName);

          var sinkFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                                         .createInstance(Components.interfaces.nsIFileOutputStream);
          sinkFileStream.init(this.sinkFile, 0x02 | 0x08 | 0x20, 0664, 0);
          this.sinkBufferStream = Components.classes["@mozilla.org/network/buffered-output-stream;1"]
                                            .createInstance(Components.interfaces.nsIBufferedOutputStream);
          this.sinkBufferStream.init(sinkFileStream, 65536 * 4);
        },
        
        onStopRequest: function(request, context, statusCode) {
          var ios = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
          var fileHandler = ios.getProtocolHandler("file")
                               .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
          this.sinkBufferStream.close();
          this.processor.mUploaderSvc.addMediaToImportQueue(fileHandler.getURLSpecFromFile(this.sinkFile), this.processor);
          this.processor.next();
        },
        
        onDataAvailable: function(request, context, inputStream, offset, count) {
          this.sinkBufferStream.writeFrom(inputStream, count);
        }
      }

      channel.asyncOpen(streamListener, null);
    }
  },
  onMediaBatchImportStart: function(aNumImport) {
  },
  onMediaBatchImportFinish: function() {
    this.mUploaderSvc.removeListener(this);
  },
  onMediaImportStart: function(aFilename, aNum) {
  },
  onMediaImportFinish: function(aPhotoUpload) {
    this.mUploaderSvc.addPhotoUpload((this.mHoverChild?this.mHoverChild:null),aPhotoUpload);
  },
  onMediaImportError: function(aFile) {
    var stringBundle = flock_photo.util.getStringBundle();
    alert(stringBundle.GetStringFromName("flock.photo.upload.error.invalid").replace("%1",aFile));
    this.next();
  },
  onUploadAdd: function() {
  },
  onUploadRemove: function() {
  },
  onError: function(aError) {
    alert(aError.errorString);
    this.next();
  },
  cancel: function() {
    // not impl
  }
}

// transform obj
flock_photo.util.transform = function() {};
flock_photo.util.transform.prototype.type = null;
flock_photo.util.transform.prototype.params = Array();

flock_photo.util.deleteChildren = function(aElm) {
  var toDelete = new Array();
  for (var i = 0; i < aElm.childNodes.length; ++i) {
    var kid = aElm.childNodes.item(i);
    if (!kid.getAttribute("dontdelete")) {
      toDelete.push(kid);
    }
  }
  for (var i = 0; i < toDelete.length; ++i) {
    var kid = toDelete[i];
    kid.parentNode.removeChild(kid);
  }
}


flock_photo.util.launchUploader = function() {
  flock_photouploader.service.launchUploader();
}


flock_photo.util.selectServiceGroupInSettings = function() {
  var sidebarObserver = Components.classes["@mozilla.org/observer-service;1"]
                                  .getService(Components.interfaces.nsIObserverService);

  sidebarObserver.notifyObservers(sidebarObserver, "accountsSidebar", "Media Sharing");    
}

flock_photo.util.launchSettings = function(aParams) {
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                .getService(Components.interfaces.nsIPromptService);
  win.toggleSidebarWithMetric("flock_AccountsSidebarBroadcaster",
                              "AccountSidebar-Open",
                              null,
                              "PhotoUploaderNoAccounts",
                              true);
  // We need to delay so that the accounts and services sidebar can open and load before we command it to do things.
  setTimeout('flock_photo.util.selectServiceGroupInSettings();', 500);  
}

flock_photo.util.launchSinglePhotoUpload = function(aPhoto, aTarget) {
  flock_photo.util.showSinglePhotoUploadWindow(aPhoto, aTarget);
}

flock_photo.util.showSinglePhotoUploadWindow = function(aPhoto, aTarget) {
  var params = {
    aPhoto: aPhoto,
    aTarget: aTarget
  };
  var win = aTarget.ownerDocument.defaultView;
  win.openDialog( "chrome://browser/content/flock/photo/photoPropsQuickUpload.xul",
                     "QuickUploader", "modal,close,centerscreen", params );
}

flock_photo.util.launchUploadOptions = function(aParams) {
  var rv = window.openDialog( "chrome://browser/content/flock/photo/flockPhotoUploadOptions.xul",
                              "_blank", "modal,centerscreen", aParams );
  return rv;
}

flock_photo.util.isAPhoto = function(aURL) {
  if (aURL.indexOf("file") != 0) return false;
  var lc = aURL.toLowerCase();
  if (lc.match(/.+jpg$/)) return true;
  if (lc.match(/.+jpeg$/)) return true;
  if (lc.match(/.+png$/)) return true;
  if (lc.match(/.+gif$/)) return true;
  if (lc.match(/.+bmp$/)) return true;
  return false;
}

flock_photo.util.getStringBundle = function() {
  var localeService =
    Components.classes["@mozilla.org/intl/nslocaleservice;1"]
              .getService(Components.interfaces.nsILocaleService);
  var stringBundleService =
    Components.classes["@mozilla.org/intl/stringbundle;1"]
              .getService(Components.interfaces.nsIStringBundleService);
  var stringBundle =
    stringBundleService.createBundle( "chrome://flock/locale/photo/photo.properties",
                                      localeService.getApplicationLocale() );
  return stringBundle;
}

flock_photo.util.mCoop = null;

flock_photo.util.createMenuItem = function(aAttribs) {
  var menuitem = document.createElement("menuitem");
  for (var attrib in aAttribs) {
    menuitem.setAttribute(attrib, aAttribs[attrib]);
  }
  return menuitem;
}

flock_photo.util.populateServicesPopup = function(aPopup, aListener) {
  PHOTO_DEBUG("flock.photo.util.populateServicesPopup()");

  // Make sure Coop is instanciated
  if (!flock_photo.util.mCoop) {
    flock_photo.util.mCoop =
      Components.classes["@flock.com/singleton;1"]
                .getService(Components.interfaces.flockISingleton)
                .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                .wrappedJSObject;
  }
  var _coop = flock_photo.util.mCoop;

  // Clear out the current contents of the popup
  flock_photo.util.deleteChildren(aPopup);

  // Iterate through all accounts, looking for photo upload accounts.
  // Separate them according to auth state
  var loggedIn = [];
  var loggedOut = [];
  var accountsEnum = _coop.accounts_root.children.enumerate();
  var defaultAccountId = flock_photouploader.service.getDefaultService();

  while (accountsEnum.hasMoreElements()) {
    var acctCoopObj = accountsEnum.getNext();
    if (!acctCoopObj) continue; // getNext() can return NULL when hasMoreElements() is TRUE.
    var svc = Components.classes[acctCoopObj.serviceId]
                        .getService(Components.interfaces.flockIWebService);
    if (svc instanceof Components.interfaces.flockIMediaUploadWebService) {
      PHOTO_DEBUG("found upload account on service: "+acctCoopObj.serviceId);
      var entry = {
        acct: acctCoopObj,
        svc: svc
      };
      if (acctCoopObj.isAuthenticated) {
        PHOTO_DEBUG(" - it's authenticated");
        loggedIn.push(entry);
      } else {
        PHOTO_DEBUG(" - it's NOT authenticated");
        loggedOut.push(entry);
      }
    }
  }

  // Grab a string bundle for the localized strings
  var stringBundle = flock_photo.util.getStringBundle();

  // Add menu items for the authenticated accounts
  for (var i = 0; i < loggedIn.length; i++) {
    var acct = loggedIn[i].acct;
    var svc = loggedIn[i].svc;
    var acctName = svc.title + " (" + acct.name + ")";
    var itemLabel = stringBundle.GetStringFromName("flock.photo.service.use")
                                .replace("%account", acctName);
    var menuitem = flock_photo.util.createMenuItem({
      value: acct.id(),
      src: acct.favicon,
      label: itemLabel
    });
    menuitem.svc = svc;
    aPopup.appendChild(menuitem);
    if (acct.id() == defaultAccountId) {
      aPopup.parentNode.selectedItem = menuitem;
    }
  }

  if (loggedIn.length && loggedOut.length) {
    aPopup.appendChild(document.createElement("menuseparator"));
  }

  // Add menu items for the unauthenticated accounts
  for (var i = 0; i < loggedOut.length; i++) {
    var acct = loggedOut[i].acct;
    var svc = loggedOut[i].svc;
    var acctName = svc.title + " (" + acct.name + ")";
    var itemLabel = stringBundle.GetStringFromName("flock.photo.service.login")
                                .replace("%account", acctName);
    var menuitem = flock_photo.util.createMenuItem({
      value: acct.id(),
      src: acct.favicon,
      label: itemLabel,
      oncommand: "flock_photo.util.authAccount(this.value, this.listener);"
    });
    menuitem.svc = svc;
    menuitem.listener = aListener;
    aPopup.appendChild(menuitem);
  }

  // Returns TRUE if there are authenticated accounts
  return (loggedIn.length > 0);
}

flock_photo.util.authAccount = function(aAccountURN, aListener) {
  PHOTO_DEBUG("flock_photo.util.authAccount('"+aAccountURN+"')");
  var acct = flock_photo.util.mCoop.get(aAccountURN);
  flock_photo.util.authForService(acct.serviceId, aListener);
}

flock_photo.util.authForService = function(aServiceID, aListener) {
  PHOTO_DEBUG("flock_photo.util.authForService('"+aServiceID+"')");
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Components.interfaces.flockIAccountUtils);
  var svc = Components.classes[aServiceID]
                      .getService(Components.interfaces.flockIWebService)
                      .QueryInterface(Components.interfaces.flockIAuthenticateNewAccount);
  // We need to de-authenticate any currently authenticated account
  var acctURN = acUtils.getFirstAuthenticatedAccountForService(aServiceID);
  if (acctURN) {
    var acct = svc.getAccount(acctURN);
    acct.logout(null);
  }
  svc.authenticateNewAccount(aListener);
}

var photoUploadDNDObserver = {
  canDrop: function (aEvent, aDragSession) {
    var isPhoto = true;
    var node = aDragSession.sourceNode;
    if (node) {
      var nodeName = node.nodeName.toLowerCase();
      if (nodeName == "photo") {
        // We currently don't support uploading mediabar videos or
        // drag-and-drop from the mediabar to itself
        isPhoto = (!this._isRichVideo(aDragSession)) &&
                  (aEvent.target.nodeName.toLowerCase() != "photo");
      } else {
        isPhoto = (nodeName == "img") ||
                  (nodeName == "shelficon") ||
                  (nodeName == "photoupload");
      }
    }
    return (isPhoto && 
            !flock_photouploader.service.isImporting &&
            !flock_photouploader.service.isUploading);
  },

  canHandleMultipleItems: true,

  onDragOver: function(aEvent, aFlavor, aSession) {
    if (!this._isRichVideo(aSession)) {
      aEvent.target.setAttribute("dropzoneengaged", "true");
    }
  },

  onDragEnter: function(aEvent, aSession) {
    if (!this._isRichVideo(aSession)) {
      var target = aEvent.target;
      if (this._isHoverOverUploadButton(target)) {
        target = target.parentNode;
      }
      target.setAttribute("dropzoneengaged", "true");
    }
  },

  onDragExit: function(aEvent, aSession) {
    if (!this._isRichVideo(aSession)) {
      var target = aEvent.target;
      if (this._isHoverOverUploadButton(target)) {
        target = target.parentNode;
      }
      target.removeAttribute("dropzoneengaged");
    } 
  },

  onDrop: function(aEvent, aDropData, aSession) {
    PHOTO_DEBUG( "photoUploadDNDObserver: drop: sourceNode: "
           + (aSession.sourceNode ? aSession.sourceNode.localName : "none")
           + " items: "+aDropData.dataList.length );
    aEvent.stopPropagation();
    var photoURLs = new Array();

    for (var c = 0; c < aDropData.dataList.length; c++) {
      var supports = aDropData.dataList[c].dataList[0].supports;
      var contentType = aDropData.dataList[c].dataList[0].flavour.contentType;
      var url;

      if (aSession.sourceNode && aSession.sourceNode.localName == "IMG") {
        contentType = null;
      }

      switch (contentType) {
        case "application/x-moz-file":
          try {
            var file = supports.QueryInterface(Components.interfaces.nsIFile);
            handleFile(file, photoURLs);
          } catch(e) {}
          break;
        case "application/flock-photo":
          url = aSession.sourceNode.largeSizePhoto;
          photoURLs.push(url);
          break;
        case "flock/shelficon":
          url = aSession.sourceNode.url;
          photoURLs.push(url);
          break;
        default:
          if (aSession.sourceNode && aSession.sourceNode.localName == "IMG") {
            url = aSession.sourceNode.getAttribute("src");
            var baseURI = aSession.sourceDocument.baseURI;
            var URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
                                     .getService(Components.interfaces.nsIURIFixup);
            url = URIFixup.createFixupURI(baseURI, URIFixup.FIXUP_FLAG_NONE).resolve(url);
            photoURLs.push(url);
          } else {
            url = supports.QueryInterface(Components.interfaces.nsISupportsString).data;
            url = url.split("\n")[0];
            if (isAPhoto(url)) {
              photoURLs.push(url);
            } else {
              if (url.indexOf("file") == 0) {
                var ios = Components.classes["@mozilla.org/network/io-service;1"]
                                    .getService(Components.interfaces.nsIIOService);
                var fileHandler = ios.getProtocolHandler("file")
                                     .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
                var sourceSpec = fileHandler.getFileFromURLSpec(url);
                if (sourceSpec.exists()) {
                  handleFile(sourceSpec, photoURLs);
                }
              }
            }
          }
          break;
      }
    }

    PHOTO_DEBUG("photoUploadDNDObserver: found "+photoURLs.length+" image(s)");

    var hoverNode = null;
    if (aEvent.target.nodeName == "slideyitem") {
      hoverNode = aEvent.target.parentNode.hoverChild.firstChild;
    }
    if (aEvent.target.nodeName == "photoupload") {
      var slideyboxHover = aEvent.target.parentNode.parentNode.hoverChild;
      if (slideyboxHover) {
        hoverNode = slideyboxHover.firstChild;
      }
    }
    
    if (photoURLs.length) {
      flock_photo.util.launchUploader();
      var dropProcessor = new PhotoDropProcessor(photoURLs, hoverNode); 
      window.setTimeout(function foo() {dropProcessor.start();}, 400);
    }
    
    function handleFile(file, photoURLs) {
      if (file.isDirectory()) {
        var theEnum = file.directoryEntries;
        while (theEnum.hasMoreElements()) {
          var child = theEnum.getNext().QueryInterface(Components.interfaces.nsIFile);
          if (!child.isHidden()) {
            handleFile(child, photoURLs);
          }
        }
      } else {
        url = flock_photo.util.getURLFromFile(file);
        if (isAPhoto(url)) {
          photoURLs.push(url);
        }
        else {
          var stringBundle = flock_photo.util.getStringBundle();
          alert(stringBundle.GetStringFromName("flock.photo.upload.error.unsupported").replace("%1",file.leafName));
        }
      }
    }

    function isAPhoto(aURL) {
      if (aURL.indexOf("file") != 0) return false;
      var lc = aURL.toLowerCase();
      if (lc.match(/.+jpg$/)) return true;
      if (lc.match(/.+jpeg$/)) return true;
      if (lc.match(/.+png$/)) return true;
      if (lc.match(/.+gif$/)) return true;
      if (lc.match(/.+bmp$/)) return true;
      return false;
    }
  },

  getSupportedFlavours: function() {
    var flavors = new FlavourSet();
    flavors.appendFlavour("application/x-moz-file", "nsIFile");
    flavors.appendFlavour("text/x-moz-url");
    flavors.appendFlavour("flock/shelficon");
    flavors.appendFlavour("application/flock-photo", "flockIMediaItem");
    return flavors;
  },

  _isRichVideo: function(aDragSession) {
    var isVideo = false;
    if (aDragSession &&
        aDragSession.isDataFlavorSupported("application/flock-photo"))
    {
      if (!this._richDNDSvc) {
        this._richDNDSvc = Components.classes["@flock.com/rich-dnd-service;1"]
                           .getService(Components.interfaces
                                                 .flockIRichDNDService);
      }
      isVideo = this._richDNDSvc.isSelectionRichVideoContent(aDragSession);
    }
    return isVideo;
  },

  _isHoverOverUploadButton: function(aTarget) {
    return aTarget.nodeName == "toolbarbutton" &&
           aTarget.className == "upload-button";
  }
}
