/* vim: ts=4 sw=4 
*/
/*
 * BEGIN FLOCK GPL
 *
 * Copyright Flock Inc. 2005-2008
 * http://flock.com
 *
 * This file may be used under the terms of the
 * GNU General Public License Version 2 or later (the "GPL"),
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * END FLOCK GPL
 */


#include "flockAsyncImageScalerImpl.h"

#include "nsIProcess.h"

#include "nsString.h"

#include "nsIEventQueueService.h"

#include "nsIComponentManager.h"
#include "nsIServiceManager.h"
#include "nsServiceManagerUtils.h"
#include "nsIProxyObjectManager.h"
#include "nsComponentManagerUtils.h"

#include "nsISimpleEnumerator.h"
#include "nsIArray.h"
#include "nsISupportsPrimitives.h"

#include "nsEmbedString.h"

#include "prlock.h"
#include "prmem.h"

#include "nsAutoLock.h"

#include "nsCRT.h"

#include "stdio.h"

static NS_DEFINE_CID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);


NS_IMPL_THREADSAFE_ISUPPORTS1(flockAsyncImageScalerImpl, flockIAsyncImageScaler)

flockAsyncImageScalerImpl::flockAsyncImageScalerImpl()
{
}

flockAsyncImageScalerImpl::~flockAsyncImageScalerImpl()
{
}

NS_IMETHODIMP flockAsyncImageScalerImpl::DoAsyncMagick(nsIArray* aParamArray, flockIAsyncImageScalerListener* aListener)
{
    nsresult rv;
    nsCOMPtr<nsIProcess> proc(do_CreateInstance("@mozilla.org/process/util;1", &rv));
    if(NS_FAILED(rv)) return rv;

    nsCOMPtr<nsIFile> gmExec;
    NS_GetSpecialDirectory("CurProcD", getter_AddRefs(gmExec));

    //nsCAutoString nativePath;
    //gmExec->GetNativePath(nativePath);
#if defined(XP_WIN)
    //_chdir(nativePath.get());
    gmExec->Append(NS_LITERAL_STRING("gm.exe"));
#else
    //chdir(nativePath.get());
    gmExec->Append(NS_LITERAL_STRING("gm"));
#endif
    PRBool gmExists = PR_FALSE;
    rv = gmExec->Exists(&gmExists);
    if(NS_FAILED(rv)) return rv;
    if(!gmExists) return -1;
    rv = proc->Init(gmExec);
    if(NS_FAILED(rv)) return rv;
    PRUint32 exitCode;
    
    PRUint32 length;
    aParamArray->GetLength(&length);
    
    printf("ImageScalerImpl: %d\n", length);

	nsCOMPtr<nsISimpleEnumerator> enumerator;
	aParamArray->Enumerate(getter_AddRefs(enumerator));
	
	char** array = (char**)nsMemory::Alloc (sizeof(char*)*length);
	
	for (PRUint32 i=0; i<length; i++) {
		nsCOMPtr<nsISupports> item;
		enumerator->GetNext(getter_AddRefs(item));
		
		nsCOMPtr<nsISupportsString> ss = do_QueryInterface (item);
		
		nsString unistring;
		ss->GetData (unistring);
		
		array[i] = ToNewCString (unistring); // ToNewUTF8String breaks extended ASCII.
		printf ("argument %d: \"%s\"\n", i, array[i]);
	}

    aListener->OnFinished( proc->Run(/*blocking*/PR_TRUE, (const char**)array, length, &exitCode) );
    
    for (PRUint32  i=0; i<length; i++) {
    	nsMemory::Free (array[i]);
    }
    nsMemory::Free (array);
    
    printf("ImageScalerImpl: exitCode: %d\n", exitCode);
    
    return NS_OK;
}