/*
 * BEGIN FLOCK GPL
 *
 * Copyright Flock Inc. 2005-2008
 * http://flock.com
 *
 * This file may be used under the terms of the
 * GNU General Public License Version 2 or later (the "GPL"),
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * END FLOCK GPL
 */

#include "nsIGenericFactory.h"
#include "flockImageScalerImpl.h"
#include "flockAsyncImageScalerCID.h"
#include "flockAsyncImageScaler.h"
#include "flockAsyncImageScalerImpl.h"
#include "flockImageTransformService.h"

#include "nsICategoryManager.h"
#include "nsServiceManagerUtils.h"
#include "nsString.h"


#define FLOCK_IMAGESCALER_CID { /* cfc1c2d0-162a-4e31-a555-e09ac02f24cb */ 0xcfc1c2d0, 0x162a, 0x4e31, {0xa5, 0x55, 0xe0, 0x9a, 0xc0, 0x2f, 0x24, 0xcb} }

#define FLOCK_IMAGESCALER_CONTRACTID "@flock.com/imagescaler;1"

#define FLOCK_IMAGESCALER_CLASSNAME "Flock Image Scaler"


NS_GENERIC_FACTORY_CONSTRUCTOR(flockImageScalerImpl)
NS_GENERIC_FACTORY_CONSTRUCTOR(flockAsyncImageScaler)
NS_GENERIC_FACTORY_CONSTRUCTOR(flockAsyncImageScalerImpl)
NS_GENERIC_FACTORY_SINGLETON_CONSTRUCTOR(flockImageTransformService,
    flockImageTransformService::GetSingleton)

static NS_METHOD
flockImageTransformServiceRegister(nsIComponentManager *aCompMgr,
                                   nsIFile *aPath,
                                   const char *registryLocation,
                                   const char *componentType,
                                   const nsModuleComponentInfo *info)
{
    nsresult rv;
    nsCOMPtr<nsICategoryManager> catmgr = 
        do_GetService (NS_CATEGORYMANAGER_CONTRACTID, &rv);
    if (NS_FAILED (rv)) {
        return rv;
    }

    nsXPIDLCString previous;
    rv = catmgr->AddCategoryEntry("flock-startup",
                                  FLOCK_IMAGETRANSFORMSERVICE_CLASSNAME,
                                  "service," FLOCK_IMAGETRANSFORMSERVICE_CONTRACTID,
                                  PR_TRUE, PR_TRUE, getter_Copies(previous));
    NS_ENSURE_SUCCESS(rv, rv);

    return rv;
}

static NS_METHOD
flockImageTransformServiceUnregister(nsIComponentManager *aCompMgr,
                                     nsIFile *aPath,
                                     const char *registryLocation,
                                     const nsModuleComponentInfo *info)
{
    nsresult rv;
    nsCOMPtr<nsICategoryManager> catmgr = 
        do_GetService (NS_CATEGORYMANAGER_CONTRACTID, &rv);
    if (NS_FAILED (rv)) {
        return rv;
    }

    rv = catmgr->DeleteCategoryEntry("flock-startup",
                                     FLOCK_IMAGETRANSFORMSERVICE_CLASSNAME,
                                     PR_TRUE);
    NS_ENSURE_SUCCESS(rv, rv);

    return rv;
}

static nsModuleComponentInfo components[] =
{
    {
        FLOCK_IMAGESCALER_CLASSNAME, 
        FLOCK_IMAGESCALER_CID,
        FLOCK_IMAGESCALER_CONTRACTID,
        flockImageScalerImplConstructor,
    },
    
    {
        FLOCK_ASYNCIMAGESCALER_CLASSNAME,
        FLOCK_ASYNCIMAGESCALER_CID,
        FLOCK_ASYNCIMAGESCALER_CONTRACTID,
        flockAsyncImageScalerConstructor,
    },

    {
        FLOCK_ASYNCIMAGESCALER_IMPL_CLASSNAME,
        FLOCK_ASYNCIMAGESCALER_IMPL_CID,
        FLOCK_ASYNCIMAGESCALER_IMPL_CONTRACTID,
        flockAsyncImageScalerImplConstructor,
    },

    {
        FLOCK_IMAGETRANSFORMSERVICE_CLASSNAME, 
        FLOCK_IMAGETRANSFORMSERVICE_CID,
        FLOCK_IMAGETRANSFORMSERVICE_CONTRACTID,
        flockImageTransformServiceConstructor,
        flockImageTransformServiceRegister,
        flockImageTransformServiceUnregister,
    },

    {0}
};

NS_IMPL_NSGETMODULE(flockImageScalerModule, components)
