// vim: ts=2 sw=2 expandtab cindent
//
// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const CC = Components.classes;
const CI = Components.interfaces;
const CR = Components.results;
const CU = Components.utils;

CU.import("resource:///modules/FlockXPCOMUtils.jsm");
CU.import("resource:///modules/FlockSvcUtils.jsm");
FlockXPCOMUtils.debug = false;

const MODULE_NAME = "MySpace";
const CLASS_NAME = "MySpace";
const CLASS_SHORT_NAME = "myspace";
const CLASS_TITLE = "MySpace";
const CLASS_ID = Components.ID("{18440fbe-2f8c-4354-b4fb-dee7a009124b}");
const CONTRACT_ID = "@flock.com/people/myspace;1";

const SERVICE_ENABLED_PREF = "flock.service.myspace.enabled";

const FAVICON = "chrome://flock/content/services/myspace/favicon.png";


/*************************************************************************
 * Component: flockMyspaceService
 *************************************************************************/
function flockMyspaceService() {
  this._init();

  FlockSvcUtils.flockIWebService.addDefaultMethod(this, "getAccount");
  FlockSvcUtils.flockIWebService.addDefaultMethod(this, "isHidden");
  FlockSvcUtils.flockIWebService.addDefaultMethod(this, "getAuthenticatedAccount");
  FlockSvcUtils.flockIWebService.addDefaultMethod(this, "getAccounts");
  FlockSvcUtils.flockIWebService.addDefaultMethod(this, "logout");
  FlockSvcUtils.flockIWebService.addDefaultMethod(this, "getStringBundle");

  FlockSvcUtils.flockIManageableWebService
               .addDefaultMethod(this, "docRepresentsSuccessfulLogin");
  FlockSvcUtils.flockIManageableWebService
               .addDefaultMethod(this, "getAccountIDFromDocument");
  FlockSvcUtils.flockIManageableWebService
               .addDefaultMethod(this, "getCredentialsFromForm");
  FlockSvcUtils.flockIManageableWebService
               .addDefaultMethod(this, "ownsDocument");
  FlockSvcUtils.flockIManageableWebService
               .addDefaultMethod(this, "ownsLoginForm");
  FlockSvcUtils.flockIManageableWebService
               .addDefaultMethod(this, "getSessionValue");
}


/*************************************************************************
 * XPCOM Component Creation
 *************************************************************************/

flockMyspaceService.prototype = new FlockXPCOMUtils.genericComponent(
  CLASS_NAME,
  CLASS_ID,
  CONTRACT_ID,
  flockMyspaceService,
  CI.nsIClassInfo.SINGLETON,
  [
    CI.nsIObserver,
    //CI.nsITimerCallback,
    CI.flockIWebService,
    CI.flockIManageableWebService,
    CI.flockIPollingService,
    //CI.flockIBlogWebService,
    CI.flockISocialWebService
    //CI.flockIMigratable
  ]
);

// FlockXPCOMUtils.genericModule() categories
flockMyspaceService.prototype._xpcom_categories = [
  { category: "wsm-startup" },
  //{ category: "flockMigratable" },
  { category: "flockWebService", entry: CLASS_SHORT_NAME }
];

/*************************************************************************
 * Private Data and Functions
 *************************************************************************/

// Member variables.
flockMyspaceService.prototype._init =
function fms_init() {
  FlockSvcUtils.getLogger(this);
  this._logger.info(".init()");

  // Determine whether this service has been disabled, and unregister if so.
  var prefService = CC["@mozilla.org/preferences-service;1"]
                    .getService(CI.nsIPrefBranch);
  if (prefService.getPrefType(SERVICE_ENABLED_PREF) &&
     !prefService.getBoolPref(SERVICE_ENABLED_PREF))
  {
    this._logger.info(SERVICE_ENABLED_PREF + " is FALSE! Not initializing.");
    this.deleteCategories();
    return;
  }

  var profiler = CC["@flock.com/profiler;1"].getService(CI.flockIProfiler);
  var evtID = profiler.profileEventStart("myspace-init");

  var obs = CC["@mozilla.org/observer-service;1"]
            .getService(CI.nsIObserverService);
  obs.addObserver(this, "xpcom-shutdown", false);

  this._accountClassCtor = flockMyspaceAccount;
  FlockSvcUtils.getCoop(this);

  this._baseUrn = "urn:myspace";
  this._serviceUrn = this._baseUrn + ":service";

  if (this._coop.Service.exists(this._serviceUrn)) {
    this._c_svc = this._coop.get(this._serviceUrn);
  } else {
    this._c_svc = new this._coop.Service(this._serviceUrn, {
      name: CLASS_SHORT_NAME,
      desc: CLASS_TITLE
    });
  }
  this._c_svc.serviceId = CONTRACT_ID;
  this._c_svc.logoutOption = true;
  this._c_svc.domains = FlockSvcUtils.getWD(this)
                        .getString("myspace", "domains", "myspace.com");

  // Initialize member that specifies path for localized string bundle
  this._stringBundlePath = "";

  profiler.profileEventEnd(evtID, "");
};


/**
 * Private helper to insert a user id in an URL from web detective.
 */
flockMyspaceService.prototype._makeMyspaceUrl =
function fms_makeMyspaceUrl(aUrlType, aUserId) {
  return this._WebDetective.getString(CLASS_SHORT_NAME, aUrlType, null)
             .replace("%friendid%", aUserId);
};


/*************************************************************************
 * flockMyspaceService: flockIWebService Implementation
 *************************************************************************/

// readonly attribute AString urn;
flockMyspaceService.prototype.urn = "urn:" + CLASS_SHORT_NAME + ":service";

// readonly attribute AString title;
flockMyspaceService.prototype.title = CLASS_TITLE;

// readonly attribute AString icon;
flockMyspaceService.prototype.icon = FAVICON;

// readonly attribute AString url;
flockMyspaceService.prototype.url = "http://www.purple.com";

// readonly attribute AString contractId;
// ALMOST duplicated by nsIClassInfo::contractID
// with different case: contractId != contractID
flockMyspaceService.prototype.contractId = CONTRACT_ID;

// readonly attribute AString shortName;
flockMyspaceService.prototype.shortName = CLASS_SHORT_NAME;


/**
 * @see flockIWebService#addAccount
 */
flockMyspaceService.prototype.addAccount =
function fms_addAccount(aAccountId, aIsTransient, aListener) {
  this._logger.debug(".addAccount('"
                     + aAccountId + "', "
                     + aIsTransient + ", "
                     + aListener + ")");

  var pw = this._acUtils.getPassword(this.urn + ":" + aAccountId);
  var name = (pw) ? pw.user : aAccountId;

  var accountUrn = "urn:flock:" + this.shortName + ":account:" + aAccountId;
  var account = new this._coop.Account(
    accountUrn, {
      name: name,
      serviceId: this.contractId,
      service: this._c_svc,
      accountId: aAccountId,
      isPollable: false,
      isTransient: aIsTransient,
      URL: this._makeMyspaceUrl("profileURL", aAccountId),
      favicon: FAVICON
    });
  this._coop.accounts_root.children.add(account);

  var friendsListUrn = accountUrn + ":friends";
  var friendsList = new this._coop.FriendsList(
    friendsListUrn,
    {
      account: account
    });
  account.friendsList = friendsList;

  // Instantiate account component
  var acct = this.getAccount(account.id());
  if (aListener) {
    aListener.onSuccess(acct, "addAccount");
  }
  return acct;
};


/*************************************************************************
 * flockMyspaceService: flockIManageableWebService Implementation
 *************************************************************************/

// DEFAULT: boolean docRepresentsSuccessfulLogin(in nsIDOMHTMLDocument aDocument);
// DEFAULT: AString getAccountIDFromDocument(in nsIDOMHTMLDocument aDocument);
// DEFAULT: nsIPassword getCredentialsFromForm(in nsIDOMHTMLFormElement aForm);
// DEFAULT: boolean ownsDocument(in nsIDOMHTMLDocument aDocument);
// DEFAULT: boolean ownsLoginForm(in nsIDOMHTMLFormElement aForm);

/**
 * void updateAccountStatusFromDocument(in nsIDOMHTMLDocument aDocument);
 *
 * @see flockIManageableWebService#updateAccountStatusFromDocument
 */
flockMyspaceService.prototype.updateAccountStatusFromDocument =
function fms_updateAccountStatusFromDocument(aDocument, aAcctURN) {
  this._logger.debug("updateAccountStatusFromDocument('" + aAcctURN + "')");
  if (aAcctURN) {
    // We're logged in to this account
    var c_acct = this._coop.get(aAcctURN);
    if (!c_acct.isAuthenticated) {
      var inst = this;
      var loginListener = {
        onSuccess: function loginListener_onSuccess() {
          inst._logger.debug(".updateAccountStatusFromDocument(): "
                             + "loginListener: onSuccess()");
          c_acct.isAuthenticated = true;
        },
        onError: function loginListener_onError(aError) {
          inst._logger.debug(".updateAccountStatusFromDocument(): "
                             + "loginListener: onError()");
          inst._logger.debug(aError ? (aError.errorString) : "No details");
          c_acct.isAuthenticated = false;
        }
      };
      this.getAccount(aAcctURN).login(loginListener);
    }
  } else if (this._WebDetective
                 .detect(this.shortName, "loggedout", aDocument, null))
  {
    // We're logged out (of all accounts)
    this._acUtils.markAllAccountsAsLoggedOut(this.contractId);
  }
};


/*************************************************************************
 * flockMyspaceService: flockIPollingService Implementation
 *************************************************************************/
/**
 * @see flockIPollingService#refresh
 */
flockMyspaceService.prototype.refresh =
function flockMyspaceService_refresh(aUrn, aListener) {
  this._logger.debug(".refresh(" + aUrn + ")");
  var refreshItem = this._coop.get(aUrn);

// XXX: short circuit this for now while testing.
aListener.onResult();
/*
  if (refreshItem instanceof this._coop.Account) {
    this._logger.debug("refreshing an Account");
    if (refreshItem.isAuthenticated) {
      this._refreshAccount(aUrn, aListener);
    } else {
      // If the user is not logged in, return a success without
      // refreshing anything
      aListener.onResult();
    }
  } else if (refreshItem instanceof this._coop.Identity) {
    this._logger.debug("refreshing an Identity");
    aListener.onResult();
  } else {
    throw CR.NS_ERROR_ABORT;
    this._logger.error("can't refresh " + aUrn + " (unsupported type)");
    aListener.onError(null);
  }
*/
};


/**************************************************************************
 * flockMyspaceService: nsIObserver Implementation
 **************************************************************************/

flockMyspaceService.prototype.observe =
function flockMyspaceService_observe(aSubject, aTopic, aState) {
  switch (aTopic) {
    case "xpcom-shutdown":
      var obs = CC["@mozilla.org/observer-service;1"]
                .getService(CI.nsIObserverService);
      obs.removeObserver(this, "xpcom-shutdown");
      break;
  }
};


/*************************************************************************
 * Component: flockMyspaceAccount
 *************************************************************************/

function flockMyspaceAccount() {
  FlockSvcUtils.getLogger(this);
  this._logger.init("myspaceAccount");

  FlockSvcUtils.getACUtils(this);
  FlockSvcUtils.getCoop(this);

  var wsa = FlockSvcUtils.flockIWebServiceAccount;
  wsa.addDefaultMethod(this, "deactivate");
  wsa.addDefaultMethod(this, "getService");
  wsa.addDefaultMethod(this, "login");
  wsa.addDefaultMethod(this, "logout");
  wsa.addDefaultMethod(this, "getParam");
  wsa.addDefaultMethod(this, "setParam");
  wsa.addDefaultMethod(this, "getCustomParam");
  wsa.addDefaultMethod(this, "getAllCustomParams");
  wsa.addDefaultMethod(this, "setCustomParam");
  wsa.addDefaultMethod(this, "isAuthenticated");
}

/*************************************************************************
 * flockMyspaceAccount: XPCOM Component Creation
 *************************************************************************/

// Use genericComponent() for the goodness it provides (QI, nsIClassInfo, etc),
// but do NOT add flockMyspaceAccount to the list of constructors passed
// to FlockXPCOMUtils.genericModule().
flockMyspaceAccount.prototype = new FlockXPCOMUtils.genericComponent(
  CLASS_NAME + " Account",
  "",
  "",
  flockMyspaceAccount,
  0,
  [
    CI.flockIWebServiceAccount,
    //CI.flockIBlogWebServiceAccount,
    CI.flockISocialWebServiceAccount
  ]
  );

/*************************************************************************
 * flockMyspaceAccount: flockIWebServiceAccount Implementation
 *************************************************************************/

// readonly attribute AString urn;
flockMyspaceAccount.prototype.urn = "";

// void activate(in flockIListener aListener);
flockMyspaceAccount.prototype.activate =
function fma_activate(aListener) {
  this._logger.debug(".activate()");
  var acctCoopObj = this._coop.get(this.urn);
  acctCoopObj.isAuthenticated = true;
  acctCoopObj.isPollable = true;
  this.keep(this.urn);

  if (aListener) {
    aListener.onSuccess(this, "activate");
  }
};

// DEFAULT: void deactivate(in flockIListener aListener);
// DEFAULT: void login(in flockIListener aListener);
// DEFAULT: void logout(in flockIListener aListener);
/*
flockMyspaceAccount.prototype.login =
function fma_login(aListener) {
  this._logger.debug(".login()");
  if (aListener) {
    aListener.onSuccess(this, "login");
  }
  var acctCoopObj = this._coop.get(this.urn);
  var flockHttpRequest = loadSubScript('chrome://flock/content/common/httprequest.js').flockHttpRequest;
  var hr = new flockHttpRequest("POST", gStrings["autologinURL"]);
  if (aListener) {
    hr.oncomplete = function (res) {
      aListener.onSuccess(this, "login");
    }
  }
  var passObj = this.acUtils.getPassword(this.service.urn+':'+acctCoopObj.accountId);
  if (passObj && passObj.user) {
    var postBody = "email="+encodeURIComponent(passObj.user)
                 + "&password="+encodeURIComponent(passObj.password);
    hr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded", false);
    hr.send(postBody);
  } else {
    if (aListener) aListener.onError();
  }
*/

// DEFAULT: void logout(in flockIListener aListener);
/*
flockMyspaceAccount.prototype.logout =
function fma_logout(aListener) {
  this._logger.debug(".logout()");
  var acctCoopObj = this._coop.get(this.urn);
  if (acctCoopObj.isAuthenticated) {
    this.acUtils.removeCookies(this.webDetective.getSessionCookies("myspace"));
    acctCoopObj.isAuthenticated = false;
  }

  if (aListener) {
    aListener.onSuccess(this, "logout");
  }
}
*/

// void keep();
flockMyspaceAccount.prototype.keep =
function fma_keep() {
  this._logger.debug(".keep()");
  this._coop.get(this.urn).isTransient = false;
  this._acUtils.makeTempPasswordPermanent(this.urn.replace("account:", ""));
};


/*************************************************************************
 * flockMyspaceAccount: flockIBlogWebServiceAccount Implementation
 *************************************************************************/

/*
flockMyspaceAccount.prototype.getBlogs =
function fma_getBlogs() {
  this._logger.debug(".getBlogs()");

  var blogsEnum = {
    QueryInterface: function QI(aIid) {
      if (!aIid.equals(CI.nsISupports) &&
          !aIid.equals(CI.nsISimpleEnumerator))
      {
        throw CR.NS_ERROR_NO_INTERFACE;
      }
      return this;
    },
    hasMoreElements: function hasMoreElements() {
      return false;
    },
    getNext: function getNext() {
    }
  };

  return blogsEnum;
}
*/


/*************************************************************************
 * flockMyspaceAccount: flockISocialWebServiceAccount Implementation
 *************************************************************************/
// readonly attribute boolean hasFriendActions;
flockMyspaceAccount.prototype.hasFriendActions = true;

// readonly attribute boolean isStatusSupported;
flockMyspaceAccount.prototype.isStatusSupported = false;

// readonly attribute boolean isStatusEditable;
flockMyspaceAccount.prototype.isStatusEditable = false;

// void setStatus(in AString aStatusMessage, in flockIListener aListener);
flockMyspaceAccount.prototype.setStatus =
function fma_setStatus(aStatusMessage, aListener) {
  this._logger.debug(".setStatus(" + aStatusMessage + ")");
  // Myspace doesn't support this, so we just happily return success.
  aListener.onSuccess();
};

// AString getEditableStatus();
flockMyspaceAccount.prototype.getEditableStatus =
function fma_getEditableStatus() {
  this._logger.debug(".getEditableStatus()");
  // Myspace doesn't support this, so we just happily return an empty string.
  return "";
};

// AString formatStatusForDisplay();
flockMyspaceAccount.prototype.formatStatusForDisplay =
function fma_formatStatusForDisplay(aStatusMessage) {
  this._logger.debug(".formatStatusForDisplay()");
  // Myspace doesn't support this, so we just happily return an empty string.
  return "";
};

// AString getMeNotifications();
flockMyspaceAccount.prototype.getMeNotifications =
function fma_getMeNotifications() {
  return "{}";
};

// AString getFriendActions(in AString aFriendUrn);
flockMyspaceAccount.prototype.getFriendActions =
function fma_getFriendActions(aFriendUrn) {
  return "{}";
};

// AString getSharingAction(in AString aFriendUrn,
//                          in nsITransferable aTransferable);
flockMyspaceAccount.prototype.getSharingAction =
function fma_getSharingAction(aFriendUrn, aTransferable) {
  return "{}";
};


/*************************************************************************
 * XPCOM Support - Module Construction
 *************************************************************************/

// Create array of components.
var gComponentsArray = [flockMyspaceService];

// Generate a module for XPCOM to find.
var NSGetModule = FlockXPCOMUtils.generateNSGetModule(MODULE_NAME,
                                                      gComponentsArray);
