// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

var _logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
_logger.init("myworld");

var _coop = Cc["@flock.com/singleton;1"]
            .getService(Ci.flockISingleton)
            .getSingleton("chrome://flock/content/common/load-faves-coop.js")
            .wrappedJSObject;

const BLOG_SVC = Cc["@flock.com/flock-blog;1"].getService(Ci.flockIBlogService);

const PREFS = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
const PREF_BROWSER_STARTUP_HOMEPAGE = "browser.startup.homepage";
const PREF_MYWORLD_CURRENT_ENGINE   = "flock.myworld.currentEngine";
const PREF_MYWORLD_TITLE            = "flock.myworld.currentTitle";
const XMLHTTPREQUEST_CONTRACTID     = "@mozilla.org/xmlextras/xmlhttprequest;1";
const nsIXMLHttpRequest             = Components.interfaces.nsIXMLHttpRequest;

const MYWORLD_SERVICE_CONTRACTID    = "@flock.com/myworld-service;1";
const MYWORLD_TITLE_FONTSCALE_LC = 13;
const MYWORLD_TITLE_FONTSCALE_UC = 19;

const nsIPrefLocalizedString = Components.interfaces.nsIPrefLocalizedString;

var gMyworldTitleMinWidth;
var gMyworldWidgets = [];
var gMyworldWidgetsVisible = 0;

var currentEng;

var gObsrvr = null;

function onUnload() {
  this.gMetrics.report("MyWorld-Close", null);

  this.os.removeObserver(gObsrvr, "browser-search-engine-modified");
  this.removeWidgetObservers();
}

function onLoad()
{
  _logger.debug("onLoad");

  document.getElementById("focusOnLoad").select();

  this.gMetrics.report("MyWorld-Opened", null);
  
  checkDefaultHomepage();  

  loadEngines();

  loadMyWorldTitle();

  // This is a nescessary hack, please see
  // https://bugzilla.mozilla.org/show_bug.cgi?id=196057
  window.onload = null;
}

function checkDefaultHomepage() {
   var defhbox = document.getElementById("makedefault");

   // Remove to re-build
   while(defhbox.firstChild) {
      defhbox.removeChild(defhbox.firstChild);
   }

   // check if flock is in the default
   if(PREFS.getPrefType(PREF_BROWSER_STARTUP_HOMEPAGE)) {
      var homepage = PREFS.getComplexValue(PREF_BROWSER_STARTUP_HOMEPAGE, nsIPrefLocalizedString).data;
      if(homepage.indexOf("about:myworld") >= 0) {
        return; 
      }
   }
   
   // Add to page if nescessary
   var button = document.createElement("button");
   button.setAttribute("onclick", "makeDefaultHomepage()");
   button.setAttribute("class", "defaultHomepageBtn");
   
   var label = document.createElement("label");
   label.setAttribute("class", "myworldHomepage");
   label.setAttribute("value", flockGetString("start/myworld", "flock.myworld.homepage.set"));
   
   button.appendChild(label);
   defhbox.appendChild(button);
}

function loadEngines() {
  // Get a list of all search engines
  var searchService = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
  if (!searchService) return;

  var engines = searchService.getVisibleEngines({ });

  var eMenu = document.getElementById("engineMenuID");

  // delete list before re-creating it
  var existingChildrenCount = eMenu.childNodes.length;
  for(var z=existingChildrenCount;z>0;z--)
  {
    var existChild = eMenu.childNodes[z-1];
    eMenu.removeChild(existChild);
  }

  var currentEngine;
  // Get current myworld engine
  if (PREFS.getPrefType(PREF_MYWORLD_CURRENT_ENGINE)) {
    currentEngine = PREFS.getCharPref(PREF_MYWORLD_CURRENT_ENGINE);
  }

  if (!currentEngine || !searchService.getEngineByName(currentEngine)) {
    // If myworld engine doesnt exist, use current default engine
    if (!searchService.currentEngine) return;
    currentEngine = searchService.currentEngine.name;
  }

  PREFS.setCharPref(PREF_MYWORLD_CURRENT_ENGINE, currentEngine);

    // Add them to the menu
  for(var i=engines.length-1;i>=0;--i) {
    var menuitem = document.createElementNS(kXULNS, "menuitem");
    menuitem.setAttribute("label", engines[i].name);
    menuitem.setAttribute("id", engines[i].name);
    menuitem.setAttribute("class", "engine-menuitem");
    menuitem.setAttribute("engine", engines[i].name);
    menuitem.setAttribute("image", engines[i].iconURI);
    menuitem.setAttribute("oncommand", "selectEngine(this)");

    if(engines[i].name == currentEngine) {
      menuitem.setAttribute("selected", "true");
      currentEng = engines[i];
    }

    if(engines[i].iconURI) {
      menuitem.setAttribute("image", engines[i].iconURI.spec);

      // Also set the image of the selected item
      if(engines[i].name == currentEngine) {
        var engineImageID = document.getElementById("engineImageID");
        engineImageID.setAttribute("src",engines[i].iconURI.spec);
      }
    }

    eMenu.insertBefore(menuitem, eMenu.firstChild);
    menuitem.engine = engines[i];
  }
}

function toggleEngineMenu() {
  var popup = document.getElementById("engineMenuID");
  var popButton = document.getElementById("popButton");

  popup.showPopup(popButton, -1, -1, "popup", "bottomleft", "topleft");
}

function selectEngine(engine) {
  var engineName  = engine.getAttribute("engine");
  var engineImage = engine.getAttribute("image");
  this.gMetrics.report("MyWorld-SearchEngineChange", engineName);

  if(engineImage != "" && engineImage != null) {
    var engineImageID = document.getElementById("engineImageID");
    engineImageID.setAttribute("src",engineImage);
  }

  // Set current engine
  var searchService = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
  var engines = searchService.getVisibleEngines({ });

  for(var i=engines.length-1;i>=0;--i) {
    if(engines[i].name == engineName) {
      currentEng = engines[i];
      PREFS.setCharPref(PREF_MYWORLD_CURRENT_ENGINE, currentEng.name);
      return;
    }
  }
}

function openService(node, event)
{
  if(event.button == 2) {
    return;
  }

  // Open the service URL in a new tab
  var svcName = node.getAttribute("serviceName");
  this.gMetrics.report("MyWorld-ServiceClick", svcName);
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    win.openUILinkIn("http://redir.flock.com/redir?component=browser&action=myworld_click&partner=" + svcName, "tab");
  }

  // Tell the Web Service Manager to automatically keep any accounts
  // created for this service in the next while...
  var wsm = Cc["@flock.com/webservice-manager;1"].getService(Ci.flockIWebServiceManager);
  wsm.autoKeepAccountForService(node.getAttribute("urn"));
}

function refreshDateTime()
{
  var timeLabel = document.getElementById("timeUpdated");

  var time = new Date();

  var dtFormat = flockGetString("start/myworld", "flock.myworld.dateTimeFormat");

  // see bug 9830#31
  dtFormat = dtFormat.replace("%e", time.getDate());

  var thedate = time.toLocaleFormat(dtFormat);

  timeLabel.setAttribute("value", thedate);

  this.os.notifyObservers(null, "refresh-myworld-date", null);
  setTimeout("refreshDateTime()", (60 - time.getSeconds() + 1) * 1000);
}

function uploadPhotos(event)
{
  if(event.button == 2) {
    return; 
  }

  this.gMetrics.report("Uploader-Open", "MyWorldUploadButton");
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    win.flock_photo.util.launchUploader();
  }
}

function createBlogPost(event)
{
  if(event.button == 2) {
    return; 
  }
  
  this.gMetrics.report("BlogDialog-Open", "MyWorldBlogButton");
  BLOG_SVC.openEditor("","","");
}

function searchboxKeyPress(event,query) {
  if (event.keyCode == KeyEvent.DOM_VK_RETURN) {
    searchFunction(query);
  }
}

function searchFunction(query)
{
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    var ss = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
    var submission = currentEng.getSubmission(query, null); // HTML response
    if (!submission) { return; }

    this.gMetrics.report("MyWorld-Search", currentEng.name);
    var submissionURL = tweakAffiliateCode(submission.uri);
    win.openUILinkIn(submissionURL, "tab");
  }
}

function tweakAffiliateCode(aSearchURI)
{
  var spec = aSearchURI.spec;
  // If this is a Yahoo! search, we want to tweak the affiliate search code so
  // we know that this search is being done from My World
  if (aSearchURI.host == "search.yahoo.com") {
    // This particular regex trick relies on the fact that My World types are
    // the same as the chrome types, except My World types have a "w" appended
    // to the type's letters.  (ex: fl1.0 becomes flw1.0)
    spec = spec.replace(/&type=([a-z]+)?1\.1/, "&type=$1w1.1");
  }
  return spec;
}

function makeDefaultHomepage() {
  // Make My World the default homepage
  PREFS.setCharPref(PREF_BROWSER_STARTUP_HOMEPAGE, "about:myworld");

  checkDefaultHomepage();

  var notificationBox = getNotificationBox();
  if(notificationBox) {
    var message = flockGetString("start/myworld", "flock.myworld.homepage");
    notificationBox.appendNotification( message,
                                        "myworldadded",
                                        "chrome://browser/skin/Info.png", 
                                        notificationBox.FLOCK_PRIORITY_MEDIUM,
                                        null,
                                        null);
  
  }
}

function getNotificationBox() {

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    var gBrowser = win.getBrowser();
    var notificationBox = gBrowser.getNotificationBox();

    return notificationBox;
  }

  return null;
}

function initWidgetObservers() {
  // Add observer for refresh
  var inst = this;
  if (!gObsrvr) {
    gObsrvr = {};
    gObsrvr.observe = function engineManager_observer(aEngine, aTopic, aVerb) {
      if(aTopic == "browser-search-engine-modified") {
        aEngine.QueryInterface(Ci.nsISearchEngine);
        switch (aVerb) {
          case "engine-added":
          case "engine-removed":
            setTimeout("loadEngines()", 1000);
        }
      }
      //for each widget, an observer is added with the form
      //refresh-myworld-{widget.label}
      if (aTopic.match(/^refresh-myworld/)) {
        _logger.debug("resizeWidgetContainer");
        inst.resizeWidgetContainer();
      }
    }
    // Add observer for engines added
    this.os = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
    this.os.addObserver(gObsrvr, "browser-search-engine-modified", false);
  
    setTimeout("refreshDateTime()", 500);

    //resize window to account for iframe content
    setTimeout("resizeWidgetContainer()", 500);
  }

  //add observers for widgets
  for each (var column in gMyworldWidgets) {
    this.os.addObserver(gObsrvr, "refresh-myworld-" + column.label, false);
  }
}

function removeWidgetObservers() {
  //remove observers for widgets
  for each (var column in gMyworldWidgets) {
    this.os.removeObserver(gObsrvr, "refresh-myworld-" + column.label);
  }
}

/**
 * Loads column layout
 */
function loadColumnLayout() {
  var myworldService = Cc[MYWORLD_SERVICE_CONTRACTID]
                       .getService(Ci.flockIMyworldService);

  var layoutObj = JSON.fromString(myworldService.getColumnLayout());
  gMyworldWidgets = eval(layoutObj);
  //add additional observers for each widget
  initWidgetObservers();
    
  var containerElement = document.getElementById("ftcontainerID");

  //clean up existing dom within container if exists
  while (containerElement.hasChildNodes()) {
    containerElement.removeChild(containerElement.lastChild);
  }

  var colIndex = 0;

  //call recurring localized variables only once
  var whatsThisStr = flockGetString("start/myworld",
                                    "flock.myworld.whatsThis");
  var moveRightStr = flockGetString("start/myworld",
                                    "flock.myworld.widgets.moveRight");
  var moveLeftStr = flockGetString("start/myworld",
                                   "flock.myworld.widgets.moveLeft");
  var closeTooltip = flockGetString("start/myworld",
                                    "flock.myworld.widgets.close");
  var moveTooltip = flockGetString("start/myworld",
                                   "flock.myworld.widgets.moveOptions");

  var showCount = 0;

  var leftMostWidget = null;
  var rightMostWidgetBtn = null;

  for each (var column in gMyworldWidgets) {
    var col_index = colIndex++;

    //iframe content for respective xul page
    var iframe = document.createElement("iframe");
    iframe.setAttribute("flex", "1");
    iframe.setAttribute("src", column.url);

    //seperator
    var columnEnclosure = document.createElement("vbox");
    columnEnclosure.setAttribute("class", "col_enclosure");

    var widgetId = column.label + "Id";
    //widget container
    var widgetVbox = document.createElement("vbox");
    widgetVbox.setAttribute("label_val", column.label);
    widgetVbox.setAttribute("iframe_src", column.url);
    widgetVbox.setAttribute("widgetContainer", "true");
    widgetVbox.setAttribute("col_index", col_index);
    widgetVbox.setAttribute("id", widgetId);
    widgetVbox.setAttribute("class", "col_hdr_container");
    widgetVbox.setAttribute("hidden", "false");
    if (!eval(column.show)) {
      widgetVbox.setAttribute("hidden", "true");
    } else {
      showCount++;
    }

    var hboxContainer = document.createElement("hbox");
    var vboxContainer = document.createElement("vbox");
    hboxContainer.appendChild(vboxContainer);

    //widget title
    var titleHbox = document.createElement("hbox");
    titleHbox.setAttribute("align", "start");
    var title = document.createElement("label");
    title.setAttribute("class", "favHeaderLabel");
    title.setAttribute("onclick", this.doWidgetAction(column.label));
    var titleStr = flockGetString("start/myworld",
                                  "flock.myworld." + column.label + ".title");
    title.setAttribute("value", titleStr);
    title.setAttribute("tooltiptext", titleStr);
    var menupopup = document.createElement("menupopup");
    menupopup.setAttribute("id", column.label + "ID");
    menupopup.setAttribute("class", "whatsThisPopup");
    menupopup.setAttribute("position", "after_start");

    titleHbox.appendChild(title);
    titleHbox.appendChild(menupopup);
    vboxContainer.appendChild(titleHbox);

    //widget sub title and help image
    var subHbox = document.createElement("hbox");
    subHbox.setAttribute("class", "col_hdr_bottom");
    var image = document.createElement("image");
    image.setAttribute("class", "whatsThis");
    image.setAttribute("descID", column.label + "ID");
    var tooltip = flockGetString("start/myworld",
                                 "flock.myworld." + column.label + ".tooltip");
    image.setAttribute("desc", tooltip);
    image.setAttribute("value", whatsThisStr);
    image.setAttribute("onclick", "popupWhatsThis(this)");
    var subTitle = document.createElement("label");
    subTitle.setAttribute("class", "col_sub_hdr");
    var subTitleStr = flockGetString("start/myworld",
                                     "flock.myworld." + 
                                      column.label + ".subTitle");
    subTitle.setAttribute("value", subTitleStr);
    subTitle.setAttribute("onclick",
                          "popupWhatsThis(this.parentNode.childNodes[0])");

    subHbox.appendChild(image);
    subHbox.appendChild(subTitle);
    vboxContainer.appendChild(subHbox);

    //widget settings
    var widgetSettings = document.createElement("vbox");
    widgetSettings.setAttribute("class", "col_hdr_settings");
    var widgetSettingsHbox = document.createElement("hbox");
    widgetSettings.appendChild(widgetSettingsHbox);

    //create move button
    var moveLeftBtn = document.createElement("toolbarbutton");
    moveLeftBtn.setAttribute("class", "widgetMoveBtn-left");
    moveLeftBtn.setAttribute("tooltiptext", moveTooltip);
    moveLeftBtn.setAttribute("oncommand", "moveColumn('" + widgetId + "'," +
                                          "'left');");
    moveLeftBtn.setAttribute("id", widgetId+"-moveLeft");

    // Check to see if the left button should be hidden
    // We need to ensure it's the left most *visible* widget.
    if (leftMostWidget == null
        && widgetVbox.getAttribute("hidden") == "false") {
      leftMostWidget = widgetVbox;
      moveLeftBtn.setAttribute("hidden", "true");
    }

    var moveRightBtn = document.createElement("toolbarbutton");
    moveRightBtn.setAttribute("class", "widgetMoveBtn-right");
    moveRightBtn.setAttribute("tooltiptext", moveTooltip);
    moveRightBtn.setAttribute("oncommand", "moveColumn('" + widgetId + "'," +
                                           "'right');");
    moveRightBtn.setAttribute("id", widgetId + "-moveRight");

    // We need to ensure it's the right most *visible* widget.
    // Track it here, hide it at the very end.
    if (widgetVbox.getAttribute("hidden") == "false") {
      rightMostWidgetBtn = moveRightBtn;
    }

    //attach move button
    widgetSettingsHbox.appendChild(moveLeftBtn);
    widgetSettingsHbox.appendChild(moveRightBtn);

    //close button
    var closeBtn = document.createElement("toolbarbutton");
    var metricVal = "Components.classes['@flock.com/metrics-service;1']"
                  + ".getService(Components.interfaces.flockIMetricsService)"
                  + ".report('MyWorld-HideWidget', 'close');";
    closeBtn.setAttribute("oncommand", "closeWidget('" + widgetId + "');"
                                       + metricVal);
    closeBtn.setAttribute("class", "widgetCloseBtn");
    closeBtn.setAttribute("tooltiptext", closeTooltip);
    widgetSettingsHbox.appendChild(closeBtn);

    //append elements
    hboxContainer.appendChild(widgetSettings);
    widgetVbox.appendChild(hboxContainer);
    widgetVbox.appendChild(iframe);
    containerElement.appendChild(widgetVbox);
    //containerElement.appendChild(columnEnclosure);

    //create menuitem for widget entry
    var widgetMenuPopup = document.getElementById("widgets-selector");
    var widgetMenuItem = document.createElement("menuitem");
    widgetMenuItem.setAttribute("label", titleStr);
    widgetMenuItem.setAttribute("id", widgetId + "-toggleItem");
    widgetMenuItem.setAttribute("class", "menuitem-iconic");
    widgetMenuItem.setAttribute("type", "checkbox");
    if (eval(column.show)) {
      widgetMenuItem.setAttribute("checked", "true");
    }
    widgetMenuItem.setAttribute("oncommand", "doWidgetToggle('" + widgetId +
                                                             "', this);");
    
    widgetMenuPopup.appendChild(widgetMenuItem);
  }

  // Hide the right-most "move right" button.
  if (rightMostWidgetBtn) {
    rightMostWidgetBtn.setAttribute("hidden", "true");
  }

  checkDiscoveryBox(showCount);
}

var gTimer = null;
gTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
gTimer.initWithCallback({
  notify: function myworld_loadColumnLayoutTimer() {
    loadColumnLayout();
  }
}, 0, Ci.nsITimer.TYPE_ONE_SHOT);

/**
 * Saves column layout
 */
function saveColumnLayout() {
  var myworldService = Cc[MYWORLD_SERVICE_CONTRACTID]
                       .getService(Ci.flockIMyworldService);
  var oldLayout = myworldService.getColumnLayout();
  var newLayout = JSON.toString(gMyworldWidgets);

  // Extra safty precaution to make sure we don't report a layout change
  // if in fact none happened
  if (oldLayout != newLayout) {
    var metricsValue = [];
    for (var i = 0; i < gMyworldWidgets.length; i++) {
      metricsValue.push({name: gMyworldWidgets[i].label,
                         position: gMyworldWidgets[i].index,
                         visible: (gMyworldWidgets[i].show == "true")});
    }
    // .report expects the value to be in an [ ] if it is a complex object,
    // even if it's already an [ ].
    gMetrics.report("MyWorld-LayoutChange", [metricsValue]);
  }

  myworldService.setColumnLayout(newLayout);
}

/**
 * Moves column left/right
 * aDirection = left/right
 */
function moveColumn(widgetId, aDirection) {
  var moveTarget = document.getElementById(widgetId);
  var parent = moveTarget.parentNode;
  var aColumn = moveTarget.getAttribute("col_index");

  switch (aDirection) {
    case "left": {
      if (aColumn == 0) {
        // We can't go left anymore
        return false;
      }
      if (moveTarget.previousSibling) {
        var prevSibling = moveTarget.previousSibling;
        parent.removeChild(moveTarget);
        parent.insertBefore(moveTarget, prevSibling);
      }
    } break;
    case "right": {
      if (gMyworldWidgets.length == aColumn) {
        return false;
      }
      if (moveTarget.nextSibling) {
        var nextSibling = moveTarget.nextSibling;
        parent.removeChild(moveTarget);
        parent.insertBefore(moveTarget, nextSibling.nextSibling);
      }
    } break;
    default: {
      // Not used yet or invalid choice
      return false;
    }
  }
  this.updateCurrentColumnLayout();
  return true;
}

function updateCurrentColumnLayout() {
  var columns = [];
  var containerElement = document.getElementById("ftcontainerID");
  var indexCount = 0;
  var showCount = 0;
  var showVal = true;
  for (var i = 0;i < containerElement.childNodes.length; i++) {
    var child = containerElement.childNodes[i];
    //add only widget elements
    if (child.getAttribute("widgetContainer") == "true") {
      //increment showCount for gMyworldWidgetsVisible
      if (child.getAttribute("hidden") == "false") {
        showCount++;
        showVal = "true";
      } else {
        showVal = "false";
      }
      //alert(child.getAttribute("hidden"));
      //reset the column index
      child.setAttribute("col_index", indexCount);
      columns.push({
        index: indexCount,
        label: child.getAttribute("label_val"),
        url: child.getAttribute("iframe_src"),
        show: showVal
      });
      indexCount++;
    }
  }

  checkDiscoveryBox(showCount);

  gMyworldWidgetsVisible = showCount;
  gMyworldWidgets = columns;
  this.refreshMoveItems();
  this.saveColumnLayout();
}

function checkDiscoveryBox(aWidgetsVisible) {
  if (aWidgetsVisible == 0) {
    // Show the discovery box
    document.getElementById("noContentID").setAttribute("hidden", "false");
  } else {
    // Hide the discovery box
    document.getElementById("noContentID").setAttribute("hidden", "true");
  }
}

function closeWidget(aWidgetId) {
  var node = document.getElementById(aWidgetId);
  var parent = node.parentNode;
  parent.removeChild(node);
  parent.appendChild(node);
  node.setAttribute("hidden","true");
  var nodeMenuItem = document.getElementById(aWidgetId + "-toggleItem");
  nodeMenuItem.setAttribute("checked", "false");
  this.updateCurrentColumnLayout();
}

function doWidgetToggle(aWidgetId, obj) {
  var node = document.getElementById(aWidgetId);
  if (node.getAttribute("hidden") == "true") {
    //ensure that widget gets appended to end of parent
    if (node.parentNode && node.parentNode.lastChild != node) {
      var parent = node.parentNode;
      parent.removeChild(node);
      parent.appendChild(node);
    }
    node.setAttribute("hidden","false");
    var nodeMenuItem = document.getElementById(aWidgetId + "-toggleItem");
    nodeMenuItem.setAttribute("checked", "true");
    this.updateCurrentColumnLayout();
  } else {
    this.closeWidget(aWidgetId);
    gMetrics.report("MyWorld-HideWidget", "uncheck");
  }
}

function refreshMoveItems() {
  var i = 0;
  for each (var column in gMyworldWidgets) {
    if (eval(column.show)) {
      var col_index = i++;
      var moveLeft = document.getElementById(column.label + "Id-moveLeft");
      var moveRight = document.getElementById(column.label + "Id-moveRight");
      moveLeft.setAttribute("hidden", "false");
      moveRight.setAttribute("hidden", "false");
      switch(col_index) {
       case 0:
         moveLeft.setAttribute("hidden", "true");
         break;
       case (gMyworldWidgetsVisible - 1):
         moveRight.setAttribute("hidden", "true");
         break;
       default:
         break;
      }
      if ((col_index == 0) && (gMyworldWidgetsVisible - 1 == 0)) {
        moveLeft.setAttribute("hidden", "true");
        moveRight.setAttribute("hidden", "true");
      }
    }
  }
}

function doWidgetAction(aAction) {
  switch(aAction) {
    case "topsites": {
      return "openFavoritesManager(event)";
    } break;
    case "topfeeds": {
      return "openNewsSidebar(null, event, 'FaveHeaderLabel');";
    } break;
    case "topmedia": {
      return "openMediaBar(null, event)";
    } break;
    case "topfriends": {
      return "openPeopleSidebar(event)";
    } break;
    default:
      break;
  }
}

/**
 * Load MyWorld Title
 */
function loadMyWorldTitle() {
  var titleElement = document.getElementById("myworld-title");
  if (PREFS.getPrefType(PREF_MYWORLD_TITLE)) {
    var myTitle = PREFS.getComplexValue(PREF_MYWORLD_TITLE,
                                        nsIPrefLocalizedString).data;
  } else {
    var myTitle = flockGetString("start/myworld", "flock.myworld.title");
  }
  titleElement.setAttribute("value", myTitle);
  titleElement.setAttribute("originalTitle", myTitle);

  // Get computed style for title to determine min-width
  var view = titleElement.ownerDocument.defaultView;
  var titleStyle = view.getComputedStyle(titleElement, "");
  gMyworldTitleMinWidth = (parseInt(titleStyle
                                    .getPropertyCSSValue("min-width")
                                    .cssText) || 0);
  //init title resize
  this.resizeMyWorldTitle(titleElement);

  document.title = myTitle;
}

function focusMyWorldTitle(obj) {
  obj.setAttribute("edit","true");
  obj.removeAttribute("readonly");
  obj.select();
}

/**
 * Handle KeyPress Events for the MyWorld Title
 */
function keypressMyWorldTitle(aEvent, obj) {
  switch (aEvent.keyCode) {
    case KeyEvent.DOM_VK_RETURN:
      document.commandDispatcher.advanceFocus();
      this.saveMyWorldTitle(obj);
      break;
    case KeyEvent.DOM_VK_TAB:
      this.saveMyWorldTitle(obj);
      break;
    case KeyEvent.DOM_VK_ESCAPE:
      obj.value = obj.getAttribute("originalTitle");
      document.commandDispatcher.advanceFocus();
      this.resizeMyWorldTitle(obj);
      break;
    default:
      //resize textbox
      this.resizeMyWorldTitle(obj);
      break;
   }
}

/**
 * Resize MyWorld Title
 */
function resizeMyWorldTitle(obj) {
  var strValue = obj.value;
  var strLength = strValue.length;
  if (strLength) {
    //need scaling factor for font-size used
    var scale = 0;
    var str = strValue.split("");
    for (var i = 0; i < str.length; i++) {
      if(str[i] != str[i].toLowerCase()) {
        scale += MYWORLD_TITLE_FONTSCALE_UC;
      } else {
        scale += MYWORLD_TITLE_FONTSCALE_LC;
      }
    }
    scale = (scale / str.length).toFixed();
    var newWidth = (strLength * scale).toFixed();
    if (newWidth > gMyworldTitleMinWidth) {
      obj.style.width = newWidth + "px";
    } else {
      obj.style.width = gMyworldTitleMinWidth + "px";
    }
  }
}

/**
 * Saves MyWorld Title
 */
function saveMyWorldTitle(obj) {
  var origTitle = obj.getAttribute("originalTitle");

  // Get current myworld title
  if (PREFS.getPrefType(PREF_MYWORLD_TITLE)) {
    origTitle = PREFS.getComplexValue(PREF_MYWORLD_TITLE,
                                      nsIPrefLocalizedString).data;
  }
  if (origTitle == null) {
    // No pref and has never been changed, therefore we should use the existing
    // document title
    origTitle = document.title;
  }

  // Save and reset css states to appear unfocused
  if (obj.value != null) {
    obj.setAttribute("originalTitle", obj.value);
    obj.removeAttribute("edit");
    obj.setAttribute("readonly", "true");
    var str = Cc["@mozilla.org/supports-string;1"]
              .createInstance(Ci.nsISupportsString);
    str.data = obj.value;
    PREFS.setComplexValue(PREF_MYWORLD_TITLE, Ci.nsISupportsString, str);
    gMetrics.report("MyWorld-NameChange", null);
    document.title = obj.value;
  }
}

//ensure column container renders the height of the iframe content
function resizeWidgetContainer() {
  //reset height for changes in size
  var columnContainerHeight = 0;

  //calculate max height of the widgets
  for each (var column in gMyworldWidgets) {
    var widget = document.getElementById(column.label + "Id");
    if (widget && widget.childNodes[1] &&
        widget.getAttribute("hidden") != "true") {
      var colDoc = widget.childNodes[1].contentDocument.documentElement;
      if (colDoc.lastChild.boxObject) {
        var colHeight = colDoc.lastChild.boxObject.height;
        if (colHeight > columnContainerHeight) {
          //take into account the height for the widget header
          var headerOffset = widget.firstChild.boxObject.height;
          columnContainerHeight = colHeight + headerOffset;
        }
      }
    }
  }
  //set the size of the container
  document.getElementById("ftcontainerID")
          .setAttribute("style", "min-height:" + columnContainerHeight + "px");
}

window.onload = onLoad;
window.onunload = onUnload;
