// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://gre/modules/JSON.jsm");

const MIN_REFRESH_INTERVAL = 5000;

const RDFS = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
function Namespace(ns) { return function (arg) { return RDFS.GetResource(ns+arg); } }
const FLRDF  = Namespace("http://flock.com/rdf#");
const NCRDF  = Namespace("http://home.netscape.com/NC-rdf#");
const kXULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var gMetrics = Cc["@flock.com/metrics-service;1"].getService(Ci.flockIMetricsService);

function openURL(node, event) {
  if (event.button == 2) {
    return;
  }

  var url = node.childNodes[1].childNodes[1].value;
  this.gMetrics.report("Favorites-FavoriteClick", "MyWorld");

  // Open in new tab
  openURLMyWorld(url, event, false);
}

function openURLMyWorld(url, event, forceWindow) {
  _logger.debug("myworld open event btn" + event.button);

  if (event.button == 2) {
    return;
  }

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    if (forceWindow) {
      var options = "centerscreen,resizable=yes";
#ifdef XP_WIN
      options += ",alwaysRaised";
#endif
      win.open(url, "miniView", options);
    } else {
      // opens url in new tab and switches to it immediately
      // win.openUILinkIn(url,"tab");

      switch (win.whereToOpenLink(event)) {
        case "current":
          win.openUILinkIn(url, "tab");
          break;
        case "window":
          win.getTopWin().openNewWindowWith(url, null, null);
          break;
        default: // "tab" or "tabshifted"
          win.getTopWin().openNewTabWith(url, null, null, event);
          break;
      }
    }
  }
}

function openPeopleSidebar(event) {
  if (event.button == 2) {
    return; 
  }

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow("navigator:browser");
  if (win) {
    // Only open, don't close if already open
    win.toggleSidebarWithMetric("flockPeopleSidebarBroadcaster",
                                "PeopleSidebar-Open",
                                null,
                                "MyWorldPeopleButton",
                                true);
  }
}

function openWebClipboard(event) {
  if (event.button == 2) {
    return;
  }

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    // Only open, don't close if already open
    win.toggleSidebarWithMetric("flock_notesSidebarBroadcaster",
                                "WebClipboard-Open",
                                null,
                                "MyWorldWebClipboardButton",
                                true);
  }
}

function openAASSidebar(node, event) {
  if (event.button == 2) {
    return;
  }
  if (node == null) {
    // Just open the sidebar.
    var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
             .getService(Ci.nsIWindowMediator);
    var win = wm.getMostRecentWindow('navigator:browser');
    if (win) {
      // Only open, don't close if already open
      win.toggleSidebarWithMetric("flock_AccountsSidebarBroadcaster",
                                  "AccountSidebar-Open",
                                  null,
                                  "MyWorldAccountsButton",
                                  true);
    }
    
    return;
  }

  var homeURL = node.getAttribute("homeURL");
  openURLMyWorld(homeURL, event, false);
}

function openFavoritesManager(event) {
  if (event.button == 2) {
    return;
  }

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    // Only open, don't close if already open
    win.toggleSidebarWithMetric("viewBookmarksSidebar",
                                "FavesSidebar-Open",
                                null,
                                "MyWorldFavesButton",
                                true);
  }
}

function openNewsSidebar(node, event, aMetricsValue) {
  if (event.button == 2) {
    return;
  }

  //want full box to be hot but we need handle headlines being clicked
  if (event.target.className == "headlinesItem") {
    return;
  }

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
          .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    win.newsSidebarOpen(aMetricsValue);
    if (node != null) {
      var url = encodeURIComponent(node.getAttribute("url"));
      openURLMyWorld("flock://favorites/urn:flock:feed:" + url, event, false);
    }
  }
}

function openImage(node, event) {
  if (event.button == 2) {
    return;
  }

  var mediaType = "image";
  if (node.getAttribute("isVideo") == "true") {
    mediaType = "video";
  }
  this.gMetrics.report("MyWorld-Media-ItemClick", mediaType);

  var imageURL = node.getAttribute("imageURL");
  var dataNode = node.parentNode;

  // for Truveo videos, always open in new window
  // c.f. http://bugzilla.flock.com/show_bug.cgi?id=7045
  if (dataNode.getAttribute("svc") == "searchvideo") {
    openURLMyWorld(imageURL, event, true);

    // Don't pop the mediabar
  } else {
    openURLMyWorld(imageURL, event, false);

    // Open the media bar
    openMediaBar(dataNode, event);
  }
}

function openMediaBar(node, event) {
  if (event.button == 2) {
    return;
  }

  this.gMetrics.report("MediaBar-Open", "MyWorld");
  if (node) {
    var service = node.getAttribute("svc");
    var query = node.getAttribute("query");
    var name = node.getAttribute("name");
  }

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
          .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow("navigator:browser");
  if (win) {
    win.flock_topbarSelectById("flock_MediabarTopbarBroadcaster", true);

    if (service != "" && node) {
      win.mediabar.loadQuery(service, query, name);
    }
  }
  event.stopPropagation();
}

function popupWhatsThis(node) {
  var description = node.getAttribute("desc");
  var descriptionID = node.getAttribute("descID");

  var popup = document.getElementById(descriptionID);

  while (popup.firstChild) {
    popup.removeChild(popup.firstChild);
  }

  var descNode = document.createElement("description");
  descNode.setAttribute("class", "whatsThisPopup");
  descNode.setAttribute("flex", "1");
  var text = document.createTextNode(description);
  descNode.appendChild(text);
  popup.appendChild(descNode);

  popup.showPopup(node, -1, -1, "popup", "bottomleft", "topleft");
}

function setOpacity(node, opacityNum) {
  node.style.opacity = opacityNum;
}

function insertChildAt(i, aChild, aParent) {
  if (i >= aParent.childNodes.length) {
    aParent.appendChild(aChild);
  } else {
    var elementAti = aParent.childNodes[i];
    aParent.insertBefore(aChild, elementAti);
  }
}

function removeChildren(aXulElement) {
  var child;
  while ((child = aXulElement.lastChild) != null) {
    aXulElement.removeChild(child);
  }
}

function setNoItemMessage(aMediaStreamBox, aNoId) {
  var hasMedia = aMediaStreamBox.firstChild != null;
  document.getElementById(aNoId).setAttribute("hidden", hasMedia);
}

function refreshTopFave(aFavBox, aTopUrn, aUpdateFct, aFavRes, aReplacedUrn) {
  var favId2Update = aFavRes ?
                     aFavRes.QueryInterface(Ci.nsIRDFResource).ValueUTF8 :
                     null;
  _logger.debug("refreshing myWorld " + aReplacedUrn + " -> " + favId2Update);

  if (aReplacedUrn != favId2Update) {
    var oldElement = document.getElementById(aReplacedUrn);
    if (aFavRes) {
      // This new favorite has kicked out the oldest from the top faves
      oldElement.setAttribute("id", favId2Update)
    } else {
      // Remove this favorite
      if (oldElement) {
        // Remove children (e.g thumbnails or feed items)
        aFavBox.removeChild(oldElement);
        removeChildren(oldElement);
      }
      return;
    }
  }

  var topfav = _coop.get(aTopUrn);
  var childEnum = topfav.children.enumerateBackwards();

  var i = 0;
  // Loop through all favorites
  while (childEnum.hasMoreElements()) {
    var favCoopObject = childEnum.getNext();
    var favId = favCoopObject.id();

    // Check if it's a feed - we do not want to display feeds with no unread
    // items
    if (favCoopObject.flockType == "feed" && favCoopObject.unseenItems == "0") {
      continue;
    }

    var favElement = document.getElementById(favId);
    if (!favElement) {
      favElement = aUpdateFct(favCoopObject, favElement);
      favElement.setAttribute("id", favId);
      insertChildAt(i, favElement, aFavBox);
    }
    else if (favId2Update == favId) {
      removeChildren(favElement);
      var elementAtIndex = aFavBox.childNodes[i];
      var needsMoving = !elementAtIndex || elementAtIndex != favElement;
      aUpdateFct(favCoopObject, favElement, needsMoving);
      if (needsMoving) {
        aFavBox.removeChild(favElement);
        insertChildAt(i, favElement, aFavBox);
      }
      break;
    }
    i++;
  }
  _logger.debug("refresh of myWorld done");
}
