// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

var _logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
_logger.init("topfeeds");

var _coop = Cc["@flock.com/singleton;1"]
            .getService(Ci.flockISingleton)
            .getSingleton("chrome://flock/content/common/load-faves-coop.js")
            .wrappedJSObject;

var draggingObserver = {
  onDragStart: function topsites_onDragStart(aEvent, aTransferData, aAction) {
    var url = aEvent.target.getAttribute("url");
    var title = aEvent.target.getAttribute("name");
    if (!url) {
      return;
    }
    aTransferData.data = new TransferData();
    aTransferData.data.addDataForFlavour("text/x-moz-url", url + "\n" + title);
    aTransferData.data.addDataForFlavour("text/html",
      <a href={url}>{title}</a>.toXMLString()
    );
    aTransferData.data.addDataForFlavour("text/unicode", url);
  },
  onDragOver: function topsites_onDragOver(aEvent, aTransferData, aAction) {
  },
  onDrop: function topsites_onDrop(aEvent, aTransferData, aAction) {
  },
  onDragExit: function topsites_onDragExit(aEvent, aTransferData, aAction) {
  }
}

var gObsrvr = null;

function onLoad() {
  _logger.debug("onLoad");
  gObsrvr = {};
  gObsrvr.observe = function feeds_observe(aFeedRes, aTopic, aReplacedUrn) {
    if (aTopic == "refresh-myworld-topfeeds") {
      refreshTopFeeds(aFeedRes, aReplacedUrn);
    } else if(aTopic == "refresh-myworld-date") {
      _logger.debug("refreshing Feed Dates");
      var feedStreamBox = document.getElementById("feedstreambox");
      for (var i = feedStreamBox.childNodes.length - 1; i >= 0; i--) {
        var id = feedStreamBox.childNodes[i].getAttribute("id");
        feedStreamBox.childNodes[i].updateFeedDate(_coop.get(id, true));
      }
    }
  }

  var os = Cc["@mozilla.org/observer-service;1"]
           .getService(Ci.nsIObserverService);
  os.addObserver(gObsrvr, "refresh-myworld-topfeeds", false);
  os.addObserver(gObsrvr, "refresh-myworld-date", false);

  refreshTopFeeds();
}

function onUnload() {
  _logger.debug("onUnload");

  var os = Cc["@mozilla.org/observer-service;1"]
           .getService(Ci.nsIObserverService);
  os.removeObserver(gObsrvr, "refresh-myworld-topfeeds");
  os.removeObserver(gObsrvr, "refresh-myworld-date", false);
}

function openFeedHeadline(headlineURL, headlineID, event) {
  if (event.button == 2) {
    return;
  }

  this.gMetrics.report("MyWorld-FeedClick", null);
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
           .getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow("navigator:browser");
  if (win) {
    openURLMyWorld(headlineURL, event, false);

    var coopObj = _coop.get(headlineID);
    coopObj.unseen = false;
  }
}

function refreshTopFeeds(aFeedRes, aReplacedUrn) {
  var feedStreamBox = document.getElementById("feedstreambox");
  refreshTopFave(feedStreamBox, "urn:flock:topfeeds", updateFeed, 
                 aFeedRes, aReplacedUrn);
  setNoItemMessage(feedStreamBox, "noFeedsID");
}

function updateFeed(aFeed, fsbEntry, aWillBeMoved) {
  if (!fsbEntry) {
    fsbEntry = document.createElement("feedstream");
    fsbEntry.setAttribute("class", "faveFeed"); 
    fsbEntry.setAttribute("flex", "1");
    var startDrag = "nsDragAndDrop.startDrag(event, draggingObserver, true);"
    fsbEntry.setAttribute("ondraggesture", startDrag);
  } else if (!aWillBeMoved) {
    // If the element is moved, no need to update it because
    // the binding is reconstructed
    fsbEntry.updateFeeds(0, aFeed)
  }

  fsbEntry.setAttribute("favicon", aFeed.favicon);
  fsbEntry.setAttribute("name", aFeed.name); 
  fsbEntry.setAttribute("url", aFeed.URL); 
  return fsbEntry;
}

window.onload = onLoad;
window.onunload = onUnload;
