// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const CC = Components.classes;
const CI = Components.interfaces;
const CR = Components.results;
const CU = Components.utils;

CU.import("resource:///modules/FlockScheduler.jsm");

const LAST_CHANGE_TIMER_INTERVAL = 60 * 1000;

var _logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
_logger.init("topfriends");

var _coop = Cc["@flock.com/singleton;1"]
            .getService(Ci.flockISingleton)
            .getSingleton("chrome://flock/content/common/load-faves-coop.js")
            .wrappedJSObject;

var _rdfSvc = Cc["@mozilla.org/rdf/rdf-service;1"]
              .getService(Ci.nsIRDFService);
var _ds = _rdfSvc.GetDataSource("rdf:flock-favorites");
var gFriendFeedController;
var gFriendFeedObserver = null;

function onLoad() {
  _logger.debug("onLoad");
  gFriendFeedController = new FriendFeedController();
  gFriendFeedObserver = {};
  gFriendFeedObserver.observe = function FriendFeedObserver_observer(aEngine,
                                                                     aTopic,
                                                                     aVerb)
  {
    if (aTopic == "refresh-myworld-topfriends") {
      gFriendFeedController.refreshFriendFeed();
    }
  }

  var os = CC["@mozilla.org/observer-service;1"]
           .getService(CI.nsIObserverService);
  os.addObserver(gFriendFeedObserver, "refresh-myworld-topfriends", false);
  gFriendFeedController.init();
}

function onUnload() {
  _logger.debug("onUnload");
  var os = CC["@mozilla.org/observer-service;1"]
           .getService(CI.nsIObserverService);
  os.removeObserver(gFriendFeedObserver, "refresh-myworld-topfriends");

  gFriendFeedController.uninit();
}

function friendClicked(aNode, aEvent) {
  var friendActivity = _coop.get(aNode.id);
  var identity = friendActivity.identity;
  var service = CC[identity.serviceId].getService(CI.flockIWebService);
  switch (friendActivity.updateType) {
    // If the friendActivity item is for a profile or status update then open
    // the service specific profile page for the friend
    case "profile":
    case "status":
      var accountUrn = identity.getAccount().id();
      var account = service.getAccount(accountUrn)
                           .QueryInterface(CI.flockISocialWebServiceAccount);
      var url = account.getProfileURLForFriend(identity.id());
      if (url) {
        var win = CC["@mozilla.org/appshell/window-mediator;1"]
                  .getService(CI.nsIWindowMediator)
                  .getMostRecentWindow("navigator:browser");

        if (win) {
          win.openUILinkIn(url, "tab");
        }
      }

      break;

    // If the friendActivity item is for a media update then display the media
    // bar with the friends media stream
    case "media":
      var query = new queryHelper();
      query.user = identity.accountId;
      query.username = identity.name;

      var win = CC["@mozilla.org/appshell/window-mediator;1"]
                .getService(CI.nsIWindowMediator)
                .getMostRecentWindow("navigator:browser");
      if (win) {
        gMetrics.report("MediaBar-Open", "MyWorld-FriendActivity");
        win.flock_topbarSelectById("flock_MediabarTopbarBroadcaster", true);
        win.mediabar.loadQuery(service.shortName,
                               query.stringVal,
                               identity.name);
      }
      break;
  }
  gMetrics.report("MyWorld-FriendActivityClick",
                  [{service: service.shortName,
                    type: friendActivity.updateType}]);
}

function FriendFeedController() {
  this.friendFeed = document.getElementById("friendsstreambox");
  this.sb = CC["@mozilla.org/intl/stringbundle;1"]
            .getService(CI.nsIStringBundleService)
            .createBundle("chrome://flock/locale/start/myworld.properties");
  this.draggingObserver = {
    onDragStart: function topfriends_onDragStart(aEvent,
                                                 aTransferData,
                                                 aAction)
    {
      var profileURL = aEvent.target.getAttribute("profileURL");
      var handle = aEvent.target.getAttribute("handle");

      if (!profileURL) {
        return;
      }

      aTransferData.data = new TransferData();
      aTransferData.data.addDataForFlavour("text/x-moz-url",
                                           profileURL + "\n" + handle);
      aTransferData.data.addDataForFlavour("text/html",
        <a href={profileURL}>{handle}</a>.toXMLString()
      );
      aTransferData.data.addDataForFlavour("text/unicode", profileURL);
    },
    onDragOver: function topfriends_onDragOver(aEvent, aTransferData, aAction)
    {
    },
    onDrop: function topfriends_onDrop(aEvent, aTransferData, aAction) {
    },
    onDragExit: function topfriends_onDragExit(aEvent, aTransferData, aAction)
    {
    }
  }

  var inst = this;
  this.schedulerTimers = [];

  this.lastChangeTimer = CC["@mozilla.org/timer;1"].createInstance(CI.nsITimer);
  this.lastChangeTimerListener = {
    notify: function FriendListController_notify(aTimer) {
      _logger.debug("FriendListController: lastChangeTimerListener");
      function updateTimes(aShouldYield) {
        var nodeList = inst.friendFeed.childNodes;
        for (var c = 0; c < nodeList.length; c++) {
          nodeList.item(c).updateTime();
          if (aShouldYield()) {
            yield;
          }
        }
      }
      FlockScheduler.schedule(inst.schedulerTimers, 0.05, 10, updateTimes);
    }
  }

}

FriendFeedController.prototype.assembleTooltip =
function FriendFeedController_assembleTooltip() {
  _logger.debug("assembleTooltip");

  var handle = document.tooltipNode.handle;
  _logger.debug("handle = " + handle);

  var tooltipHandle =
    document.getElementById("tooltipHandle");
  tooltipHandle.innerHTML = handle;

  var description = document.tooltipNode.description;
  _logger.debug("description = " + description);

  var tooltipDescription =
    document.getElementById("tooltipDescription");
  tooltipDescription.innerHTML = description;
}

FriendFeedController.prototype.serviceClicked =
function FriendFeedController_serviceClicked(aNode) {
  _logger.debug("serviceClicked");

  var shortName = aNode.getAttribute("shortName");
  var Urn = aNode.getAttribute("Urn");

  // Log the action with the metric service
  CC["@flock.com/metrics-service;1"].getService(CI.flockIMetricsService)
                                    .report("MyWorld-TopFriendsDiscoverService",
                                            shortName);
  // Remember the account
  var wsMgr = CC["@flock.com/webservice-manager;1"]
              .getService(CI.flockIWebServiceManager);
  wsMgr.autoKeepAccountForService(Urn);

  // Redirect the user to the service
  var win = CC["@mozilla.org/appshell/window-mediator;1"]
            .getService(CI.nsIWindowMediator)
            .getMostRecentWindow("navigator:browser");

  var url = "http://redir.flock.com/redir?component=browser&action=people_click";
  url += "&partner=" + shortName;

  if (win) {
    win.openUILinkIn(url, "tab");
  }
}

FriendFeedController.prototype.buildEmptyFeedContent =
function FriendFeedController_buildEmptyFeedContent() {
  _logger.debug("buildEmptyFeedContent");

  var catMgr = CC["@mozilla.org/categorymanager;1"]
               .getService(CI.nsICategoryManager);
  var webServiceList = catMgr.enumerateCategory("flockWebService");
  var serviceBox = document.getElementById("availableServicesContent");
  var svcChild = serviceBox.firstChild;
  while (svcChild) {
    serviceBox.removeChild(svcChild);
    svcChild = serviceBox.firstChild;
  }

  while (webServiceList.hasMoreElements()) {
    var webService = webServiceList.getNext()
                                   .QueryInterface(CI.nsISupportsCString);
    if (!webService) {
      continue;
    }

    var contractID = catMgr.getCategoryEntry("flockWebService", webService.data);
    var svc = CC[contractID].getService(CI.flockIWebService);

    if (svc instanceof CI.flockISocialWebService) {
      _logger.debug("refreshDiscoveryPanel(): Found a service - "
                    + svc.shortName);

      var favicon = document.createElement("image");
      favicon.setAttribute("src", svc.icon);

      var label = document.createElement("label");
      label.setAttribute("class", "discovery-service-link");
      var linkText = svc.title +
                     " - " +
                     this.sb
                         .GetStringFromName("flock.myworld.topfriends.services.clickMessage");
      label.setAttribute("value", linkText);
      label.setAttribute("shortName", svc.shortName);
      label.setAttribute("Urn", svc.urn);
      label.setAttribute("onclick", "gFriendFeedController.serviceClicked(this);");
      label.setAttribute("crop", "end");

      var hbox = document.createElement("hbox");
      hbox.setAttribute("class", "discovery-service-entry");
      hbox.appendChild(favicon);
      hbox.appendChild(label);
      serviceBox.appendChild(hbox);
    }
  }
}

FriendFeedController.prototype.refreshFriendFeed =
function FriendFeedController_refreshFriendFeed() {
  _logger.debug("refreshFriendFeed");

  const FLOCK_SNOWMAN_URL = "chrome://flock/skin/people/no_avatar.png";

  while (this.friendFeed.firstChild) {
    this.friendFeed.removeChild(this.friendFeed.firstChild);
  }

  var topfriends = _coop.get("urn:flock:aggregatedFriendActivityLog");
  if (!topfriends || !topfriends.children) {
    return;
  }
  var topFriendActivities = topfriends.children.enumerateBackwards();

  var hasFriends = false;
  if (topFriendActivities.hasMoreElements()) {
    hasFriends = true;
  }

  document.getElementById("discovery-message-container")
          .setAttribute("hidden", hasFriends);

  if (!hasFriends) {
    this.buildEmptyFeedContent();
    return;
  }

  var dateFormatter = CC["@flock.com/date-formatter;1"]
                      .getService(CI.flockIDateFormatter);

  // Build the friend feed using the aggregated friend activity log
  while (topFriendActivities.hasMoreElements()) {
    var friendActivity = topFriendActivities.getNext();
    var friend = friendActivity.identity;

    if (!friend) {
      continue;
    }

    var feedItem = document.createElement("friendfeeditem");
    this.friendFeed.appendChild(feedItem);
    feedItem.setAttribute("tooltip", "friendActivityTooltip");
    feedItem.setAttribute("id", friendActivity.id());
    feedItem.setAttribute("lastUpdateType", friendActivity.updateType);
    feedItem.setAttribute("lastupdate", Number(friendActivity.updateDate));

    var friendActivityDate = friendActivity.updateDate;
    if (!friendActivityDate) {
      friendActivityDate = new Date();
    }

    var friendlyDate = dateFormatter.getFriendlyLastDate(friendActivityDate.getTime());
    feedItem.setAttribute("timeInfo", friendlyDate);
    feedItem.handle = friend.name;

    if (friend.avatar && (friend.avatar != "")) {
      feedItem.setAttribute("avatarUrl", friend.avatar);
    } else {
      feedItem.setAttribute("avatarUrl", FLOCK_SNOWMAN_URL);
    }

    feedItem.setAttribute("serviceId", friend.serviceId);
    feedItem.setAttribute("onclick", "friendClicked(this, event);");

    var service = CC[friend.serviceId].getService(CI.flockIWebService);
    var accountUrn = friend.getAccount().id();
    var serviceAccount = service.getAccount(accountUrn)
                                .QueryInterface(CI.flockISocialWebServiceAccount);

    var description =
      serviceAccount.formatFriendActivityForDisplay(friendActivity.id());

    feedItem.setAttribute("profileURL", 
                          serviceAccount.getProfileURLForFriend(friend.id()));
    feedItem.setAttribute("ondraggesture",
                          "nsDragAndDrop.startDrag(event, gFriendFeedController.draggingObserver, true);");
    feedItem.description = description;
    //feedItem.setAttribute("tooltiptext", friend.name + " " + description);
    feedItem.setAttribute("serviceName", service.title);
    feedItem.setAttribute("faviconUrl", service.icon);
  }
}

FriendFeedController.prototype.init =
function FriendFeedController_init() {
  this.refreshFriendFeed();

  this.lastChangeTimer.initWithCallback(this.lastChangeTimerListener,
                                        LAST_CHANGE_TIMER_INTERVAL,
                                        CI.nsITimer.TYPE_REPEATING_SLACK);
}

FriendFeedController.prototype.uninit =
function FriendFeedController_uninit() {
  this.lastChangeTimer.cancel();
  this.lastChangeTimer = null;
  for (var c = this.schedulerTimers.length - 1; c >= 0; c--) {
    FlockScheduler.cancel(this.schedulerTimers, c);
  }
}

window.onload = onLoad;
window.onunload = onUnload;
