// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

var _logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
_logger.init("topmedia");

var _coop = Cc["@flock.com/singleton;1"]
            .getService(Ci.flockISingleton)
            .getSingleton("chrome://flock/content/common/load-faves-coop.js")
            .wrappedJSObject;

var _photoApiMgr = Cc["@flock.com/photo-api-manager;1?"]
                   .getService(Ci.flockIPhotoAPIManager);

var draggingObserver = {
  onDragStart: function topsites_onDragStart(aEvent, aTransferData, aAction) {
    var url = aEvent.target.getAttribute("imageURL")
    var title = aEvent.target.getAttribute("title");

    if (!url) {
      return;
    }

    aTransferData.data = new TransferData();
    aTransferData.data.addDataForFlavour("text/x-moz-url", url + "\n" + title);
    aTransferData.data.addDataForFlavour("text/html",
      <a href={url}>{title}</a>.toXMLString()
    );
    aTransferData.data.addDataForFlavour("text/unicode", url);
  },
  onDragOver: function topsites_onDragOver(aEvent, aTransferData, aAction) {
  },
  onDrop: function topsites_onDrop(aEvent, aTransferData, aAction) {
  },
  onDragExit: function topsites_onDragExit(aEvent, aTransferData, aAction) {
  }
}

var gObsrvr = null;

function onLoad() {
  _logger.debug("onLoad");
  var sb = Cc["@mozilla.org/intl/stringbundle;1"]
            .getService(Ci.nsIStringBundleService)
            .createBundle("chrome://flock/locale/start/myworld.properties");
  this.mQueryOn = sb.GetStringFromName("flock.myworld.media.queryOn");

  var inst = this;

  gObsrvr = {};
  gObsrvr.observe = function media_observe(aMediaQuery, aTopic, aReplacedUrn) {
    if (aTopic == "refresh-myworld-topmedia") {
      refreshTopMedia(aMediaQuery, aReplacedUrn);
    } else if (aTopic == "refresh-myworld-date") {
      _logger.debug("refreshing Media Dates");
      var mediaStreamBox = document.getElementById("mediastreambox");
      var nodeList = mediaStreamBox.childNodes;
      for (var c = 0; c < nodeList.length; c++) {
        nodeList.item(c).updateDate();
      }
    }
  }

  // Add observer for engines added
  var os = Cc["@mozilla.org/observer-service;1"]
          .getService(Ci.nsIObserverService);
  os.addObserver(gObsrvr, "refresh-myworld-topmedia", false);
  os.addObserver(gObsrvr, "refresh-myworld-date", false);

  refreshTopMedia();
}

function onUnload() {
  _logger.debug("onUnload");

  var os = Cc["@mozilla.org/observer-service;1"]
           .getService(Ci.nsIObserverService);
  os.removeObserver(gObsrvr, "refresh-myworld-topmedia");
  os.removeObserver(gObsrvr, "refresh-myworld-date");
}

function refreshTopMedia(aMediaQueryRes, aReplacedUrn) {
  var mediaStreamBox = document.getElementById("mediastreambox");
  refreshTopFave(mediaStreamBox, "urn:flock:topmedia", updateMediaQuery, 
                 aMediaQueryRes, aReplacedUrn);
  setNoItemMessage(mediaStreamBox, "noMediaID");
}

function updateMediaQuery(aMediaQuery, aMediaElement, aWillBeMoved) {
  if (!aMediaElement) {
    aMediaElement = document.createElement("mediaquery");
  } else if (!aWillBeMoved) {
    // If the element is moved, no need to update it because
    // the binding is reconstructed
    aMediaElement.updateDate(aMediaQuery);
  }

  var name = aMediaQuery.name;
  var service = _photoApiMgr.getAPIFromShortname(aMediaQuery.service);
  aMediaElement.setAttribute("svc", service.shortName);
  aMediaElement.setAttribute("query", aMediaQuery.query);
  aMediaElement.setAttribute("favIconLoc", aMediaQuery.favicon);
  aMediaElement.setAttribute("name", name);
  var text = this.mQueryOn.replace("%query", name)
                          .replace("%service", service.title);
  aMediaElement.setAttribute("description", text);
  var startDrag = "nsDragAndDrop.startDrag(event, draggingObserver, true);"

  // Loop through all RichPhotos.
  var richPhotoEntryEnum = aMediaQuery.children.enumerate();

  while (richPhotoEntryEnum.hasMoreElements()) {
    var richEntry = richPhotoEntryEnum.getNext();
    var thumbnail = document.createElement("mediaitem");
    thumbnail.setAttribute("ondraggesture", startDrag);
    thumbnail.setAttribute("isVideo", richEntry.is_video);
    thumbnail.setAttribute("username", richEntry.username);
    thumbnail.setAttribute("title", richEntry.name);
    thumbnail.setAttribute("tooltiptext", richEntry.name);
    thumbnail.setAttribute("imageURL", richEntry.URL);
    thumbnail.setAttribute("imgClass", "imagePreview");
    thumbnail.setAttribute("imgSrc", richEntry.thumbnail);

    // Add to related MediaQuery
    aMediaElement.appendChild(thumbnail);
  }
  return aMediaElement;
}

window.onload = onLoad;
window.onunload = onUnload;
