// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

var _logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
_logger.init("topsites");

var _coop = Cc["@flock.com/singleton;1"]
            .getService(Ci.flockISingleton)
            .getSingleton("chrome://flock/content/common/load-faves-coop.js")
            .wrappedJSObject;

var draggingObserver = {
  onDragStart: function topsites_onDragStart(aEvent, aTransferData, aAction) {
    var url = aEvent.target.getAttribute("url");
    var title = aEvent.target.getAttribute("name");
    if (!url) {
      return;
    }
    aTransferData.data = new TransferData();
    aTransferData.data.addDataForFlavour("text/x-moz-url", url + "\n" + title);
    aTransferData.data.addDataForFlavour("text/html",
      <a href={url}>{title}</a>.toXMLString()
    );
    aTransferData.data.addDataForFlavour("text/unicode", url);
  },
  onDragOver: function topsites_onDragOver(aEvent, aTransferData, aAction) {
  },
  onDrop: function topsites_onDrop(aEvent, aTransferData, aAction) {
  },
  onDragExit: function topsites_onDragExit(aEvent, aTransferData, aAction) {
  }
}

var gObsrvr = null;

function onLoad() {
  _logger.debug("onLoad");

  var inst = this;

  gObsrvr = {};
  gObsrvr.observe = function sites_observe(aSiteRes, aTopic, aReplacedUrn) {
    if (aTopic == "refresh-myworld-topsites") {
      refreshTopSites(aSiteRes, aReplacedUrn);
    }
  }

  var os = Cc["@mozilla.org/observer-service;1"]
           .getService(Ci.nsIObserverService);
  os.addObserver(gObsrvr, "refresh-myworld-topsites", false);

  refreshTopSites();
}

function onUnload() {
  _logger.debug("onUnload");

  var os = Cc["@mozilla.org/observer-service;1"]
          .getService(Ci.nsIObserverService);
  os.removeObserver(gObsrvr, "refresh-myworld-topsites");
}

function refreshTopSites(aSiteRes, aReplacedUrn) {
  var siteStreamBox = document.getElementById("sitestreambox");
  refreshTopFave(siteStreamBox, "urn:flock:topsites", updateSite, 
                 aSiteRes, aReplacedUrn);
  setNoItemMessage(siteStreamBox, "noSitesID");
}

function updateSite(aSite, ssbEntry, aWillBeMoved) {
  if (!ssbEntry) {
    ssbEntry = document.createElement("sitestream");
    ssbEntry.setAttribute("class", "faveSite"); 
    ssbEntry.setAttribute("crop", "end"); 
    ssbEntry.setAttribute("flex", "1"); 
    var startDrag = "nsDragAndDrop.startDrag(event, draggingObserver, true);"
    ssbEntry.setAttribute("ondraggesture", startDrag);
  }
  ssbEntry.setAttribute("favicon", aSite.favicon);
  ssbEntry.setAttribute("name", aSite.name); 
  ssbEntry.setAttribute("url", aSite.URL); 
  return ssbEntry;
}

window.onload = onLoad;
window.onunload = onUnload;
