// BEGIN FLOCK PRIV1
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file (or files) is the sole property of Flock Inc.  You must recieve
// explicit written permision from a legal representative of Flock Inc.
// to modify, or redistribute this file (or files) in any form or terms,
// other than the form and terms under which you recieved it.  If you do not
// have a written licence from a legal representive of flock defining the terms
// which you can modify or redistribute this file (or files) you may assume
// that you do not have any rights to modify or redistribute this file (or
// files).
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
// License for the specific language governing rights and limitations under
// the License.
//
// END FLOCK PRIV1

/**
 * On first launch, if the user does not have an account then fire a
 * notification.
 */
function raisePhotobucketNotifications(){
  // Only prompt if this is the first run AND the user does not have a
  // photobucket account OR this is the first run and the user's account has
  // not been authenticated (which happens if remember me was not checked)
  var launchCount = flock_getIntPref("flock.launch_count");

  if (launchCount == 1) {
    // Determine whether the user has a photobucket account
    var acUtils = Components.classes["@flock.com/account-utils;1"]
                            .getService(Components.interfaces.flockIAccountUtils);
    var hasAccount = acUtils.activatedAccountExists("urn:photobucket:service");
    var isAuthenticated = false;

    // If they have an account check whether it has been authenticated as we
    // don't need the notification to show up if the user is already logged in.
    if (hasAccount){
      var _coop = Components.classes["@flock.com/singleton;1"]
                            .getService(Components.interfaces.flockISingleton)
                            .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                            .wrappedJSObject;

      var mediaAccts = acUtils.getAccountsByInterface("flockIMediaWebService");
      while (mediaAccts.hasMoreElements()) {
        var acct = mediaAccts.getNext()
                             .QueryInterface(Components.interfaces.flockIWebServiceAccount);
        var c_acct = _coop.get(acct.urn);
        var svc = Components.classes[c_acct.serviceId]
                            .getService(Components.interfaces.flockIWebService);
        if (svc.shortName == "photobucket") {
          isAuthenticated = c_acct.isAuthenticated;
          continue;
        }
      }
    }
    
    if (!hasAccount || (hasAccount && !isAuthenticated)) {
      var nBox = gBrowser.getNotificationBox(gBrowser.getBrowserAtIndex(0));
      // Localize me
      var msg = "Please log in to your Photobucket account to activate " +
                "Flock’s advanced features.";
      nBox.appendUniqueNotification(msg,
                                    "suggest-registration",
                                    "http://photobucket.com/favicon.ico",
                                    nBox.PRIORITY_INFO_MEDIUM,
                                    null,
                                    null);
      return;
    }
  }

  // Fire notification if the uploader has never been used and there have been
  // 10 browser launches since the session in which the account was created.
  var uploaderFirstRunComplete = flock_getBoolPref("flock.photo.uploader.firstruncomplete");
  if (!uploaderFirstRunComplete) {
    
    var acUtils = Components.classes["@flock.com/account-utils;1"]
                            .getService(Components.interfaces.flockIAccountUtils);
    var hasAccount = acUtils.activatedAccountExists("urn:photobucket:service");
    
    // Increment the counter pref that specifies how many times the browser
    // has been launched with a Photobucket account
    const FLOCK_PREF_PHOTOBUCKET_USER_START_COUNTER = "flock.branding.photobucket.userStartCounter";
    var starts = flock_getIntPref(FLOCK_PREF_PHOTOBUCKET_USER_START_COUNTER) || 0;
    if (hasAccount) {
      starts += 1;
      flock_setIntPref(FLOCK_PREF_PHOTOBUCKET_USER_START_COUNTER, starts);
    } else {
      flock_setIntPref(FLOCK_PREF_PHOTOBUCKET_USER_START_COUNTER, 0);
      return;
    }
    if (starts == 10) {
      var nBox = gBrowser.getNotificationBox(gBrowser.getBrowserAtIndex(0));
      var links = [{
        // Localize me
        label: "Upload Photos Now",
        callback: function () {
          var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                             .getService(Components.interfaces.nsIWindowMediator);
          var win = wm.getMostRecentWindow('navigator:browser');
          if (win) {
            win.flock_photo.util.launchUploader();
          }
        }
      }];

      // Localize me
      var msg = "You haven't tried Flock’s Photo Uploader yet. " +
                "It’s an easy way to upload your photos to Photobucket.";
      nBox.appendUniqueNotification(msg,
                                    "recommend-uploader",
                                    "http://photobucket.com/favicon.ico",
                                    nBox.PRIORITY_INFO_MEDIUM,
                                    null,
                                    links);
    }
  }
}

// WORKAROUND:
// Firing the notification immediately caused notification DOM elements (the
// link) to be unavailable.  Delay firing for 2 seconds.
setTimeout(raisePhotobucketNotifications, 2000); // usec
